@@ -1,11 +1,11 @@
-/* Please note!  This is extremely alpha code, and is really intended as
- * a "proof of concept" since I don't yet know whether it's going to 
- * to be practical and/or possible to implement a the complete backend.
- * It's also been tested with only cameras I have access to: the Kodak DC240
+/* Please note!  Although intended to support multiple camera types
+ * it's been tested with only cameras I have access to: the Kodak DC240
  * and the Directory Browse "camera."  I'm very interested 
  * in learning what it would take to support more cameras.  In 
  * particular, the current incarnation will only support cameras
  * that directly generate jpeg files.
+ * 
+ * Please report sucesses or failures using this backend!
  *  
  * However, having said that, I've already found it to be quite useful
  * even in its current form - one reason is that gphoto2 provides access
@@ -85,7 +85,7 @@
 #include <unistd.h>
 #include <fcntl.h>
 #include <limits.h>
-#include "cdjpeg.h"
+#include "sane/sanei_jpeg.h"
 #include <sys/ioctl.h>
 
 #include "sane/sane.h"
@@ -106,8 +106,7 @@
 #include <gphoto2-camera.h>
 #include <gphoto2-port-log.h>
 
-#define CHECK_EXIT(f) {int res = f; if (res < 0) {DBG (0,"ERROR: %s\n", gp_result_as_string (res)); return (SANE_STATUS_INVAL);}}
-#define CHECK_RET(f) {int res = f; if (res < 0) {DBG (0,"ERROR: %s\n", gp_result_as_string (res)); return (SANE_STATUS_INVAL);}}
+#define CHECK_RET(f) {int res = f; if (res < 0) {DBG (1,"ERROR: %s\n", gp_result_as_string (res)); return (SANE_STATUS_INVAL);}}
 
 #ifndef PATH_MAX
 # define PATH_MAX	1024
@@ -328,7 +327,7 @@ static SANE_Char cmdbuf[256];
 /* Structures used by gphoto2 API */
 static CameraAbilities abilities;
 static CameraFile *data_file;
-static const char *data_ptr;
+static const unsigned char *data_ptr;
 static unsigned long data_file_total_size, data_file_current_index;
 
 static SANE_Int hack_fd;
@@ -433,7 +432,7 @@ init_gphoto2 (void)
 	}
     }
 
-  if (abilities.speed[n] == 0 && !strncmp (Cam_data.port, "serial:", 7) )
+  if (abilities.speed[n] == 0 && !strncmp (Cam_data.port, "serial:", 7))
     {
       DBG (0,
 	   "%s: error: %d is not a valid speed for this camers.  Use \"gphoto2 --camera \"%s\" --abilities\" for list.\n",
@@ -664,7 +663,7 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback UNUSEDARG authorize)
   DBG_INIT ();
 
   DBG (1,
-       "GPHOTO2 Backend $Id: gphoto2.c,v 1.27 2003/02/12 02:54:47 psfales Exp $\n");
+       "GPHOTO2 Backend $Id: gphoto2.c,v 1.31 2005/05/05 03:52:03 psfales-guest Exp $\n");
 
   if (getenv ("GP_DEBUG"))
     {
@@ -684,9 +683,9 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback UNUSEDARG authorize)
        * file is unlikely.  So, give and return failure 
        */
       DBG (0, "warning: %s:  missing config file '%s'\n"
-	"If you aren't using gphoto2, you should disable it in dll.conf.\n"
-	"If you do want to use gphoto2, you'll need to install the config\n"
-	"file in %s.\n", f, GPHOTO2_CONFIG_FILE,GPHOTO2_CONFIG_FILE);
+	   "If you aren't using gphoto2, you should disable it in dll.conf.\n"
+	   "If you do want to use gphoto2, you'll need to install the config\n"
+	   "file in %s.\n", f, GPHOTO2_CONFIG_FILE, GPHOTO2_CONFIG_FILE);
 
       return SANE_STATUS_INVAL;
     }
@@ -834,13 +833,13 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback UNUSEDARG authorize)
       fclose (fp);
     }
 
-  dev[0].name = strdup (Cam_data.port);
-
   DBG (3, "sane_init: about to init_gphoto2\n");
 
   if (init_gphoto2 () != SANE_STATUS_GOOD)
     return SANE_STATUS_INVAL;
 
+  dev[0].name = strdup (Cam_data.port);
+
   DBG (3, "sane_init: about to get_info\n");
   if (get_info () != SANE_STATUS_GOOD)
     {
@@ -1437,9 +1436,10 @@ sane_start (SANE_Handle handle)
     }
 
   CHECK_RET (gp_file_get_data_and_size
-	     (data_file, &data_ptr, &data_file_total_size));
+	     (data_file, (const char **)&data_ptr, &data_file_total_size));
 
-  converter_init ();
+  if ( converter_init (handle) != SANE_STATUS_GOOD )
+    return SANE_STATUS_INVAL;
 
   /* Check if a linebuffer has been allocated.  If we had one 
    * previously, free it up and allocate one for (possibly) new
@@ -1562,7 +1562,7 @@ sane_set_io_mode (SANE_Handle UNUSEDARG handle, SANE_Bool
  * sane_get_select_fd() - From SANE API
  */
 SANE_Status
-sane_get_select_fd (SANE_Handle UNUSEDARG handle, SANE_Int * UNUSEDARG fd)
+sane_get_select_fd (SANE_Handle UNUSEDARG handle, SANE_Int UNUSEDARG *  fd)
 {
   return SANE_STATUS_UNSUPPORTED;
 }
@@ -1932,8 +1932,8 @@ converter_scan_complete (void)
  *	Currently assumes jpeg, but this is where we would put the 
  *	switch to handle other image types.
  */
-static void
-converter_init (void)
+static SANE_Status
+converter_init (SANE_Handle handle)
 {
   SANE_Int row_stride;
   struct jpeg_error_mgr jerr;
@@ -1941,6 +1941,13 @@ converter_init (void)
 
   data_file_current_index = 0;
 
+  /* Basic check to see if this is really a jpeg file */
+  if ( data_ptr[0] != 0xff || data_ptr[1] != 0xd8 ) {
+    sane_cancel(handle);
+exit(1);
+    return SANE_STATUS_INVAL;
+  }
+
   cinfo.err = jpeg_std_error (&jerr);
   jpeg_create_decompress (&cinfo);
 
@@ -1975,4 +1982,6 @@ converter_init (void)
 
   linebuffer_size = 0;
   linebuffer_index = 0;
+
+  return(SANE_STATUS_GOOD);
 }
