@@ -1,7 +1,7 @@
 /* sane - Scanner Access Now Easy.
 
    Copyright (C) 2002 Sergey Vlasov <vsu@altlinux.ru>
-   Copyright (C) 2002 - 2004 Henning Meier-Geinitz <henning@meier-geinitz.de>
+   Copyright (C) 2002 - 2005 Henning Meier-Geinitz <henning@meier-geinitz.de>
 
    This file is part of the SANE package.
    
@@ -48,7 +48,7 @@
 
 #include "../include/sane/config.h"
 
-#define BUILD 66
+#define BUILD 71
 #define MAX_DEBUG
 #define WARMUP_TIME 60
 #define CALIBRATION_HEIGHT 2.5
@@ -91,6 +91,7 @@
 #include <sys/time.h>
 #include <time.h>
 #include <math.h>
+#include <dirent.h>
 
 #include "../include/sane/sane.h"
 #include "../include/sane/sanei.h"
@@ -155,11 +156,6 @@ static SANE_Range y_range = {
   SANE_FIX (0.0)		/* quantization */
 };
 
-static const SANE_Range exposure_range = {
-  0,				/* minimum */
-  511,				/* maximum */
-  1				/* quantization */
-};
 
 static const SANE_Range offset_range = {
   -63,				/* minimum */
@@ -207,11 +203,11 @@ max_string_size (const SANE_String_Const strings[])
 }
 
 static SANE_Status
-get_afe_values (SANE_String_Const cp, GT68xx_AFE_Parameters *afe)
+get_afe_values (SANE_String_Const cp, GT68xx_AFE_Parameters * afe)
 {
   SANE_Char *word, *end;
   int i;
-  
+
   for (i = 0; i < 6; i++)
     {
       cp = sanei_config_get_string (cp, &word);
@@ -223,7 +219,7 @@ get_afe_values (SANE_String_Const cp, GT68xx_AFE_Parameters *afe)
 
 	  if (end == word)
 	    {
-	      DBG (5, "get_afe_values: can't parse %d. parameter `%s'\n", 
+	      DBG (5, "get_afe_values: can't parse %d. parameter `%s'\n",
 		   i + 1, word);
 	      free (word);
 	      word = 0;
@@ -239,7 +235,7 @@ get_afe_values (SANE_String_Const cp, GT68xx_AFE_Parameters *afe)
 	    }
 	  else if (long_value < 0)
 	    {
-	      DBG (5, "get_afe_values: %d. parameter < 0 (%d)\n", i + 1, 
+	      DBG (5, "get_afe_values: %d. parameter < 0 (%d)\n", i + 1,
 		   (int) long_value);
 	      free (word);
 	      word = 0;
@@ -259,12 +255,24 @@ get_afe_values (SANE_String_Const cp, GT68xx_AFE_Parameters *afe)
 		   (int) long_value);
 	      switch (i)
 		{
-		case 0: afe->r_offset = (SANE_Byte) long_value; break;
-		case 1: afe->r_pga = (SANE_Byte) long_value; break;
-		case 2: afe->g_offset = (SANE_Byte) long_value; break;
-		case 3: afe->g_pga = (SANE_Byte) long_value; break;
-		case 4: afe->b_offset = (SANE_Byte) long_value; break;
-		case 5: afe->b_pga = (SANE_Byte) long_value; break;
+		case 0:
+		  afe->r_offset = (SANE_Byte) long_value;
+		  break;
+		case 1:
+		  afe->r_pga = (SANE_Byte) long_value;
+		  break;
+		case 2:
+		  afe->g_offset = (SANE_Byte) long_value;
+		  break;
+		case 3:
+		  afe->g_pga = (SANE_Byte) long_value;
+		  break;
+		case 4:
+		  afe->b_offset = (SANE_Byte) long_value;
+		  break;
+		case 5:
+		  afe->b_pga = (SANE_Byte) long_value;
+		  break;
 		}
 	      free (word);
 	      word = 0;
@@ -305,7 +313,7 @@ setup_scan_request (GT68xx_Scanner * s, GT68xx_Scan_Request * scan_request)
     scan_request->xdpi = s->dev->model->optical_xdpi;
   scan_request->ydpi = s->val[OPT_RESOLUTION].w;
 
-  if (IS_ACTIVE (OPT_BIT_DEPTH) && (!s->val[OPT_PREVIEW].w || !s->val[OPT_FAST_PREVIEW].w))
+  if (IS_ACTIVE (OPT_BIT_DEPTH) && !s->val[OPT_PREVIEW].w)
     scan_request->depth = s->val[OPT_BIT_DEPTH].w;
   else
     scan_request->depth = 8;
@@ -353,7 +361,8 @@ calc_parameters (GT68xx_Scanner * s)
   val = s->val[OPT_MODE].s;
 
   s->params.last_frame = SANE_TRUE;
-  if (strcmp (val, SANE_VALUE_SCAN_MODE_GRAY) == 0 || strcmp (val, SANE_VALUE_SCAN_MODE_LINEART) == 0)
+  if (strcmp (val, SANE_VALUE_SCAN_MODE_GRAY) == 0
+      || strcmp (val, SANE_VALUE_SCAN_MODE_LINEART) == 0)
     s->params.format = SANE_FRAME_GRAY;
   else				/* Color */
     s->params.format = SANE_FRAME_RGB;
@@ -459,8 +468,9 @@ init_options (GT68xx_Scanner * s)
   /* scan mode */
   s->opt[OPT_GRAY_MODE_COLOR].name = "gray-mode-color";
   s->opt[OPT_GRAY_MODE_COLOR].title = SANE_I18N ("Gray mode color");
-  s->opt[OPT_GRAY_MODE_COLOR].desc = SANE_I18N ("Selects which scan color is used "
-						"gray mode (default: green).");
+  s->opt[OPT_GRAY_MODE_COLOR].desc =
+    SANE_I18N ("Selects which scan color is used "
+	       "gray mode (default: green).");
   s->opt[OPT_GRAY_MODE_COLOR].type = SANE_TYPE_STRING;
   s->opt[OPT_GRAY_MODE_COLOR].constraint_type = SANE_CONSTRAINT_STRING_LIST;
   s->opt[OPT_GRAY_MODE_COLOR].size = max_string_size (gray_mode_list);
@@ -552,20 +562,21 @@ init_options (GT68xx_Scanner * s)
   /* auto warmup */
   s->opt[OPT_AUTO_WARMUP].name = "auto-warmup";
   s->opt[OPT_AUTO_WARMUP].title = SANE_I18N ("Automatic warmup");
-  s->opt[OPT_AUTO_WARMUP].desc = 
+  s->opt[OPT_AUTO_WARMUP].desc =
     SANE_I18N ("Warm-up until the lamp's brightness is constant "
 	       "instead of insisting on 60 seconds warm-up time.");
   s->opt[OPT_AUTO_WARMUP].type = SANE_TYPE_BOOL;
   s->opt[OPT_AUTO_WARMUP].unit = SANE_UNIT_NONE;
   s->opt[OPT_AUTO_WARMUP].constraint_type = SANE_CONSTRAINT_NONE;
   s->val[OPT_AUTO_WARMUP].w = SANE_TRUE;
-  if ((s->dev->model->is_cis && !(s->dev->model->flags & GT68XX_FLAG_CIS_LAMP)) || !debug_options)
+  if ((s->dev->model->is_cis
+       && !(s->dev->model->flags & GT68XX_FLAG_CIS_LAMP)) || !debug_options)
     DISABLE (OPT_AUTO_WARMUP);
 
   /* full scan */
   s->opt[OPT_FULL_SCAN].name = "full-scan";
   s->opt[OPT_FULL_SCAN].title = SANE_I18N ("Full scan");
-  s->opt[OPT_FULL_SCAN].desc = 
+  s->opt[OPT_FULL_SCAN].desc =
     SANE_I18N ("Scan the complete scanning area including calibration strip. "
 	       "Be careful. Don't select the full height. For testing only.");
   s->opt[OPT_FULL_SCAN].type = SANE_TYPE_BOOL;
@@ -592,7 +603,7 @@ init_options (GT68xx_Scanner * s)
 
   /* coarse calibration only once */
   s->opt[OPT_COARSE_CAL_ONCE].name = "coarse-calibration-once";
-  s->opt[OPT_COARSE_CAL_ONCE].title = 
+  s->opt[OPT_COARSE_CAL_ONCE].title =
     SANE_I18N ("Coarse calibration for first scan only");
   s->opt[OPT_COARSE_CAL_ONCE].desc =
     SANE_I18N ("Coarse calibration is only done for the first scan. Works "
@@ -617,22 +628,10 @@ init_options (GT68xx_Scanner * s)
   if (!debug_options)
     DISABLE (OPT_QUALITY_CAL);
 
-  /* fast preview */
-  s->opt[OPT_FAST_PREVIEW].name = "fast-preview";
-  s->opt[OPT_FAST_PREVIEW].title = SANE_I18N ("Fast preview");
-  s->opt[OPT_FAST_PREVIEW].desc =
-    SANE_I18N ("Request that all previews are done in the fastest "
-	       "(low-quality) mode. This may be a non-color mode or a low "
-	       "resolution mode.");
-  s->opt[OPT_FAST_PREVIEW].type = SANE_TYPE_BOOL;
-  s->val[OPT_FAST_PREVIEW].w = SANE_TRUE;
-  if (!debug_options)
-    DISABLE (OPT_FAST_PREVIEW);
-
   /* backtrack lines */
   s->opt[OPT_BACKTRACK_LINES].name = "backtrack-lines";
-  s->opt[OPT_BACKTRACK_LINES].title = SANE_I18N("Backtrack lines");
-  s->opt[OPT_BACKTRACK_LINES].desc = 
+  s->opt[OPT_BACKTRACK_LINES].title = SANE_I18N ("Backtrack lines");
+  s->opt[OPT_BACKTRACK_LINES].desc =
     SANE_I18N ("Number of lines the scan slider moves back when backtracking "
 	       "occurs. That happens when the scanner scans faster than the "
 	       "computer can receive the data. Low values cause faster scans "
@@ -659,14 +658,15 @@ init_options (GT68xx_Scanner * s)
   /* internal gamma value */
   s->opt[OPT_GAMMA_VALUE].name = "gamma-value";
   s->opt[OPT_GAMMA_VALUE].title = SANE_I18N ("Gamma value");
-  s->opt[OPT_GAMMA_VALUE].desc = SANE_I18N ("Sets the gamma value of all channels.");
+  s->opt[OPT_GAMMA_VALUE].desc =
+    SANE_I18N ("Sets the gamma value of all channels.");
   s->opt[OPT_GAMMA_VALUE].type = SANE_TYPE_FIXED;
   s->opt[OPT_GAMMA_VALUE].unit = SANE_UNIT_NONE;
   s->opt[OPT_GAMMA_VALUE].constraint_type = SANE_CONSTRAINT_RANGE;
   s->opt[OPT_GAMMA_VALUE].constraint.range = &gamma_range;
   s->opt[OPT_GAMMA_VALUE].cap |= SANE_CAP_EMULATED;
   s->val[OPT_GAMMA_VALUE].w = s->dev->gamma_value;
-  
+
   /* threshold */
   s->opt[OPT_THRESHOLD].name = SANE_NAME_THRESHOLD;
   s->opt[OPT_THRESHOLD].title = SANE_TITLE_THRESHOLD;
@@ -778,14 +778,17 @@ attach (SANE_String_Const devname, GT68xx_Device ** devp, SANE_Bool may_wait)
 
   if (!gt68xx_device_is_configured (dev))
     {
-      GT68xx_Model * model;
-      DBG (2, "attach: Warning: device `%s' is not listed in device table\n", devname);
-      DBG (2, "attach: If you have manually added it, use override in gt68xx.conf\n");
+      GT68xx_Model *model;
+      DBG (2, "attach: Warning: device `%s' is not listed in device table\n",
+	   devname);
+      DBG (2,
+	   "attach: If you have manually added it, use override in gt68xx.conf\n");
       gt68xx_device_get_model ("unknown-scanner", &model);
       status = gt68xx_device_set_model (dev, model);
       if (status != SANE_STATUS_GOOD)
 	{
-	  DBG (4, "attach: couldn't set model: %s\n", sane_strstatus (status));
+	  DBG (4, "attach: couldn't set model: %s\n",
+	       sane_strstatus (status));
 	  gt68xx_device_free (dev);
 	  if (devp)
 	    *devp = 0;
@@ -853,46 +856,93 @@ download_firmware_file (GT68xx_Device * dev)
   SANE_Status status = SANE_STATUS_GOOD;
   SANE_Byte *buf = NULL;
   int size = -1;
-  SANE_Char filename[PATH_MAX];
+  SANE_Char filename[PATH_MAX], dirname[PATH_MAX], basename[PATH_MAX];
   FILE *f;
 
   if (strncmp (dev->model->firmware_name, PATH_SEP, 1) != 0)
     {
+      /* probably filename only */
       snprintf (filename, PATH_MAX, "%s%s%s%s%s%s%s",
 		STRINGIFY (PATH_SANE_DATA_DIR),
 		PATH_SEP, "sane", PATH_SEP, "gt68xx", PATH_SEP,
 		dev->model->firmware_name);
+      snprintf (dirname, PATH_MAX, "%s%s%s%s%s",
+		STRINGIFY (PATH_SANE_DATA_DIR),
+		PATH_SEP, "sane", PATH_SEP, "gt68xx");
+      strncpy (basename, dev->model->firmware_name, PATH_MAX);
+    }
+  else
+    {
+      /* absolute path */
+      char *pos;
+      strncpy (filename, dev->model->firmware_name, PATH_MAX);
+      strncpy (dirname, dev->model->firmware_name, PATH_MAX);
+      pos = strrchr (dirname, PATH_SEP[0]);
+      if (pos)
+	pos[0] = '\0';
+      strncpy (basename, pos + 1, PATH_MAX);
     }
-  else				/* absolute path */
-    strncpy (filename, dev->model->firmware_name, PATH_MAX);
 
-
-  /* Check both mixed and lower case */
+  /* first, try to open with exact case */
   DBG (5, "download_firmware: trying %s\n", filename);
   f = fopen (filename, "rb");
   if (!f)
     {
-      SANE_Char filename_lower[PATH_MAX];
-      unsigned int i;
+      /* and now any case */
+      DIR *dir;
+      struct dirent *direntry;
 
       DBG (5,
 	   "download_firmware_file: Couldn't open firmware file `%s': %s\n",
 	   filename, strerror (errno));
 
-      for (i = 0; i <= strlen (filename); i++)
-	filename_lower[i] = tolower (filename[i]);
-
-      DBG (5, "download_firmware: trying %s\n", filename_lower);
-      f = fopen (filename_lower, "rb");
-      if (!f)
+      dir = opendir (dirname);
+      if (!dir)
 	{
-	  DBG (5,
-	       "download_firmware_file: Couldn't open firmware file `%s': %s\n",
-	       filename, strerror (errno));
-	  DBG (0, "Couldn't open firmware file (neither `%s' nor `%s'): %s\n",
-	       filename, filename_lower, strerror (errno));
+	  DBG (5, "download_firmware: couldn't open directory `%s': %s\n",
+	       dirname, strerror (errno));
 	  status = SANE_STATUS_INVAL;
 	}
+      if (status == SANE_STATUS_GOOD)
+	{
+	  do
+	    {
+	      direntry = readdir (dir);
+	      if (direntry
+		  && (strncasecmp (direntry->d_name, basename, PATH_MAX) ==
+		      0))
+		{
+		  snprintf (filename, PATH_MAX, "%s%s%s",
+			    dirname, PATH_SEP, direntry->d_name);
+		  break;
+		}
+	    }
+	  while (direntry != 0);
+	  if (direntry == 0)
+	    {
+	      DBG (5, "download_firmware: file `%s' not found\n", filename);
+	      status = SANE_STATUS_INVAL;
+	    }
+	  closedir (dir);
+	}
+      if (status == SANE_STATUS_GOOD)
+	{
+	  DBG (5, "download_firmware: trying %s\n", filename);
+	  f = fopen (filename, "rb");
+	  if (!f)
+	    {
+	      DBG (5,
+		   "download_firmware_file: Couldn't open firmware file `%s': %s\n",
+		   filename, strerror (errno));
+	      status = SANE_STATUS_INVAL;
+	    }
+	}
+
+      if (status != SANE_STATUS_GOOD)
+	{
+	  DBG (0, "Couldn't open firmware file (`%s'): %s\n",
+	       filename, strerror (errno));
+	}
     }
 
   if (status == SANE_STATUS_GOOD)
@@ -987,7 +1037,7 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
       debug_options = SANE_TRUE;
     }
 #endif
-    
+
   fp = sanei_config_open (GT68XX_CONFIG_FILE);
   if (!fp)
     {
@@ -1142,7 +1192,7 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
 
 	  free (word);
 	  word = 0;
-	  
+
 	  status = get_afe_values (cp, &afe);
 	  if (status == SANE_STATUS_GOOD)
 	    {
@@ -1154,7 +1204,8 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
 		       new_dev[i]->model->name);
 		}
 	      if (i == 0)
-		DBG (5, "sane_init: can't set afe values, set device first\n");
+		DBG (5,
+		     "sane_init: can't set afe values, set device first\n");
 	    }
 	  else
 	    DBG (3, "sane_init: can't set afe values\n");
@@ -1297,19 +1348,23 @@ sane_open (SANE_String_Const devicename, SANE_Handle * handle)
   if (dev->manual_selection)
     {
       DBG (0, "WARNING: You have manually added the ids of your scanner \n");
-      DBG (0, "         to gt68xx.conf. Please use an appropriate override \n");
-      DBG (0, "         for your scanner. Use extreme care and switch off \n");
-      DBG (0, "         the scanner immediately if you hear unusual noise. \n");
+      DBG (0,
+	   "         to gt68xx.conf. Please use an appropriate override \n");
+      DBG (0,
+	   "         for your scanner. Use extreme care and switch off \n");
+      DBG (0,
+	   "         the scanner immediately if you hear unusual noise. \n");
       DBG (0, "         Please report any success to \n");
       DBG (0, "         henning@meier-geinitz.de. Please provide as many\n");
       DBG (0, "         details as possible, e.g. the exact name of your\n");
       DBG (0, "         scanner, ids, settings etc.\n");
-      
+
       if (strcmp (dev->model->name, "unknown-scanner") == 0)
 	{
 	  GT68xx_USB_Device_Entry *entry;
 
-	  DBG (0, "ERROR: You haven't chosen an override in gt68xx.conf. Please use \n");
+	  DBG (0,
+	       "ERROR: You haven't chosen an override in gt68xx.conf. Please use \n");
 	  DBG (0, "       one of the following: \n");
 
 	  for (entry = gt68xx_usb_device_list; entry->model; ++entry)
@@ -1330,19 +1385,19 @@ sane_open (SANE_String_Const devicename, SANE_Handle * handle)
     DBG (3, "sane_open: firmware already loaded, skipping load\n");
   else
     RIE (download_firmware_file (dev));
-  /*  RIE (gt68xx_device_check_firmware (dev, &firmware_loaded));*/
+  /*  RIE (gt68xx_device_check_firmware (dev, &firmware_loaded)); */
   if (!firmware_loaded)
     {
       DBG (1, "sane_open: firmware still not loaded? Proceeding anyway\n");
       /* return SANE_STATUS_IO_ERROR; */
     }
 #else
-    RIE (download_firmware_file (dev));
+  RIE (download_firmware_file (dev));
 #endif
 
   RIE (gt68xx_device_get_id (dev));
 
-  if (!(dev->model->flags | GT68XX_FLAG_NO_STOP))
+  if (!(dev->model->flags & GT68XX_FLAG_NO_STOP))
     RIE (gt68xx_device_stop_scan (dev));
 
   RIE (gt68xx_device_get_power_status (dev, &power_ok));
@@ -1477,7 +1532,6 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 	case OPT_COARSE_CAL:
 	case OPT_COARSE_CAL_ONCE:
 	case OPT_QUALITY_CAL:
-	case OPT_FAST_PREVIEW:
 	case OPT_BACKTRACK:
 	case OPT_BACKTRACK_LINES:
 	case OPT_PREVIEW:
@@ -1524,7 +1578,6 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 	case OPT_RESOLUTION:
 	case OPT_BIT_DEPTH:
 	case OPT_FULL_SCAN:
-	case OPT_FAST_PREVIEW:
 	case OPT_PREVIEW:
 	case OPT_TL_X:
 	case OPT_TL_Y:
@@ -1705,7 +1758,7 @@ sane_start (SANE_Handle handle)
     s->dev->gray_mode_color = 0x02;
   else
     s->dev->gray_mode_color = 0x03;
-    
+
   setup_scan_request (s, &scan_request);
   if (!s->first_scan && s->val[OPT_COARSE_CAL_ONCE].w == SANE_TRUE)
     s->auto_afe = SANE_FALSE;
@@ -1723,20 +1776,20 @@ sane_start (SANE_Handle handle)
     }
   for (i = 0; i < gamma_size; i++)
     {
-      s->gamma_table [i] = 
+      s->gamma_table[i] =
 	(gamma_size - 1) * pow (((double) i + 1) / (gamma_size),
-			  1.0 / SANE_UNFIX(s->dev->gamma_value)) + 0.5;
-      if (s->gamma_table [i] > (gamma_size - 1))
-	s->gamma_table [i] = (gamma_size - 1);
-      if (s->gamma_table [i] < 0) 
-	s->gamma_table [i] = 0;
+				1.0 / SANE_UNFIX (s->dev->gamma_value)) + 0.5;
+      if (s->gamma_table[i] > (gamma_size - 1))
+	s->gamma_table[i] = (gamma_size - 1);
+      if (s->gamma_table[i] < 0)
+	s->gamma_table[i] = 0;
 #if 0
-      printf ("%d %d\n", i, s->gamma_table [i]);
+      printf ("%d %d\n", i, s->gamma_table[i]);
 #endif
     }
 
   s->calib = s->val[OPT_QUALITY_CAL].w;
-  if (!(s->dev->model->flags | GT68XX_FLAG_NO_STOP))
+  if (!(s->dev->model->flags & GT68XX_FLAG_NO_STOP))
     RIE (gt68xx_device_stop_scan (s->dev));
 
   RIE (gt68xx_device_carriage_home (s->dev));
@@ -1753,7 +1806,7 @@ sane_start (SANE_Handle handle)
       else
 	scan_request.backtrack = SANE_TRUE;
     }
-    
+
   if (scan_request.backtrack)
     scan_request.backtrack_lines = s->val[OPT_BACKTRACK_LINES].w;
   else
@@ -1870,9 +1923,10 @@ sane_read (SANE_Handle handle, SANE_Byte * buf, SANE_Int max_len,
 	    for (i = 0; i < s->reader->pixels_per_line; i++)
 	      {
 		if (s->reader->params.depth > 8)
-		  buffer_pointers[color][i] = s->gamma_table[buffer_pointers[color][i]];
+		  buffer_pointers[color][i] =
+		    s->gamma_table[buffer_pointers[color][i]];
 		else
-		  buffer_pointers[color][i] = 
+		  buffer_pointers[color][i] =
 		    (s->gamma_table[buffer_pointers[color][i] >> 8] << 8) +
 		    (s->gamma_table[buffer_pointers[color][i] >> 8]);
 	      }
@@ -1945,8 +1999,10 @@ sane_read (SANE_Handle handle, SANE_Byte * buf, SANE_Int max_len,
 		s->byte_count++;
 #ifdef DEBUG_BRIGHTNESS
 	      s->average_white += buf[*len];
-	      s->max_white = (buf[*len] > s->max_white) ? buf[*len] : s->max_white;
-	      s->min_black = (buf[*len] < s->min_black) ? buf[*len] : s->min_black;
+	      s->max_white =
+		(buf[*len] > s->max_white) ? buf[*len] : s->max_white;
+	      s->min_black =
+		(buf[*len] < s->min_black) ? buf[*len] : s->min_black;
 #endif
 	    }
 	}
@@ -2018,7 +2074,8 @@ sane_cancel (SANE_Handle handle)
 	       "sane_cancel: scan finished, scanned %d bytes in %d seconds\n",
 	       s->total_bytes, secs);
 #ifdef DEBUG_BRIGHTNESS
-	  DBG (1, "sane_cancel: average white: %d, max_white=%d, min_black=%d\n",
+	  DBG (1,
+	       "sane_cancel: average white: %d, max_white=%d, min_black=%d\n",
 	       s->average_white / s->total_bytes, s->max_white, s->min_black);
 #endif
 
