@@ -1,7 +1,7 @@
 /* sane - Scanner Access Now Easy.
 
    Copyright (C) 2002 Sergey Vlasov <vsu@altlinux.ru>
-   Copyright (C) 2002, 2003 Henning Meier-Geinitz <henning@meier-geinitz.de>
+   Copyright (C) 2002 - 2004 Henning Meier-Geinitz <henning@meier-geinitz.de>
    
    This file is part of the SANE package.
    
@@ -156,8 +156,8 @@ gt68xx_device_new (GT68xx_Device ** dev_return)
 
   if (!dev)
     {
-      DBG (3, "gt68xx_device_new: couldn't malloc %d bytes for device\n",
-	   sizeof (GT68xx_Device));
+      DBG (3, "gt68xx_device_new: couldn't malloc %lu bytes for device\n",
+	   (u_long) sizeof (GT68xx_Device));
       *dev_return = 0;
       return SANE_STATUS_NO_MEM;
     }
@@ -174,6 +174,8 @@ gt68xx_device_new (GT68xx_Device ** dev_return)
   dev->read_buffer = NULL;
   dev->read_buffer_size = 32768;
 
+  dev->manual_selection = SANE_FALSE;
+
 #ifdef USE_FORK
   dev->shm_channel = NULL;
 #endif /* USE_FORK */
@@ -348,45 +350,12 @@ gt68xx_device_get_model (SANE_String name, GT68xx_Model ** model)
   return SANE_FALSE;
 }
 
-/* currently not used */
-#if 0
-SANE_Status
-gt68xx_device_unshare_model (GT68xx_Device * dev)
-{
-  CHECK_DEV_OPEN (dev, "gt68xx_device_unshare_model");
-
-  if (dev->active)
-    {
-      DBG ((3, "gt68xx_device_unshare_model: device already active\n");
-      return SANE_STATUS_INVAL;
-    }
-
-  if (!dev->model)
-    return SANE_STATUS_INVAL;
-
-  if (!dev->model->allocated)
-    {
-      GT68xx_Model *new_model;
-
-      new_model = (GT68xx_Model *) malloc (sizeof (GT68xx_Model));
-      if (!new_model)
-	return SANE_STATUS_NO_MEM;
-      memcpy (new_model, dev->model, sizeof (GT68xx_Model));
-      new_model->allocated = SANE_TRUE;
-      dev->model = new_model;
-    }
-
-  return SANE_STATUS_GOOD;
-}
-#endif
 
 SANE_Status
 gt68xx_device_activate (GT68xx_Device * dev)
 {
   SANE_Status status;
-
   CHECK_DEV_OPEN (dev, "gt68xx_device_activate");
-
   if (dev->active)
     {
       DBG (3, "gt68xx_device_activate: device already active\n");
@@ -400,7 +369,6 @@ gt68xx_device_activate (GT68xx_Device * dev)
     }
 
   DBG (7, "gt68xx_device_activate: model \"%s\"\n", dev->model->name);
-
   if (dev->model->command_set->activate)
     {
       status = (*dev->model->command_set->activate) (dev);
@@ -415,13 +383,10 @@ gt68xx_device_activate (GT68xx_Device * dev)
   dev->exposure = malloc (sizeof (*dev->exposure));
   if (!dev->afe || !dev->exposure)
     return SANE_STATUS_NO_MEM;
-
   memcpy (dev->afe, &dev->model->afe_params, sizeof (*dev->afe));
   memcpy (dev->exposure, &dev->model->exposure, sizeof (*dev->exposure));
   dev->gamma_value = dev->model->default_gamma_value;
-
   dev->active = SANE_TRUE;
-
   return SANE_STATUS_GOOD;
 }
 
@@ -429,18 +394,16 @@ SANE_Status
 gt68xx_device_deactivate (GT68xx_Device * dev)
 {
   SANE_Status status = SANE_STATUS_GOOD;
-
   CHECK_DEV_ACTIVE (dev, "gt68xx_device_deactivate");
-
   if (dev->read_active)
     gt68xx_device_read_finish (dev);
-
   if (dev->model->command_set->deactivate)
     {
       status = (*dev->model->command_set->deactivate) (dev);
       if (status != SANE_STATUS_GOOD)
 	{
-	  DBG (3, "gt68xx_device_deactivate: command set-specific deactivate failed: %s\n",
+	  DBG (3,
+	       "gt68xx_device_deactivate: command set-specific deactivate failed: %s\n",
 	       sane_strstatus (status));
 	  /* proceed with deactivate anyway */
 	}
@@ -451,9 +414,7 @@ gt68xx_device_deactivate (GT68xx_Device * dev)
   if (dev->exposure)
     free (dev->exposure);
   dev->exposure = 0;
-
   dev->active = SANE_FALSE;
-
   return status;
 }
 
@@ -462,22 +423,21 @@ gt68xx_device_memory_write (GT68xx_Device * dev,
 			    SANE_Word addr, SANE_Word size, SANE_Byte * data)
 {
   SANE_Status status;
-
-  DBG (8, "gt68xx_device_memory_write: dev=%p, addr=0x%x, size=0x%x, data=%p\n",
+  DBG (8,
+       "gt68xx_device_memory_write: dev=%p, addr=0x%x, size=0x%x, data=%p\n",
        (void *) dev, addr, size, data);
   CHECK_DEV_ACTIVE (dev, "gt68xx_device_memory_write");
-
-  status = sanei_usb_control_msg (dev->fd, 0x40,
-				  dev->model->command_set->request,
-				  dev->model->command_set->memory_write_value,
-				  addr, size, data);
-
+  status =
+    sanei_usb_control_msg (dev->fd, 0x40,
+			   dev->model->command_set->request,
+			   dev->model->command_set->memory_write_value,
+			   addr, size, data);
   if (status != SANE_STATUS_GOOD)
     {
-      DBG (3, "gt68xx_device_memory_write: sanei_usb_control_msg failed: %s\n",
+      DBG (3,
+	   "gt68xx_device_memory_write: sanei_usb_control_msg failed: %s\n",
 	   sane_strstatus (status));
     }
-
   return status;
 }
 
@@ -486,19 +446,19 @@ gt68xx_device_memory_read (GT68xx_Device * dev,
 			   SANE_Word addr, SANE_Word size, SANE_Byte * data)
 {
   SANE_Status status;
-
-  DBG (8, "gt68xx_device_memory_read: dev=%p, addr=0x%x, size=0x%x, data=%p\n",
+  DBG (8,
+       "gt68xx_device_memory_read: dev=%p, addr=0x%x, size=0x%x, data=%p\n",
        (void *) dev, addr, size, data);
   CHECK_DEV_ACTIVE (dev, "gt68xx_device_memory_read");
-
-  status = sanei_usb_control_msg (dev->fd, 0xc0,
-				  dev->model->command_set->request,
-				  dev->model->command_set->memory_read_value,
-				  addr, size, data);
-
+  status =
+    sanei_usb_control_msg (dev->fd, 0xc0,
+			   dev->model->command_set->request,
+			   dev->model->command_set->memory_read_value,
+			   addr, size, data);
   if (status != SANE_STATUS_GOOD)
     {
-      DBG (3, "gt68xx_device_memory_read: sanei_usb_control_msg failed: %s\n",
+      DBG (3,
+	   "gt68xx_device_memory_read: sanei_usb_control_msg failed: %s\n",
 	   sane_strstatus (status));
     }
 
@@ -510,33 +470,31 @@ gt68xx_device_generic_req (GT68xx_Device * dev,
 			   SANE_Byte request_type, SANE_Word request,
 			   SANE_Word cmd_value, SANE_Word cmd_index,
 			   SANE_Word res_value, SANE_Word res_index,
-			   GT68xx_Packet cmd, GT68xx_Packet res, size_t res_size)
+			   GT68xx_Packet cmd, GT68xx_Packet res,
+			   size_t res_size)
 {
   SANE_Status status;
-
   DBG (7, "gt68xx_device_generic_req: command=0x%02x\n", cmd[0]);
   DUMP_REQ (">>", cmd);
   CHECK_DEV_ACTIVE (dev, "gt68xx_device_generic_req");
-
   status = sanei_usb_control_msg (dev->fd,
-				  request_type, request, cmd_value, cmd_index,
-				  GT68XX_PACKET_SIZE, cmd);
+				  request_type, request, cmd_value,
+				  cmd_index, GT68XX_PACKET_SIZE, cmd);
   if (status != SANE_STATUS_GOOD)
     {
       DBG (3, "gt68xx_device_generic_req: writing command failed: %s\n",
-	     sane_strstatus (status));
+	   sane_strstatus (status));
       return status;
     }
 
   memset (res, 0, sizeof (GT68xx_Packet));
-
   status = sanei_usb_control_msg (dev->fd,
-				  request_type | 0x80, request, res_value, res_index,
-				  res_size, res);
+				  request_type | 0x80, request,
+				  res_value, res_index, res_size, res);
   if (status != SANE_STATUS_GOOD)
     {
       DBG (3, "gt68xx_device_generic_req: reading response failed: %s\n",
-	     sane_strstatus (status));
+	   sane_strstatus (status));
       return status;
     }
 
@@ -554,8 +512,8 @@ gt68xx_device_req (GT68xx_Device * dev, GT68xx_Packet cmd, GT68xx_Packet res)
 				    command_set->send_cmd_value,
 				    command_set->send_cmd_index,
 				    command_set->recv_res_value,
-				    command_set->recv_res_index, cmd, res,
-				    GT68XX_PACKET_SIZE);
+				    command_set->recv_res_index, cmd,
+				    res, GT68XX_PACKET_SIZE);
 }
 
 SANE_Status
@@ -565,10 +523,8 @@ gt68xx_device_small_req (GT68xx_Device * dev, GT68xx_Packet cmd,
   GT68xx_Command_Set *command_set = dev->model->command_set;
   GT68xx_Packet fixed_cmd;
   int i;
-
   for (i = 0; i < 8; ++i)
     memcpy (fixed_cmd + i * 8, cmd, 8);
-
   return gt68xx_device_generic_req (dev,
 				    command_set->request_type,
 				    command_set->request,
@@ -580,26 +536,11 @@ gt68xx_device_small_req (GT68xx_Device * dev, GT68xx_Packet cmd,
 }
 
 
-#if 0
-/* currently not used */
-SANE_Status
-gt68xx_device_check_firmware (GT68xx_Device * dev, SANE_Bool * loaded)
-{
-  CHECK_DEV_ACTIVE (dev, "gt68xx_device_check_firmware");
-
-  if (dev->model->command_set->check_firmware)
-    return (*dev->model->command_set->check_firmware) (dev, loaded);
-  else
-    return SANE_STATUS_UNSUPPORTED;
-}
-#endif
-
 SANE_Status
 gt68xx_device_download_firmware (GT68xx_Device * dev,
 				 SANE_Byte * data, SANE_Word size)
 {
   CHECK_DEV_ACTIVE (dev, "gt68xx_device_download_firmware");
-
   if (dev->model->command_set->download_firmware)
     return (*dev->model->command_set->download_firmware) (dev, data, size);
   else
@@ -610,7 +551,6 @@ SANE_Status
 gt68xx_device_get_power_status (GT68xx_Device * dev, SANE_Bool * power_ok)
 {
   CHECK_DEV_ACTIVE (dev, "gt68xx_device_get_power_status");
-
   if (dev->model->command_set->get_power_status)
     return (*dev->model->command_set->get_power_status) (dev, power_ok);
   else
@@ -621,7 +561,6 @@ SANE_Status
 gt68xx_device_get_ta_status (GT68xx_Device * dev, SANE_Bool * ta_attached)
 {
   CHECK_DEV_ACTIVE (dev, "gt68xx_device_get_ta_status");
-
   if (dev->model->command_set->get_ta_status)
     return (*dev->model->command_set->get_ta_status) (dev, ta_attached);
   else
@@ -633,7 +572,6 @@ gt68xx_device_lamp_control (GT68xx_Device * dev, SANE_Bool fb_lamp,
 			    SANE_Bool ta_lamp)
 {
   CHECK_DEV_ACTIVE (dev, "gt68xx_device_lamp_control");
-
   if (dev->model->command_set->lamp_control)
     return (*dev->model->command_set->lamp_control) (dev, fb_lamp, ta_lamp);
   else
@@ -644,7 +582,6 @@ SANE_Status
 gt68xx_device_is_moving (GT68xx_Device * dev, SANE_Bool * moving)
 {
   CHECK_DEV_ACTIVE (dev, "gt68xx_device_is_moving");
-
   if (dev->model->command_set->is_moving)
     return (*dev->model->command_set->is_moving) (dev, moving);
   else
@@ -657,7 +594,6 @@ static SANE_Status
 gt68xx_device_move_relative (GT68xx_Device * dev, SANE_Int distance)
 {
   CHECK_DEV_ACTIVE (dev, "gt68xx_device_move_relative");
-
   if (dev->model->command_set->move_relative)
     return (*dev->model->command_set->move_relative) (dev, distance);
   else
@@ -669,7 +605,6 @@ SANE_Status
 gt68xx_device_carriage_home (GT68xx_Device * dev)
 {
   CHECK_DEV_ACTIVE (dev, "gt68xx_device_carriage_home");
-
   if (dev->model->command_set->carriage_home)
     return (*dev->model->command_set->carriage_home) (dev);
   else
@@ -680,7 +615,6 @@ SANE_Status
 gt68xx_device_start_scan (GT68xx_Device * dev)
 {
   CHECK_DEV_ACTIVE (dev, "gt68xx_device_start_scan");
-
   if (dev->model->command_set->start_scan)
     return (*dev->model->command_set->start_scan) (dev);
   else
@@ -691,7 +625,6 @@ SANE_Status
 gt68xx_device_read_scanned_data (GT68xx_Device * dev, SANE_Bool * ready)
 {
   CHECK_DEV_ACTIVE (dev, "gt68xx_device_read_scanned_data");
-
   if (dev->model->command_set->read_scanned_data)
     return (*dev->model->command_set->read_scanned_data) (dev, ready);
   else
@@ -705,7 +638,6 @@ gt68xx_device_setup_scan (GT68xx_Device * dev,
 			  GT68xx_Scan_Parameters * params)
 {
   CHECK_DEV_ACTIVE (dev, "gt68xx_device_setup_scan");
-
   if (dev->model->command_set->setup_scan)
     return (*dev->model->command_set->setup_scan) (dev, request, action,
 						   params);
@@ -717,7 +649,6 @@ SANE_Status
 gt68xx_device_set_afe (GT68xx_Device * dev, GT68xx_AFE_Parameters * params)
 {
   CHECK_DEV_ACTIVE (dev, "gt68xx_device_set_afe");
-
   if (dev->model->command_set->set_afe)
     return (*dev->model->command_set->set_afe) (dev, params);
   else
@@ -729,7 +660,6 @@ gt68xx_device_set_exposure_time (GT68xx_Device * dev,
 				 GT68xx_Exposure_Parameters * params)
 {
   CHECK_DEV_ACTIVE (dev, "gt68xx_device_set_exposure_time");
-
   if (dev->model->command_set->set_exposure_time)
     return (*dev->model->command_set->set_exposure_time) (dev, params);
   else
@@ -740,7 +670,6 @@ SANE_Status
 gt68xx_device_stop_scan (GT68xx_Device * dev)
 {
   CHECK_DEV_ACTIVE (dev, "gt68xx_device_stop_scan");
-
   if (dev->model->command_set->stop_scan)
     return (*dev->model->command_set->stop_scan) (dev);
   else
@@ -752,22 +681,16 @@ gt68xx_device_read_raw (GT68xx_Device * dev, SANE_Byte * buffer,
 			size_t * size)
 {
   SANE_Status status;
-
   CHECK_DEV_ACTIVE (dev, "gt68xx_device_read_raw");
-
   DBG (7, "gt68xx_device_read_raw: enter: size=%lu\n", (unsigned long) *size);
-
   status = sanei_usb_read_bulk (dev->fd, buffer, size);
-
   if (status != SANE_STATUS_GOOD)
     {
       DBG (3, "gt68xx_device_read_raw: bulk read failed: %s\n",
 	   sane_strstatus (status));
       return status;
     }
-
   DBG (7, "gt68xx_device_read_raw: leave: size=%lu\n", (unsigned long) *size);
-
   return SANE_STATUS_GOOD;
 }
 
@@ -775,7 +698,6 @@ SANE_Status
 gt68xx_device_set_read_buffer_size (GT68xx_Device * dev, size_t buffer_size)
 {
   CHECK_DEV_NOT_NULL (dev, "gt68xx_device_set_read_buffer_size");
-
   if (dev->read_active)
     {
       DBG (3, "gt68xx_device_set_read_buffer_size: BUG: read already "
@@ -795,21 +717,18 @@ gt68xx_device_set_read_buffer_size (GT68xx_Device * dev, size_t buffer_size)
 }
 
 SANE_Status
-gt68xx_device_read_prepare (GT68xx_Device * dev, size_t expected_count,
-			    SANE_Bool final_scan)
+gt68xx_device_read_prepare (GT68xx_Device * dev,
+			    size_t expected_count, SANE_Bool final_scan)
 {
   size_t buffer_size;
-
   CHECK_DEV_ACTIVE (dev, "gt68xx_device_read_prepare");
-
   if (dev->read_active)
     {
       DBG (3, "gt68xx_device_read_prepare: read already active\n");
       return SANE_STATUS_INVAL;
     }
   DBG (5, "gt68xx_device_read_prepare: total size: %lu bytes\n",
-	 (unsigned long) expected_count);
-
+       (unsigned long) expected_count);
   buffer_size = dev->requested_buffer_size;
   DBG (5, "gt68xx_device_read_prepare: requested buffer size: %lu\n",
        (unsigned long) buffer_size);
@@ -819,12 +738,12 @@ gt68xx_device_read_prepare (GT68xx_Device * dev, size_t expected_count,
     }
   DBG (5, "gt68xx_device_read_prepare: real size: %lu\n",
        (unsigned long) buffer_size);
-
   dev->read_buffer_size = buffer_size;
   dev->read_buffer = (SANE_Byte *) malloc (buffer_size);
   if (!dev->read_buffer)
     {
-      DBG (3, "gt68xx_device_read_prepare: not enough memory for the read buffer (%lu bytes)\n",
+      DBG (3,
+	   "gt68xx_device_read_prepare: not enough memory for the read buffer (%lu bytes)\n",
 	   (unsigned long) buffer_size);
       return SANE_STATUS_NO_MEM;
     }
@@ -833,23 +752,21 @@ gt68xx_device_read_prepare (GT68xx_Device * dev, size_t expected_count,
   dev->final_scan = final_scan;
   dev->read_pos = dev->read_bytes_in_buffer = 0;
   dev->read_bytes_left = expected_count;
-
   return SANE_STATUS_GOOD;
 }
 
 #ifdef USE_FORK
 
-static void
+static SANE_Status
 gt68xx_reader_process (GT68xx_Device * dev)
 {
-  SANE_Status status;
+  SANE_Status status = SANE_STATUS_GOOD;
   SANE_Int buffer_id;
   SANE_Byte *buffer_addr;
   size_t size;
   SANE_Int line = 0;
   size_t read_bytes_left = dev->read_bytes_left;
   shm_channel_writer_init (dev->shm_channel);
-
   while (read_bytes_left > 0)
     {
       status = shm_channel_writer_get_buffer (dev->shm_channel,
@@ -857,21 +774,18 @@ gt68xx_reader_process (GT68xx_Device * dev)
       if (status != SANE_STATUS_GOOD)
 	break;
       DBG (9, "gt68xx_reader_process: buffer %d: get\n", buffer_id);
-
       size = dev->read_buffer_size;
-
       DBG (9, "gt68xx_reader_process: buffer %d: trying to read %lu bytes "
 	   "(%lu bytes left, line %d)\n", buffer_id, (unsigned long) size,
 	   (unsigned long) read_bytes_left, line);
       status = gt68xx_device_read_raw (dev, buffer_addr, &size);
-
       if (status != SANE_STATUS_GOOD)
 	break;
-      DBG (9, "gt68xx_reader_process: buffer %d: read %lu bytes (line %d)\n",
+      DBG (9,
+	   "gt68xx_reader_process: buffer %d: read %lu bytes (line %d)\n",
 	   buffer_id, (unsigned long) size, line);
-
-      status = shm_channel_writer_put_buffer (dev->shm_channel,
-					      buffer_id, size);
+      status =
+	shm_channel_writer_put_buffer (dev->shm_channel, buffer_id, size);
       if (status != SANE_STATUS_GOOD)
 	break;
       DBG (9, "gt68xx_reader_process: buffer %d: put\n", buffer_id);
@@ -879,8 +793,11 @@ gt68xx_reader_process (GT68xx_Device * dev)
       line++;
     }
   DBG (9, "gt68xx_reader_process: finished, now sleeping\n");
+  if (status != SANE_STATUS_GOOD)
+    return status;
   sleep (5 * 60);		/* wait until we are killed (or timeout) */
   shm_channel_writer_close (dev->shm_channel);
+  return status;
 }
 
 static SANE_Status
@@ -888,10 +805,10 @@ gt68xx_device_read_start_fork (GT68xx_Device * dev)
 {
   SANE_Status status;
   int pid;
-
   if (dev->shm_channel)
     {
-      DBG (3, "gt68xx_device_read_start_fork: BUG: shm_channel already created\n");
+      DBG (3,
+	   "gt68xx_device_read_start_fork: BUG: shm_channel already created\n");
       return SANE_STATUS_INVAL;
     }
 
@@ -899,14 +816,14 @@ gt68xx_device_read_start_fork (GT68xx_Device * dev)
     shm_channel_new (dev->read_buffer_size, SHM_BUFFERS, &dev->shm_channel);
   if (status != SANE_STATUS_GOOD)
     {
-      DBG (3, "gt68xx_device_read_start_fork: cannot create shared memory channel: "
+      DBG (3,
+	   "gt68xx_device_read_start_fork: cannot create shared memory channel: "
 	   "%s\n", sane_strstatus (status));
       dev->shm_channel = NULL;
       return status;
     }
 
   pid = fork ();
-
   if (pid == -1)
     {
       DBG (3, "gt68xx_device_read_start_fork: cannot fork: %s\n",
@@ -919,8 +836,8 @@ gt68xx_device_read_start_fork (GT68xx_Device * dev)
   if (pid == 0)
     {
       /* Child process */
-      gt68xx_reader_process (dev);
-      _exit (0);
+      status = gt68xx_reader_process (dev);
+      _exit (status);
     }
   else
     {
@@ -928,7 +845,6 @@ gt68xx_device_read_start_fork (GT68xx_Device * dev)
       dev->reader_pid = pid;
       shm_channel_reader_init (dev->shm_channel);
       shm_channel_reader_start (dev->shm_channel);
-
       return SANE_STATUS_GOOD;
     }
 }
@@ -940,13 +856,11 @@ static SANE_Status
 gt68xx_device_read_start (GT68xx_Device * dev)
 {
   CHECK_DEV_ACTIVE (dev, "gt68xx_device_read_start");
-
 #ifdef USE_FORK
   /* Don't fork a separate process for every calibration scan. */
   if (dev->final_scan)
     return gt68xx_device_read_start_fork (dev);
 #endif /* USE_FORK */
-
   return SANE_STATUS_GOOD;
 }
 
@@ -962,9 +876,7 @@ gt68xx_device_read (GT68xx_Device * dev, SANE_Byte * buffer, size_t * size)
   SANE_Byte *buffer_addr;
   SANE_Int buffer_bytes;
 #endif /* USE_FORK */
-
   CHECK_DEV_ACTIVE (dev, "gt68xx_device_read");
-
   if (!dev->read_active)
     {
       DBG (3, "gt68xx_device_read: read not active\n");
@@ -983,7 +895,6 @@ gt68xx_device_read (GT68xx_Device * dev, SANE_Byte * buffer, size_t * size)
 	  raw_block_size = (block_size + 63UL) & ~63UL;
 	  DBG (7, "gt68xx_device_read: trying to read %ld bytes\n",
 	       (long) raw_block_size);
-
 #ifdef USE_FORK
 	  if (dev->shm_channel)
 	    {
@@ -993,12 +904,10 @@ gt68xx_device_read (GT68xx_Device * dev, SANE_Byte * buffer, size_t * size)
 						      &buffer_bytes);
 	      if (status == SANE_STATUS_GOOD && buffer_addr != NULL)
 		{
-		  DBG (9, "gt68xx_device_read: buffer %d: get\n",
-			 buffer_id);
+		  DBG (9, "gt68xx_device_read: buffer %d: get\n", buffer_id);
 		  memcpy (dev->read_buffer, buffer_addr, buffer_bytes);
 		  shm_channel_reader_put_buffer (dev->shm_channel, buffer_id);
-		  DBG (9, "gt68xx_device_read: buffer %d: put\n",
-			 buffer_id);
+		  DBG (9, "gt68xx_device_read: buffer %d: put\n", buffer_id);
 		}
 	    }
 	  else
@@ -1030,7 +939,6 @@ gt68xx_device_read (GT68xx_Device * dev, SANE_Byte * buffer, size_t * size)
     }
 
   *size = byte_count;
-
   if (byte_count == 0)
     return SANE_STATUS_EOF;
   else
@@ -1040,8 +948,8 @@ gt68xx_device_read (GT68xx_Device * dev, SANE_Byte * buffer, size_t * size)
 SANE_Status
 gt68xx_device_read_finish (GT68xx_Device * dev)
 {
+  SANE_Status status = SANE_STATUS_GOOD;
   CHECK_DEV_ACTIVE (dev, "gt68xx_device_read_finish");
-
   if (!dev->read_active)
     {
       DBG (3, "gt68xx_device_read_finish: read not active\n");
@@ -1050,15 +958,16 @@ gt68xx_device_read_finish (GT68xx_Device * dev)
 
   DBG (7, "gt68xx_device_read_finish: read_bytes_left = %ld\n",
        (long) dev->read_bytes_left);
-
 #ifdef USE_FORK
   if (dev->reader_pid != 0)
     {
-      int status;
-      /* usleep (100000);*/
+      int pid_status;
+      /* usleep (100000); */
       DBG (7, "gt68xx_device_read_finish: trying to kill reader process\n");
       kill (dev->reader_pid, SIGKILL);
-      waitpid (dev->reader_pid, &status, 0);
+      waitpid (dev->reader_pid, &pid_status, 0);
+      if (WIFEXITED (pid_status))
+	status = WEXITSTATUS (pid_status);
       DBG (7, "gt68xx_device_read_finish: reader process killed\n");
       dev->reader_pid = 0;
     }
@@ -1072,10 +981,9 @@ gt68xx_device_read_finish (GT68xx_Device * dev)
 
   free (dev->read_buffer);
   dev->read_buffer = NULL;
-
   dev->read_active = SANE_FALSE;
-
-  return SANE_STATUS_GOOD;
+  DBG (7, "gt68xx_device_read_finish: exit (%s)\n", sane_strstatus (status));
+  return status;
 }
 
 static SANE_Status
@@ -1099,7 +1007,6 @@ SANE_Status
 gt68xx_device_get_id (GT68xx_Device * dev)
 {
   CHECK_DEV_ACTIVE (dev, "gt68xx_device_get_id");
-
   if (dev->model->command_set->get_id)
     return (*dev->model->command_set->get_id) (dev);
   else
