@@ -60,6 +60,8 @@
 #include <unistd.h>
 #include <math.h>
 #include <netinet/in.h>		/* for htons */
+#include <string.h>
+
 
 #include "hp5400.h"
 #include "hp5400_xfer.h"
@@ -74,6 +76,11 @@
 #define max(A,B) (((A)>(B)) ? (A) : (B))
 #endif
 
+
+#ifndef STRING_VERSION_MATCH
+#define NO_STRING_VERSION_MATCH 1
+#endif
+
 #ifdef __GNUC__
 #define PACKED __attribute__ ((packed))
 #else
@@ -204,6 +211,11 @@ SetCalibration (int iHandle, int numPixels, unsigned int *low_vals[3],
 		unsigned int *high_vals[3], int dpi)
 {
   char cmd[8];
+  /* unsigned char cmd[8]; */ /* should fix the compilation warning
+				 but I don't have a scanner right now 
+				 to check that the fix does not break
+				 calibration */
+  
   int i, j;
   struct CalPixel
   {
@@ -231,7 +243,7 @@ SetCalibration (int iHandle, int numPixels, unsigned int *low_vals[3],
   int calSize = numLoop * calBlockSize * numCalBlock;
 
   calinfo = malloc (calSize);
-  bzero (calinfo, calSize);
+  memset (calinfo, 0, calSize);
 
   for (j = 0; j < numLoop * numCalBlock * CALPIXBYBLOCK; j++)
     {
@@ -304,6 +316,7 @@ WriteGammaCalibTable (int iHandle, const int *pabGammaR, const int *pabGammaG,
   return;
 }
 
+#ifdef STANDALONE
 HP5400_SANE_STATIC
 void
 SetDefaultGamma (int iHandle)
@@ -316,6 +329,7 @@ SetDefaultGamma (int iHandle)
 
   WriteGammaCalibTable (iHandle, buffer, buffer, buffer);
 }
+#endif
 
 #define BUFFER_SIZE (6*65536)
 
@@ -384,9 +398,9 @@ int
 CircBufferGetLine (int iHandle, TDataPipe * p, void *pabLine)
 {
   int i;
-
   int maxoff = 0;
-
+  char* buftmp = (char*) (p->buffer);
+  
 /*  HP5400_DBG(DBG_MSG, "CircBufferGetLine:\n");   */
 
   if (p->roff > maxoff)
@@ -416,13 +430,13 @@ CircBufferGetLine (int iHandle, TDataPipe * p, void *pabLine)
 
       /* free old buffer */
       free (tmpBuf);
+      buftmp = (char*)(p->buffer);
     }
 
   while (p->bufstart + maxoff >= p->bufend)	/* Not enough data in buffer */
     {
       int res;
       unsigned short cmd[4] = { 0, 0, 0, 0 };
-
       cmd[2] = p->blksize;
 
       assert ((p->bufend + p->blksize) <= p->buffersize);
@@ -432,14 +446,19 @@ CircBufferGetLine (int iHandle, TDataPipe * p, void *pabLine)
 
       res =
 	hp5400_bulk_read_block (iHandle, CMD_INITBULK3, cmd, sizeof (cmd),
-    				p->buffer + p->bufend, p->blksize);
+    				buftmp + p->bufend, p->blksize);
       if (res != p->blksize)
 	{
 	  HP5400_DBG (DBG_ERR, "*** ERROR: Read returned %d. FATAL.", res);
 	  return -1;
 	}
 #ifdef IMAGE_DEBUG
-      fwrite (p->buffer + p->bufend, p->blksize, 1, temp);
+      fwrite (
+    	buftmp + p->bufend
+	,p->blksize
+	,1
+	,temp
+	);
 #endif
       p->bufend += p->blksize;
     }
@@ -450,9 +469,9 @@ CircBufferGetLine (int iHandle, TDataPipe * p, void *pabLine)
   if (p->bpp == 1)
     {
       char *itPix = (char *) pabLine;
-      char *itR = (char *) (p->buffer + p->bufstart + p->roff);
-      char *itG = (char *) (p->buffer + p->bufstart + p->goff);
-      char *itB = (char *) (p->buffer + p->bufstart + p->boff);
+      char *itR = (char *) (buftmp + p->bufstart + p->roff);
+      char *itG = (char *) (buftmp + p->bufstart + p->goff);
+      char *itB = (char *) (buftmp + p->bufstart + p->boff);
       for (i = 0; i < p->pixels; i++)
 	{
 	  /* pointer move goes a little bit faster than vector access */
@@ -466,9 +485,9 @@ CircBufferGetLine (int iHandle, TDataPipe * p, void *pabLine)
   else
     {
       short *itPix = (short *) pabLine;
-      short *itR = (short *) (p->buffer + p->bufstart + p->roff);
-      short *itG = (short *) (p->buffer + p->bufstart + p->goff);
-      short *itB = (short *) (p->buffer + p->bufstart + p->boff);
+      short *itR = (short *) (buftmp + p->bufstart + p->roff);
+      short *itG = (short *) (buftmp + p->bufstart + p->goff);
+      short *itB = (short *) (buftmp + p->bufstart + p->boff);
       for (i = 0; i < p->pixels; i++)
 	{
 #if 0
@@ -496,7 +515,11 @@ CircBufferGetLine (int iHandle, TDataPipe * p, void *pabLine)
   /* If we've used a whole block at the beginning, move it */
   if (p->bufstart > p->blksize)
     {
-      memmove (p->buffer, p->buffer + p->bufstart, p->bufend - p->bufstart);
+      memmove (
+	buftmp
+	,buftmp + p->bufstart
+	,p->bufend - p->bufstart
+	);
 
       p->bufend -= p->bufstart;
       p->bufstart = 0;
@@ -519,6 +542,7 @@ CircBufferExit (TDataPipe * p)
 }
 
 
+#ifdef STANDALONE
 /* bpp is BYTES per pixel */
 HP5400_SANE_STATIC
 void
@@ -577,7 +601,6 @@ DecodeImage (FILE * file, int planes, int bpp, int xsize, int ysize,
   free (buf);
 }
 
-
 HP5400_SANE_STATIC
 int
 hp5400_test_scan_response (struct ScanResponse *resp, struct ScanRequest *req)
@@ -592,6 +615,7 @@ hp5400_test_scan_response (struct ScanResponse *resp, struct ScanRequest *req)
        htons (resp->ysize));
   return 1;
 }
+#endif
 
 /* This is a very specialised scanning function. It does the scan request as
  * usual but instead of producing an image it returns three arrays of ints.
@@ -650,6 +674,7 @@ DoAverageScan (int iHandle, struct ScanRequest *req, int code,
   return 0;
 }
 
+#ifdef STANDALONE
 HP5400_SANE_STATIC
 int
 DoScan (int iHandle, struct ScanRequest *req, const char *filename, int code,
@@ -699,6 +724,7 @@ DoScan (int iHandle, struct ScanRequest *req, const char *filename, int code,
   fclose (file);
   return 0;
 }
+#endif
 
 HP5400_SANE_STATIC
 int
@@ -714,7 +740,7 @@ Calibrate (int iHandle, int dpi)
 #endif
 
 /* The first calibration scan. Finds maximum of each CCD */
-  bzero (&req, sizeof (req));
+  memset(&req, 0, sizeof(req));
 
   req.x1 = 0x08;
   req.dpix = htons (300);	/* = 300 dpi */
@@ -772,7 +798,7 @@ Calibrate (int iHandle, int dpi)
 #endif
 
 /* The second calibration scan. Finds minimum of each CCD */
-  bzero (&req, sizeof (req));
+  memset(&req, 0, sizeof(req));
 
   req.x1 = 0x08;
   req.dpix = htons (300);	/* = 300 dpi */
@@ -834,6 +860,7 @@ Calibrate (int iHandle, int dpi)
   return 0;
 }
 
+#ifdef STANDALONE
 HP5400_SANE_STATIC
 int
 hp5400_scan (int iHandle, TScanParams * params, THWParams * pHWParams,
@@ -855,7 +882,7 @@ hp5400_scan (int iHandle, TScanParams * params, THWParams * pHWParams,
   HP5400_DBG (DBG_MSG, "   height : %d\n", params->iHeight);
   HP5400_DBG (DBG_MSG, "\n");
 
-  bzero (&req, sizeof (req));
+  memset(&req, 0, sizeof(req));
 
   req.x1 = 0x08;
   req.dpix = htons (params->iDpi);
@@ -901,7 +928,6 @@ hp5400_scan (int iHandle, TScanParams * params, THWParams * pHWParams,
   return result;
 }
 
-
 HP5400_SANE_STATIC
 int
 PreviewScan (int iHandle)
@@ -921,6 +947,7 @@ PreviewScan (int iHandle)
   return hp5400_scan (iHandle, &params, &pHWParams, "output.ppm");
 }
 
+
 static char UISetup1[] = {
 /* Offset 40 */
   0x50, 0x72, 0x6F, 0x63, 0x65, 0x73, 0x73, 0x69, 0x6E, 0x67, 0x14, 0x00,
@@ -952,7 +979,7 @@ InitScanner (int iHandle)
 
   if (hp5400_command_write (iHandle, 0xF10B, sizeof (UISetup1), UISetup1) < 0)
     {
-      HP5400_DBG (DBG_MSG, "failed to send UISetup1 (%d)\n", sizeof (UISetup1));
+      HP5400_DBG (DBG_MSG, "failed to send UISetup1 (%lu)\n", (u_long) sizeof (UISetup1));
       return -1;
     }
 
@@ -966,6 +993,7 @@ InitScanner (int iHandle)
     }
   return 0;
 }
+#endif
 
 /* Warning! The caller must have configured the gamma tables at this stage */
 HP5400_SANE_STATIC
@@ -976,8 +1004,8 @@ InitScan (enum ScanType scantype, TScanParams * pParams,
   struct ScanRequest req;
   struct ScanResponse res;
   int ret;
-
-  bzero (&req, sizeof (req));
+  
+  memset(&req, 0, sizeof(req));
 
   req.x1 = 0x08;
   req.dpix = htons (pParams->iDpi);	/* = 300 dpi */
@@ -1030,7 +1058,7 @@ InitScan2 (enum ScanType scantype, struct ScanRequest *req,
   struct ScanResponse res;
   int iHandle = pHWParams->iXferHandle;
 
-  bzero (&res, sizeof (res));
+  memset(&res, 0, sizeof(res));
 
   /* Protect scanner from damage. This stops stpuid errors.  It basically
    * limits you to the scanner glass. Stuff like calibrations which need
@@ -1227,7 +1255,7 @@ FinishScan (THWParams * pHWParams)
 
 HP5400_SANE_STATIC
 int
-HP5400Open (THWParams * params, char *filename)
+HP5400Open (THWParams * params, const char *filename)
 {
   int iHandle = hp5400_open (filename);
   char szVersion[32];
@@ -1306,9 +1334,9 @@ HP5400Close (THWParams * params)
 
 HP5400_SANE_STATIC
 int
-HP5400Detect (char *filename,
+HP5400Detect (const char *filename,
 	      int (*_ReportDevice) (TScannerModel * pModel,
-				    char *pszDeviceName))
+				    const char *pszDeviceName))
 {
   int iHandle = hp5400_open (filename);
 
