@@ -1,6 +1,6 @@
 /* sane - Scanner Access Now Easy.
 
-   Copyright (C) 2002-2003 Frank Zago (fzago at austin dot rr dot com)
+   Copyright (C) 2002-2003 Frank Zago (sane at zago dot net)
    
    This file is part of the SANE package.
    
@@ -42,7 +42,7 @@
 */
 
 /*
-   $Id: leo.c,v 1.9 2004/07/01 02:35:20 fzago-guest Exp $
+   $Id: leo.c,v 1.11 2005/07/07 11:55:42 fzago-guest Exp $
    Across FS-1130
 */
 
@@ -433,15 +433,13 @@ leo_identify_scanner (Leo_Scanner * dev)
 
 /* SCSI sense handler. Callback for SANE. */
 static SANE_Status
-leo_sense_handler (int scsi_fd, unsigned char *result, void *arg)
+leo_sense_handler (int scsi_fd, unsigned char *result, void __unused__ *arg)
 {
   int asc, ascq, sensekey;
   int len;
 
   DBG (DBG_proc, "leo_sense_handler (scsi_fd = %d)\n", scsi_fd);
 
-  arg = arg;			/* silence gcc */
-
   sensekey = get_RS_sense_key (result);
   len = 7 + get_RS_additional_length (result);
 
@@ -1292,7 +1290,7 @@ leo_send_halftone_pattern (Leo_Scanner * dev)
 /* Sane entry points */
 
 SANE_Status
-sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
+sane_init (SANE_Int * version_code, SANE_Auth_Callback __unused__ authorize)
 {
   FILE *fp;
   char dev_name[PATH_MAX];
@@ -1302,8 +1300,6 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
 
   DBG (DBG_sane_init, "sane_init\n");
 
-  authorize = authorize;	/* silence gcc */
-
   DBG (DBG_error, "This is sane-leo version %d.%d-%d\n", V_MAJOR,
        V_MINOR, BUILD);
   DBG (DBG_error, "(C) 2002 by Frank Zago\n");
@@ -1341,15 +1337,13 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
 }
 
 SANE_Status
-sane_get_devices (const SANE_Device *** device_list, SANE_Bool local_only)
+sane_get_devices (const SANE_Device *** device_list, SANE_Bool __unused__ local_only)
 {
   Leo_Scanner *dev;
   int i;
 
   DBG (DBG_proc, "sane_get_devices: enter\n");
 
-  local_only = local_only;	/* silence gcc */
-
   if (devlist)
     free (devlist);
 
@@ -1917,16 +1911,13 @@ sane_read (SANE_Handle handle, SANE_Byte * buf, SANE_Int max_len,
 }
 
 SANE_Status
-sane_set_io_mode (SANE_Handle handle, SANE_Bool non_blocking)
+sane_set_io_mode (SANE_Handle __unused__ handle, SANE_Bool __unused__ non_blocking)
 {
   SANE_Status status;
   Leo_Scanner *dev = handle;
 
   DBG (DBG_proc, "sane_set_io_mode: enter\n");
 
-  handle = handle;		/* silence gcc */
-  non_blocking = non_blocking;	/* silence gcc */
-
   if (!dev->scanning)
     {
       return SANE_STATUS_INVAL;
@@ -1947,13 +1938,10 @@ sane_set_io_mode (SANE_Handle handle, SANE_Bool non_blocking)
 }
 
 SANE_Status
-sane_get_select_fd (SANE_Handle handle, SANE_Int * fd)
+sane_get_select_fd (SANE_Handle __unused__ handle, SANE_Int __unused__ * fd)
 {
   DBG (DBG_proc, "sane_get_select_fd: enter\n");
 
-  handle = handle;		/* silence gcc */
-  fd = fd;			/* silence gcc */
-
   DBG (DBG_proc, "sane_get_select_fd: exit\n");
 
   return SANE_STATUS_UNSUPPORTED;
