@@ -1,6 +1,6 @@
 /* sane - Scanner Access Now Easy.
 
-   Copyright (C) 2002, 2004 Frank Zago (fzago at austin dot rr dot com)
+   Copyright (C) 2002, 2004 Frank Zago (sane at zago dot net)
    Copyright (C) 2002 Other SANE contributors
    
    This file is part of the SANE package.
@@ -43,7 +43,7 @@
 */
 
 /*
-   $Id: matsushita.c,v 1.7 2004/02/12 01:59:53 fzago-guest Exp $
+   $Id: matsushita.c,v 1.9 2005/07/07 11:55:42 fzago-guest Exp $
    Matsushita/Panasonic KV-SS25, KV-SS50, KV-SS55, KV-SS50EX, 
                         KV-SS55EX, KV-SS850, KV-SS855 SCSI scanners.
 
@@ -840,15 +840,13 @@ get_int_list_index (const SANE_Word list[], const SANE_Word value)
 
 /* SCSI sense handler. Callback for SANE. */
 static SANE_Status
-matsushita_sense_handler (int scsi_fd, unsigned char *result, void *arg)
+matsushita_sense_handler (int scsi_fd, unsigned char *result, void __unused__ *arg)
 {
   int asc, ascq, sensekey;
   int len;
 
   DBG (DBG_proc, "matsushita_sense_handler (scsi_fd = %d)\n", scsi_fd);
 
-  arg = arg;			/* silence gcc */
-
   sensekey = get_RS_sense_key (result);
   len = 7 + get_RS_additional_length (result);
 
@@ -1705,7 +1703,7 @@ do_cancel (Matsushita_Scanner * dev)
 /* Entry points */
 
 SANE_Status
-sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
+sane_init (SANE_Int * version_code, SANE_Auth_Callback __unused__ authorize)
 {
   FILE *fp;
   char dev_name[PATH_MAX];
@@ -1715,8 +1713,6 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
 
   DBG (DBG_sane_init, "sane_init\n");
 
-  authorize = authorize;	/* silence gcc */
-
   DBG (DBG_error, "This is sane-matsushita version %d.%d-%d\n", V_MAJOR,
        V_MINOR, BUILD);
   DBG (DBG_error, "(C) 2002 by Frank Zago\n");
@@ -1754,15 +1750,13 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
 }
 
 SANE_Status
-sane_get_devices (const SANE_Device *** device_list, SANE_Bool local_only)
+sane_get_devices (const SANE_Device *** device_list, SANE_Bool __unused__ local_only)
 {
   Matsushita_Scanner *dev;
   int i;
 
   DBG (DBG_proc, "sane_get_devices: enter\n");
 
-  local_only = local_only;	/* silence gcc */
-
   if (devlist)
     free (devlist);
 
@@ -2418,16 +2412,13 @@ sane_read (SANE_Handle handle, SANE_Byte * buf, SANE_Int max_len,
 }
 
 SANE_Status
-sane_set_io_mode (SANE_Handle handle, SANE_Bool non_blocking)
+sane_set_io_mode (SANE_Handle __unused__ handle, SANE_Bool __unused__ non_blocking)
 {
 	SANE_Status status;
 	Matsushita_Scanner *dev = handle;
 
   DBG (DBG_proc, "sane_set_io_mode: enter\n");
 
-  handle = handle;		/* silence gcc */
-  non_blocking = non_blocking;	/* silence gcc */
-
     if (dev->scanning == SANE_FALSE)
     {
       return (SANE_STATUS_INVAL);
@@ -2445,13 +2436,10 @@ sane_set_io_mode (SANE_Handle handle, SANE_Bool non_blocking)
 }
 
 SANE_Status
-sane_get_select_fd (SANE_Handle handle, SANE_Int * fd)
+sane_get_select_fd (SANE_Handle __unused__ handle, SANE_Int __unused__ * fd)
 {
   DBG (DBG_proc, "sane_get_select_fd: enter\n");
 
-  handle = handle;		/* silence gcc */
-  fd = fd;			/* silence gcc */
-
   DBG (DBG_proc, "sane_get_select_fd: exit\n");
 
   return SANE_STATUS_UNSUPPORTED;
