@@ -7,7 +7,7 @@
  *  @brief The interface functions to the USB driver stuff.
  *
  * Based on sources acquired from Plustek Inc.<br>
- * Copyright (C) 2001-2004 Gerhard Jaeger <gerhard@gjaeger.de>
+ * Copyright (C) 2001-2005 Gerhard Jaeger <gerhard@gjaeger.de>
  *
  * History:
  * - 0.40 - starting version of the USB support
@@ -38,6 +38,7 @@
  *        - change usbDev_stopScan and usbDev_open prototype
  *        - cleanup
  * - 0.48 - added function usb_CheckAndCopyAdjs()
+ * - 0.49 - changed autodetection
  * .
  * <hr>
  * This file is part of the SANE package.
@@ -86,7 +87,7 @@ typedef struct {
 	int	  id;
 	char *desc;
 } TabDef, *pTabDef;
-  
+
 /** to allow different vendors...
  */
 static TabDef usbVendors[] = {
@@ -103,9 +104,6 @@ static TabDef usbVendors[] = {
 	{ 0xFFFF, NULL              }
 };
 
-/** for autodetection */
-static SANE_Char USB_devname[1024];
-
 /** we use at least 8 megs for scanning... */
 #define _SCANBUF_SIZE (8 * 1024 * 1024)
 
@@ -194,7 +192,8 @@ static void usb_initDev( pPlustek_Device dev, int idx, int handle, int vendor )
 	}
 
 	usb_CheckAndCopyAdjs( dev );
-	DBG( _DBG_INFO, "Device WAF: 0x%08lx\n", dev->usbDev.Caps.workaroundFlag );
+	DBG( _DBG_INFO, "Device WAF  : 0x%08lx\n", dev->usbDev.Caps.workaroundFlag );
+	DBG( _DBG_INFO, "Transferrate: %lu Bytes/s\n", dev->transferRate );
 
 	/* adjust data origin
 	 */
@@ -271,7 +270,6 @@ static void usb_initDev( pPlustek_Device dev, int idx, int handle, int vendor )
 	sParam.bSource       = SOURCE_Reflection;
 	sParam.Origin.x      = 0;
 	sParam.Origin.y      = 0;
-	sParam.siThreshold   = 0;
 	sParam.UserDpi.x     = 150;
 	sParam.UserDpi.y     = 150;
 	sParam.dMCLK         = 4;
@@ -385,9 +383,8 @@ static void usbDev_shutdown( Plustek_Device *dev  )
 	SANE_Int handle;
 
 	DBG( _DBG_INFO, "Shutdown called (dev->fd=%d, %s)\n",
-													dev->fd, dev->sane.name );
+	                dev->fd, dev->sane.name );
 	if( NULL == dev->usbDev.ModelStr ) {
-	
 		DBG( _DBG_INFO, "Function ignored!\n" );
 		return;
 	}
@@ -433,112 +430,206 @@ static SANE_Bool usb_IsDeviceInList( char *usbIdStr )
 	return SANE_FALSE;
 }
 
-/**
+/** get last valid entry of a list
+ */
+static DevList*
+getLast( DevList *l )
+{
+	if( l == NULL )
+		return NULL;
+
+	while( l->next != NULL )
+		l = l->next;
+	return l; 
+}
+
+/** add a new entry to our internal device list, when a device is detected
  */
 static SANE_Status usb_attach( SANE_String_Const dev_name )
 {
-	if( USB_devname[0] == '\0' ) {
-		DBG( _DBG_INFO, "Found device at >%s<\n", dev_name );
-		strncpy( USB_devname, dev_name, 1023 );
-		USB_devname[1023] = '\0';
-	} else {
-		DBG( _DBG_INFO, "Device >%s< ignoring\n", dev_name );
-	}
+	int      len;
+	DevList *tmp, *last;
 
+	/* get some memory for the entry... */
+	len = sizeof(DevList) + strlen(dev_name) + 1;
+	tmp = (DevList*)malloc(len);
+
+	/* initialize and set the values */
+	memset(tmp, 0, len );
+
+	/* the device name is part of our structure space */
+	tmp->dev_name = &((char*)tmp)[sizeof(DevList)];
+	strcpy( tmp->dev_name, dev_name );
+	tmp->attached = SANE_FALSE;
+
+	/* root set ? */
+	if( usbDevs == NULL ) {
+		usbDevs = tmp;
+	} else {
+		last = getLast(usbDevs); /* append... */
+		last->next = tmp;
+	}
 	return SANE_STATUS_GOOD;
 }
 
-/** here we roam through our list of supported devices and
- * cross check with the ones the system reports...
- * @param vendor  - pointer to receive vendor ID
- * @param product - pointer to receive product ID
- * @return SANE_TRUE if a matching device has been found or
- *         SANE_FALSE if nothing supported found...
+/**
  */
-static SANE_Bool usbDev_autodetect( SANE_Word *vendor, SANE_Word *product )
+static void
+usbGetList( DevList **devs )
 {
-	int       i;
-	SANE_Word v, p;
-
-	DBG( _DBG_INFO, "Autodetection...\n" );
+	int        i;
+	SANE_Bool  il;
+	SANE_Word  v, p;
+	DevList   *tmp;
 
+	DBG( _DBG_INFO, "Retrieving all supported and conntected devices\n" );
 	for( i = 0; NULL != Settings[i].pIDString; i++ ) {
 
 		v = strtol( &(Settings[i].pIDString)[0], 0, 0 );
 		p = strtol( &(Settings[i].pIDString)[7], 0, 0 );
 
-		DBG( _DBG_INFO2, "Checking for 0x%04x-0x%04x\n", v, p );
+		/* check if this vendor- and product-ID has already been added, needed
+		 * for Plustek devices - here on product-ID is used for more than one
+		 * device type...
+		 */
+		il = SANE_FALSE;
+		for( tmp = *devs; tmp ; tmp = tmp->next ) {
+			if( tmp->device_id == p && tmp->vendor_id == v ) {
+				il = SANE_TRUE;
+				break;
+			}
+		} 
+		if( il ) {
+			DBG( _DBG_INFO2, "Already in list: 0x%04x-0x%04x\n", v, p );
+			continue;
+		}
 
+		/* get the last entry... */
+		tmp = getLast(*devs);
+		DBG( _DBG_INFO2, "Checking for 0x%04x-0x%04x\n", v, p );
 		sanei_usb_find_devices( v, p, usb_attach );
 
-		if( USB_devname[0] != '\0' ) {
+		if( getLast(*devs) != tmp ) {
+			
+			if( tmp == NULL )
+				tmp = *devs;
+			else
+				tmp = tmp->next;
 
-			*vendor  = v;
-			*product = p;
-			return SANE_TRUE;
+			while( tmp != NULL ) {
+				tmp->vendor_id = v;
+				tmp->device_id = p;
+				tmp = tmp->next;
+			}
 		}
 	}
 
-	return SANE_FALSE;
+	DBG( _DBG_INFO, "Available and supported devices:\n" );
+	if( *devs == NULL )
+		DBG( _DBG_INFO, "NONE.\n" );
+
+	for( tmp = *devs; tmp; tmp = tmp->next ) {
+		DBG( _DBG_INFO, "Device: >%s< - 0x%04xx0x%04x\n", 
+			tmp->dev_name, tmp->vendor_id, tmp->device_id );
+	}
 }
 
 /**
  */
-static int usbDev_open( Plustek_Device *dev )
+static int usbDev_open( Plustek_Device *dev, DevList *devs, int keep_lock )
 {
-	char      devStr[50];
-	int       result;
-	int       i;
-	int       lc;
-	SANE_Int  handle;
-	SANE_Byte version;
-	SANE_Word vendor, product;
-	SANE_Bool was_empty;
+	char        dn[512];
+	char        devStr[50];
+	int         result;
+	int         i;
+	int         lc;
+	SANE_Int    handle;
+	SANE_Byte   version;
+	SANE_Word   vendor, product;
+	SANE_Bool   was_empty;
+	SANE_Status status;
+	DevList    *tmp;
 
-	DBG( _DBG_INFO, "usbDev_open(%s,%s)\n", dev->name, dev->usbId );
+	DBG( _DBG_INFO, "usbDev_open(%s,%s) - %p\n", 
+	    dev->name, dev->usbId, (void*)devs );
 
 	/* preset our internal usb device structure */
 	memset( &dev->usbDev, 0, sizeof(DeviceDef));
-	USB_devname[0] = '\0';
 
-	if( !strcmp( dev->name, "auto" )) {
+	/* devs is NULL, when called from sane_start */
+	if( devs ) {
+	
+		dn[0] = '\0';
+		if( !strcmp( dev->name, "auto" )) {
 
-		if( dev->usbId[0] == '\0' ) {
-
-			if( !usbDev_autodetect( &vendor, &product )) {
-				DBG( _DBG_ERROR, "No supported device found!\n" );
-				return -1;
+			/* use the first "unattached"... */
+			for( tmp = devs; tmp; tmp = tmp->next ) {
+				if( !tmp->attached ) {
+					tmp->attached = SANE_TRUE;
+					strcpy( dn, tmp->dev_name );
+					break;
+				}
 			}
-
 		} else {
 
 			vendor  = strtol( &dev->usbId[0], 0, 0 );
 			product = strtol( &dev->usbId[7], 0, 0 );
 
-			sanei_usb_find_devices( vendor, product, usb_attach );
-
-			if( USB_devname[0] == '\0' ) {
-				DBG( _DBG_ERROR, "No matching device found!\n" );
-        		return -1;
+			/* check the first match... */
+			for( tmp = devs; tmp; tmp = tmp->next ) {
+				if( tmp->vendor_id == vendor && tmp->device_id == product ) {
+					if( !tmp->attached ) {
+						tmp->attached = SANE_TRUE;
+						strcpy( dn, tmp->dev_name );
+						break;
+					}
+				}
 			}
 		}
 
-		if( SANE_STATUS_GOOD != sanei_usb_open( USB_devname, &handle ))
-    		return -1;
+		if( !dn[0] ) {
+			DBG( _DBG_ERROR, "No supported device found!\n" );
+			return -1;
+		}
+
+		status = sanei_access_lock( dn, 5 );
+		if( SANE_STATUS_GOOD != status ) {
+			DBG( _DBG_ERROR, "sanei_access_lock failed: %d\n", status );
+			return -1;
+		}
+
+		status = sanei_usb_open( dn, &handle );
+		if( SANE_STATUS_GOOD != status ) {
+			DBG( _DBG_ERROR, "sanei_usb_open failed: %s (%d)\n", 
+			     strerror(errno), errno);
+			sanei_access_unlock( dev->sane.name );
+			return -1;
+		}
 
 		/* replace the old devname, so we are able to have multiple
-         * auto-detected devices
-         */
+		 * auto-detected devices
+		 */
 		free( dev->name );
-	    dev->name      = strdup( USB_devname );
+		dev->name      = strdup(dn);
 		dev->sane.name = dev->name; 
 
 	} else {
 
-		if( SANE_STATUS_GOOD != sanei_usb_open( dev->name, &handle ))
-    		return -1;
+		status = sanei_access_lock( dev->sane.name, 5 );
+		if( SANE_STATUS_GOOD != status ) {
+			DBG( _DBG_ERROR, "sanei_access_lock failed: %d\n", status );
+			return -1;
+		}
+
+		status = sanei_usb_open( dev->name, &handle );
+		if( SANE_STATUS_GOOD != status ) {
+			DBG( _DBG_ERROR, "sanei_usb_open failed: %s (%d)\n", 
+			     strerror(errno), errno);
+			sanei_access_unlock( dev->sane.name );
+			return -1;
+		}
 	}
-	
+
 	was_empty = SANE_FALSE;
 
 	result = sanei_usb_get_vendor_product( handle, &vendor, &product );
@@ -555,8 +646,9 @@ static int usbDev_open( Plustek_Device *dev )
 				DBG( _DBG_ERROR, "Specified Vendor and Product ID "
 								 "doesn't match with the ones\n"
 								 "in the config file\n" );
+				sanei_access_unlock( dev->sane.name );
 				sanei_usb_close( handle );
-		        return -1;
+				return -1;
 			}
 		} else {
 			sprintf( dev->usbId, "0x%04X-0x%04X", vendor, product );
@@ -568,15 +660,16 @@ static int usbDev_open( Plustek_Device *dev )
 		DBG( _DBG_INFO, "Can't get vendor & product ID from driver...\n" );
 
 		/* if the ioctl stuff is not supported by the kernel and we have
-         * nothing specified, we have to give up...
-         */
+		 * nothing specified, we have to give up...
+		 */
 		if( dev->usbId[0] == '\0' ) {
 			DBG( _DBG_ERROR, "Cannot autodetect Vendor an Product ID, "
 							 "please specify in config file.\n" );
+			sanei_access_unlock( dev->sane.name );
 			sanei_usb_close( handle );
 	        return -1;
 		}
-		
+
 		vendor  = strtol( &dev->usbId[0], 0, 0 );
 		product = strtol( &dev->usbId[7], 0, 0 );
 		DBG( _DBG_INFO, "... using the specified: "
@@ -587,18 +680,22 @@ static int usbDev_open( Plustek_Device *dev )
 	 */
 	if( !usb_IsDeviceInList( dev->usbId )) {
 		DBG( _DBG_ERROR, "Device >%s<, is not supported!\n", dev->usbId );
+		sanei_access_unlock( dev->sane.name );
 		sanei_usb_close( handle );
 		return -1;
 	}
 
-	if( SANE_STATUS_GOOD != usbio_DetectLM983x( handle, &version )) {
+	status = usbio_DetectLM983x( handle, &version );
+	if( SANE_STATUS_GOOD != status ) {
 		sanei_usb_close( handle );
+		sanei_access_unlock( dev->sane.name );
 		return -1;
 	}
 
 	if ((version < 3) || (version > 4)) {
 		DBG( _DBG_ERROR, "This is not a LM9831 or LM9832 chip based scanner.\n" );
 		sanei_usb_close( handle );
+		sanei_access_unlock( dev->sane.name );
 		return -1;
 	}
 
@@ -625,8 +722,11 @@ static int usbDev_open( Plustek_Device *dev )
 		if( was_empty )
 			dev->usbId[0] = '\0';
 
-		if( handle >= 0 )
+		if( handle >= 0 ) {
+			if( !keep_lock )
+				sanei_access_unlock( dev->sane.name );
 			return handle;	
+		}
 		
 	} else {
 
@@ -655,11 +755,14 @@ static int usbDev_open( Plustek_Device *dev )
 			if( 0 == strncmp( Settings[i].pIDString, devStr, lc )) {
 				DBG( _DBG_INFO, "Device description for >%s< found.\n", devStr );
 				usb_initDev( dev, i, handle, vendor );
+				if( !keep_lock )
+					sanei_access_unlock( dev->sane.name );
 			    return handle;
 			}
 		}
 	}
 
+	sanei_access_unlock( dev->sane.name );
 	sanei_usb_close( handle );
 	DBG( _DBG_ERROR, "No matching device found >%s<\n", devStr );
 	return -1;
@@ -819,7 +922,6 @@ static int usbDev_setScanEnv( Plustek_Device *dev, pScanInfo si )
 		dev->scanning.dwFlag |= SCANDEF_QualityScan;
 	}
 
-	dev->scanning.sParam.siThreshold = si->siBrightness;
 	dev->scanning.sParam.brightness  = si->siBrightness;
 	dev->scanning.sParam.contrast    = si->siContrast;
 
@@ -900,9 +1002,9 @@ static int usbDev_setScanEnv( Plustek_Device *dev, pScanInfo si )
 	if( dev->scanning.sParam.bSource == SOURCE_ADF ) {
 
 		if( dev->scanning.dwFlag & SCANDEF_ContinuousScan )
-			fLastScanIsAdf = SANE_TRUE;
+			dev->usbDev.fLastScanIsAdf = SANE_TRUE;
 		else
-			fLastScanIsAdf = SANE_FALSE;
+			dev->usbDev.fLastScanIsAdf = SANE_FALSE;
 	}
 
     return 0;
@@ -932,47 +1034,41 @@ static int usbDev_stopScan( Plustek_Device *dev )
  */
 static int usbDev_startScan( Plustek_Device *dev )
 {
-	pScanDef   scanning = &dev->scanning;
-	static int iSkipLinesForADF = 0;
-
+	ScanDef *scan = &dev->scanning;
 	DBG( _DBG_INFO, "usbDev_startScan()\n" );
 
 /* HEINER: Preview currently not working correctly */
 #if 0
-	if( scanning->dwFlag & SCANDEF_QualityScan )
-		a_bRegs[0x0a] = 0;
+	if( scan->dwFlag & SCANDEF_QualityScan )
+		dev->usbDev.a_bRegs[0x0a] = 0;
 	else
-		a_bRegs[0x0a] = 1;
+		dev->usbDev.a_bRegs[0x0a] = 1;
 #endif
-	a_bRegs[0x0a] = 0;
+	dev->usbDev.a_bRegs[0x0a] = 0;
 
-	/* Allocate shading buffer */
-	if((dev->scanning.dwFlag & SCANDEF_Adf) &&
-		(dev->scanning.dwFlag & SCANDEF_ContinuousScan)) {
-		dev->scanning.fCalibrated = SANE_TRUE;
+	if((scan->dwFlag & SCANDEF_Adf) && (scan->dwFlag & SCANDEF_ContinuousScan)) {
+		scan->fCalibrated = SANE_TRUE;
 	} else {
-
-		dev->scanning.fCalibrated = SANE_FALSE;
-		iSkipLinesForADF = 0;
+		scan->fCalibrated = SANE_FALSE;
 	}
 
-	scanning->sParam.PhyDpi.x = usb_SetAsicDpiX(dev,scanning->sParam.UserDpi.x);
-	scanning->sParam.PhyDpi.y = usb_SetAsicDpiY(dev,scanning->sParam.UserDpi.y);
-	scanning->pScanBuffer = (u_char*)malloc( _SCANBUF_SIZE );
+	scan->sParam.PhyDpi.x = usb_SetAsicDpiX(dev,scan->sParam.UserDpi.x);
+	scan->sParam.PhyDpi.y = usb_SetAsicDpiY(dev,scan->sParam.UserDpi.y);
 
-	if( NULL != scanning->pScanBuffer ) {
+	/* Allocate shading/scan buffer */
+	scan->pScanBuffer = (u_char*)malloc( _SCANBUF_SIZE );
 
-		scanning->dwFlag |= SCANFLAG_StartScan;
-		usb_LampOn( dev, SANE_TRUE, SANE_TRUE );
+	if( scan->pScanBuffer == NULL )
+		return _E_ALLOC;
 
-		m_fStart    = m_fFirst = SANE_TRUE;
-		m_fAutoPark =
-				(scanning->dwFlag & SCANFLAG_StillModule)?SANE_FALSE:SANE_TRUE;
-		
-		usb_StopLampTimer( dev );
-		return 0;
-	}
-	return _E_ALLOC;
+	scan->dwFlag |= SCANFLAG_StartScan;
+	usb_LampOn( dev, SANE_TRUE, SANE_TRUE );
+
+	m_fStart    = m_fFirst = SANE_TRUE;
+	m_fAutoPark = (scan->dwFlag&SCANFLAG_StillModule)?SANE_FALSE:SANE_TRUE;
+
+	usb_StopLampTimer( dev );
+	return 0;
 }
 
 /**
@@ -984,9 +1080,9 @@ static int usbDev_Prepare( Plustek_Device *dev, SANE_Byte *buf )
 {
 	int       result;
 	SANE_Bool use_alt_cal = SANE_FALSE;
-	pScanDef  scanning    = &dev->scanning;
-	pDCapsDef scaps       = &dev->usbDev.Caps;
-	pHWDef    hw          = &dev->usbDev.HwSetting;
+	ScanDef  *scan        = &dev->scanning;
+	DCapsDef *scaps       = &dev->usbDev.Caps;
+	HWDef    *hw          = &dev->usbDev.HwSetting;
 
 	DBG( _DBG_INFO, "usbDev_PrepareScan()\n" );
 
@@ -1028,17 +1124,16 @@ static int usbDev_Prepare( Plustek_Device *dev, SANE_Byte *buf )
 
 	DBG( _DBG_INFO, "calibration done.\n" );
 
-	if( !( scanning->dwFlag & SCANFLAG_Scanning )) {
+	if( !( scan->dwFlag & SCANFLAG_Scanning )) {
 
 		usleep( 10 * 1000 );
 
-		if( !usb_SetScanParameters( dev, &scanning->sParam )) {
+		if( !usb_SetScanParameters( dev, &scan->sParam )) {
 			DBG( _DBG_ERROR, "Setting Scan Parameters failed!\n" );
 			return 0;
 		}
 
-		/*
-		 * if we bypass the calibration step, we wait on lamp warmup here...
+		/* if we bypass the calibration step, we wait on lamp warmup here...
 		 */
 		if( scaps->workaroundFlag & _WAF_BYPASS_CALIBRATION ) {
     		if( !usb_Wait4Warmup( dev )) {
@@ -1047,211 +1142,189 @@ static int usbDev_Prepare( Plustek_Device *dev, SANE_Byte *buf )
 			}
 		}
 
-		scanning->pbScanBufBegin = scanning->pScanBuffer;
+		scan->pbScanBufBegin = scan->pScanBuffer;
 
 		if((dev->caps.dwFlag & SFLAG_ADF) && (scaps->OpticDpi.x == 600))
-			scanning->dwLinesScanBuf = 8;
+			scan->dwLinesScanBuf = 8;
 		else
-			scanning->dwLinesScanBuf = 32;
+			scan->dwLinesScanBuf = 32;
 
-		/* gives faster feedback to the frontend ! */
-		scanning->dwLinesScanBuf = 2;
+		scan->dwBytesScanBuf     = scan->dwLinesScanBuf *
+		                           scan->sParam.Size.dwPhyBytes;
 
-		scanning->dwBytesScanBuf     = scanning->dwLinesScanBuf *
-									   scanning->sParam.Size.dwPhyBytes;
-
-		scanning->dwNumberOfScanBufs = _SCANBUF_SIZE /
-									   scanning->dwBytesScanBuf;
-		scanning->dwLinesPerScanBufs = scanning->dwNumberOfScanBufs *
-									   scanning->dwLinesScanBuf;
-		scanning->pbScanBufEnd       = scanning->pbScanBufBegin +
-									   scanning->dwLinesPerScanBufs *
-									   scanning->sParam.Size.dwPhyBytes;
-		scanning->dwRedShift   = 0;
-		scanning->dwBlueShift  = 0;
-		scanning->dwGreenShift = 0;
+		scan->dwNumberOfScanBufs = _SCANBUF_SIZE / scan->dwBytesScanBuf;
+		scan->dwLinesPerScanBufs = scan->dwNumberOfScanBufs *
+		                           scan->dwLinesScanBuf;
+		scan->pbScanBufEnd       = scan->pbScanBufBegin +
+		                           scan->dwLinesPerScanBufs *
+		                           scan->sParam.Size.dwPhyBytes;
+		scan->dwRedShift   = 0;
+		scan->dwBlueShift  = 0;
+		scan->dwGreenShift = 0;
 
 		/* CCD scanner */
-		if( scanning->sParam.bChannels == 3 ) {
+		if( scan->sParam.bChannels == 3 ) {
 
-			scanning->dwLinesDiscard = (u_long)scaps->bSensorDistance *
-			                     scanning->sParam.PhyDpi.y / scaps->OpticDpi.y;
+			scan->dwLinesDiscard = (u_long)scaps->bSensorDistance *
+			                       scan->sParam.PhyDpi.y / scaps->OpticDpi.y;
 
 			switch( scaps->bSensorOrder ) {
 
 			case SENSORORDER_rgb:
-				scanning->Red.pb   = scanning->pbScanBufBegin;
-				scanning->Green.pb = scanning->pbScanBufBegin +
-				                     scanning->dwLinesDiscard *
-				                     scanning->sParam.Size.dwPhyBytes;
-				scanning->Blue.pb  = scanning->pbScanBufBegin +
-				                     scanning->dwLinesDiscard *
-				                     scanning->sParam.Size.dwPhyBytes * 2UL;
+				scan->Red.pb   = scan->pbScanBufBegin;
+				scan->Green.pb = scan->pbScanBufBegin + scan->dwLinesDiscard *
+				                 scan->sParam.Size.dwPhyBytes;
+				scan->Blue.pb  = scan->pbScanBufBegin + scan->dwLinesDiscard *
+				                 scan->sParam.Size.dwPhyBytes * 2UL;
 				break;
 
 			case SENSORORDER_rbg:
-				scanning->Red.pb   = scanning->pbScanBufBegin;
-				scanning->Blue.pb  = scanning->pbScanBufBegin +
-				                     scanning->dwLinesDiscard *
-				                     scanning->sParam.Size.dwPhyBytes;
-				scanning->Green.pb = scanning->pbScanBufBegin +
-				                     scanning->dwLinesDiscard *
-				                     scanning->sParam.Size.dwPhyBytes * 2UL;
+				scan->Red.pb   = scan->pbScanBufBegin;
+				scan->Blue.pb  = scan->pbScanBufBegin + scan->dwLinesDiscard *
+				                 scan->sParam.Size.dwPhyBytes;
+				scan->Green.pb = scan->pbScanBufBegin + scan->dwLinesDiscard *
+				                 scan->sParam.Size.dwPhyBytes * 2UL;
 				break;
 
 			case SENSORORDER_gbr:
-				scanning->Green.pb = scanning->pbScanBufBegin;
-				scanning->Blue.pb  = scanning->pbScanBufBegin +
-				                     scanning->dwLinesDiscard *
-				                     scanning->sParam.Size.dwPhyBytes;
-				scanning->Red.pb   = scanning->pbScanBufBegin +
-				                     scanning->dwLinesDiscard *
-				                     scanning->sParam.Size.dwPhyBytes * 2UL;
+				scan->Green.pb = scan->pbScanBufBegin;
+				scan->Blue.pb  = scan->pbScanBufBegin + scan->dwLinesDiscard *
+				                 scan->sParam.Size.dwPhyBytes;
+				scan->Red.pb   = scan->pbScanBufBegin + scan->dwLinesDiscard *
+				                 scan->sParam.Size.dwPhyBytes * 2UL;
 				break;
 
 			case SENSORORDER_grb:
-				scanning->Green.pb = scanning->pbScanBufBegin;
-				scanning->Red.pb   = scanning->pbScanBufBegin +
-					                 scanning->dwLinesDiscard *
-				                     scanning->sParam.Size.dwPhyBytes;
-   				scanning->Blue.pb  = scanning->pbScanBufBegin +
-   					                 scanning->dwLinesDiscard *
-                                        scanning->sParam.Size.dwPhyBytes * 2UL;
-   				break;
+				scan->Green.pb = scan->pbScanBufBegin;
+				scan->Red.pb   = scan->pbScanBufBegin + scan->dwLinesDiscard *
+				                 scan->sParam.Size.dwPhyBytes;
+				scan->Blue.pb  = scan->pbScanBufBegin + scan->dwLinesDiscard *
+                                 scan->sParam.Size.dwPhyBytes * 2UL;
+				break;
 
-   			case SENSORORDER_brg:
-   				scanning->Blue.pb  = scanning->pbScanBufBegin;
-   				scanning->Red.pb   = scanning->pbScanBufBegin +
-   					                 scanning->dwLinesDiscard *
-                                        scanning->sParam.Size.dwPhyBytes;
-   				scanning->Green.pb = scanning->pbScanBufBegin +
-   					                 scanning->dwLinesDiscard *
-                                        scanning->sParam.Size.dwPhyBytes * 2UL;
-   				break;
+			case SENSORORDER_brg:
+				scan->Blue.pb  = scan->pbScanBufBegin;
+				scan->Red.pb   = scan->pbScanBufBegin + scan->dwLinesDiscard *
+				                 scan->sParam.Size.dwPhyBytes;
+				scan->Green.pb = scan->pbScanBufBegin + scan->dwLinesDiscard *
+				                 scan->sParam.Size.dwPhyBytes * 2UL;
+				break;
 
-   			case SENSORORDER_bgr:
-   				scanning->Blue.pb  = scanning->pbScanBufBegin;
-   				scanning->Green.pb = scanning->pbScanBufBegin +
-   					                 scanning->dwLinesDiscard *
-                                        scanning->sParam.Size.dwPhyBytes;
-   				scanning->Red.pb   = scanning->pbScanBufBegin +
-   					                 scanning->dwLinesDiscard *
-                                       scanning->sParam.Size.dwPhyBytes * 2UL;
-   			}
+			case SENSORORDER_bgr:
+				scan->Blue.pb  = scan->pbScanBufBegin;
+				scan->Green.pb = scan->pbScanBufBegin + scan->dwLinesDiscard *
+				                 scan->sParam.Size.dwPhyBytes;
+				scan->Red.pb   = scan->pbScanBufBegin + scan->dwLinesDiscard *
+				                 scan->sParam.Size.dwPhyBytes * 2UL;
+			}
 
-   			/* double it for last channel */
-   			scanning->dwLinesDiscard <<= 1;
-   			scanning->dwGreenShift = (7UL+scanning->sParam.bBitDepth) >> 3;
-   			scanning->Green.pb += scanning->dwGreenShift;
-   			scanning->Blue.pb  += (scanning->dwGreenShift * 2);
+			/* double it for last channel */
+			scan->dwLinesDiscard <<= 1;
+			scan->dwGreenShift = (7UL+scan->sParam.bBitDepth) >> 3;
+			scan->Green.pb += scan->dwGreenShift;
+			scan->Blue.pb  += (scan->dwGreenShift * 2);
 
-   			if( scanning->dwFlag & SCANFLAG_bgr) {
+			if( scan->dwFlag & SCANFLAG_bgr) {
 
-   				u_char *pb = scanning->Blue.pb;
+				u_char *pb = scan->Blue.pb;
 
-   				scanning->Blue.pb = scanning->Red.pb;
-   				scanning->Red.pb  = pb;
-   				scanning->dwBlueShift = 0;
-   				scanning->dwRedShift  = scanning->dwGreenShift << 1;
-   			} else {
-   				scanning->dwRedShift  = 0;
-   				scanning->dwBlueShift = scanning->dwGreenShift << 1;
-   			}
+				scan->Blue.pb = scan->Red.pb;
+				scan->Red.pb  = pb;
+				scan->dwBlueShift = 0;
+				scan->dwRedShift  = scan->dwGreenShift << 1;
+			} else {
+				scan->dwRedShift  = 0;
+				scan->dwBlueShift = scan->dwGreenShift << 1;
+			}
 
-   		} else {
+		} else {
 
-   			/* this might be simple gray operation or AFE 1 channel op */
-   			scanning->dwLinesDiscard = 0;
-   			scanning->Green.pb       = scanning->pbScanBufBegin;
+			/* this might be a simple gray operation or AFE 1 channel op */
+			scan->dwLinesDiscard = 0;
+			scan->Green.pb       = scan->pbScanBufBegin;
 
-   			if(( scanning->sParam.bDataType == SCANDATATYPE_Color ) &&
-   			   ( hw->bReg_0x26 & _ONE_CH_COLOR )) {
+			if(( scan->sParam.bDataType == SCANDATATYPE_Color ) &&
+			   ( hw->bReg_0x26 & _ONE_CH_COLOR )) {
 
-            	u_long len = scanning->sParam.Size.dwPhyBytes / 3;
+				u_long len = scan->sParam.Size.dwPhyBytes / 3;
 
-   				scanning->Red.pb   = scanning->pbScanBufBegin;
-   				scanning->Green.pb = scanning->pbScanBufBegin + len;
-   				scanning->Blue.pb  = scanning->pbScanBufBegin + len * 2UL;
-       		}
-   		}
+				scan->Red.pb   = scan->pbScanBufBegin;
+				scan->Green.pb = scan->pbScanBufBegin + len;
+				scan->Blue.pb  = scan->pbScanBufBegin + len * 2UL;
+			}
+		}
 
 		/* set a funtion to process the RAW data... */
 		usb_GetImageProc( dev );
 
-		scanning->bLinesToSkip = (u_char)(scanning->sParam.PhyDpi.y / 50);
-		if( scanning->sParam.bSource == SOURCE_ADF )
-			scanning->dwFlag |= SCANFLAG_StillModule;
+		scan->bLinesToSkip = (u_char)(scan->sParam.PhyDpi.y / 50);
+		if( scan->sParam.bSource == SOURCE_ADF )
+			scan->dwFlag |= SCANFLAG_StillModule;
 
-		DBG( _DBG_INFO, "* scanning->dwFlag=0x%08lx\n", scanning->dwFlag );
+		DBG( _DBG_INFO, "* scan->dwFlag=0x%08lx\n", scan->dwFlag );
 
 		if( !usb_ScanBegin( dev,
-			(scanning->dwFlag&SCANFLAG_StillModule) ? SANE_FALSE:SANE_TRUE)) {
+			(scan->dwFlag&SCANFLAG_StillModule) ? SANE_FALSE:SANE_TRUE)) {
 	
 			return _E_INTERNAL;
 		}
 		
-  		scanning->dwFlag |= SCANFLAG_Scanning;
+		scan->dwFlag |= SCANFLAG_Scanning;
 
-		if( scanning->sParam.UserDpi.y != scanning->sParam.PhyDpi.y ) {
+		if( scan->sParam.UserDpi.y != scan->sParam.PhyDpi.y ) {
 		
-			if( scanning->sParam.UserDpi.y < scanning->sParam.PhyDpi.y ) {
+			if( scan->sParam.UserDpi.y < scan->sParam.PhyDpi.y ) {
 			
-				scanning->wSumY = scanning->sParam.PhyDpi.y -
-								  scanning->sParam.UserDpi.y;
-				scanning->dwFlag |= SCANFLAG_SampleY;
+				scan->wSumY = scan->sParam.PhyDpi.y - scan->sParam.UserDpi.y;
+				scan->dwFlag |= SCANFLAG_SampleY;
 				DBG( _DBG_INFO, "SampleY Flag set (%u != %u, wSumY=%u)\n",
-								scanning->sParam.UserDpi.y,
-								scanning->sParam.PhyDpi.y, scanning->wSumY );
+				  scan->sParam.UserDpi.y, scan->sParam.PhyDpi.y, scan->wSumY );
 			}
 		}
 	}
 
-	dumpPicInit( &scanning->sParam, "plustek-pic.raw" );
-	
+	dumpPicInit( &scan->sParam, "plustek-pic.raw" );
+
 	/* here the NT driver uses an extra reading thread...
-     * as the SANE stuff already forked the driver to read data, I think
-     * we should only read data by using a function...
+	 * as the SANE stuff already forked the driver to read data, I think
+	 * we should only read data by using a function...
      */
-	scanning->dwLinesUser = scanning->sParam.Size.dwLines; 
-	if( !scanning->dwLinesUser )
+	scan->dwLinesUser = scan->sParam.Size.dwLines; 
+	if( !scan->dwLinesUser )
 		return _E_BUFFER_TOO_SMALL;
 
-	if( !scanning->sParam.Size.dwLines )
-		return _E_NODATA;
-	
-	if( scanning->sParam.Size.dwLines < scanning->dwLinesUser )
-		scanning->dwLinesUser = scanning->sParam.Size.dwLines;
+	if( scan->sParam.Size.dwLines < scan->dwLinesUser )
+		scan->dwLinesUser = scan->sParam.Size.dwLines;
 
-	scanning->sParam.Size.dwLines -= scanning->dwLinesUser;
-	if( scanning->dwFlag & SCANFLAG_BottomUp )
-		scanning->UserBuf.pb = buf + (scanning->dwLinesUser - 1) *
-		                                    scanning->dwBytesLine;
+	scan->sParam.Size.dwLines -= scan->dwLinesUser;
+	if( scan->dwFlag & SCANFLAG_BottomUp )
+		scan->UserBuf.pb = buf + (scan->dwLinesUser - 1) * scan->dwBytesLine;
 	else
-		scanning->UserBuf.pb = buf;
+		scan->UserBuf.pb = buf;
 
 	DBG(_DBG_INFO,"Reading the data now!\n" );
-	DBG(_DBG_INFO,"PhyDpi.x         = %u\n", scanning->sParam.PhyDpi.x  );	
-	DBG(_DBG_INFO,"PhyDpi.y         = %u\n", scanning->sParam.PhyDpi.y  );	
-	DBG(_DBG_INFO,"UserDpi.x        = %u\n", scanning->sParam.UserDpi.x );	
-	DBG(_DBG_INFO,"UserDpi.y        = %u\n", scanning->sParam.UserDpi.y );	
-	DBG(_DBG_INFO,"NumberOfScanBufs = %lu\n",scanning->dwNumberOfScanBufs);
-	DBG(_DBG_INFO,"LinesPerScanBufs = %lu\n",scanning->dwLinesPerScanBufs);
-	DBG(_DBG_INFO,"dwPhyBytes       = %lu\n",scanning->sParam.Size.dwPhyBytes);
-	DBG(_DBG_INFO,"dwPhyPixels      = %lu\n",scanning->sParam.Size.dwPhyPixels);
-	DBG(_DBG_INFO,"dwTotalBytes     = %lu\n",scanning->sParam.Size.dwTotalBytes);
-	DBG(_DBG_INFO,"dwPixels         = %lu\n",scanning->sParam.Size.dwPixels);
-	DBG(_DBG_INFO,"dwBytes          = %lu\n",scanning->sParam.Size.dwBytes);
-	DBG(_DBG_INFO,"dwValidPixels    = %lu\n",scanning->sParam.Size.dwValidPixels);
-	DBG(_DBG_INFO,"dwBytesScanBuf   = %lu\n",scanning->dwBytesScanBuf );
-	DBG(_DBG_INFO,"dwLinesDiscard   = %lu\n",scanning->dwLinesDiscard );
-	DBG(_DBG_INFO,"dwLinesToSkip    = %u\n", scanning->bLinesToSkip );
-	DBG(_DBG_INFO,"dwLinesUser      = %lu\n",scanning->dwLinesUser  );
-	DBG(_DBG_INFO,"dwBytesLine      = %lu\n",scanning->dwBytesLine  );
+	DBG(_DBG_INFO,"PhyDpi.x         = %u\n", scan->sParam.PhyDpi.x  );	
+	DBG(_DBG_INFO,"PhyDpi.y         = %u\n", scan->sParam.PhyDpi.y  );	
+	DBG(_DBG_INFO,"UserDpi.x        = %u\n", scan->sParam.UserDpi.x );	
+	DBG(_DBG_INFO,"UserDpi.y        = %u\n", scan->sParam.UserDpi.y );	
+	DBG(_DBG_INFO,"NumberOfScanBufs = %lu\n",scan->dwNumberOfScanBufs);
+	DBG(_DBG_INFO,"LinesPerScanBufs = %lu\n",scan->dwLinesPerScanBufs);
+	DBG(_DBG_INFO,"dwPhyBytes       = %lu\n",scan->sParam.Size.dwPhyBytes);
+	DBG(_DBG_INFO,"dwPhyPixels      = %lu\n",scan->sParam.Size.dwPhyPixels);
+	DBG(_DBG_INFO,"dwTotalBytes     = %lu\n",scan->sParam.Size.dwTotalBytes);
+	DBG(_DBG_INFO,"dwPixels         = %lu\n",scan->sParam.Size.dwPixels);
+	DBG(_DBG_INFO,"dwBytes          = %lu\n",scan->sParam.Size.dwBytes);
+	DBG(_DBG_INFO,"dwValidPixels    = %lu\n",scan->sParam.Size.dwValidPixels);
+	DBG(_DBG_INFO,"dwBytesScanBuf   = %lu\n",scan->dwBytesScanBuf );
+	DBG(_DBG_INFO,"dwLinesDiscard   = %lu\n",scan->dwLinesDiscard );
+	DBG(_DBG_INFO,"dwLinesToSkip    = %u\n", scan->bLinesToSkip );
+	DBG(_DBG_INFO,"dwLinesUser      = %lu\n",scan->dwLinesUser  );
+	DBG(_DBG_INFO,"dwBytesLine      = %lu\n",scan->dwBytesLine  );
 
-	scanning->pbGetDataBuf = scanning->pbScanBufBegin;
+	scan->pbGetDataBuf = scan->pbScanBufBegin;
 
-	scanning->dwLinesToProcess = usb_ReadData( dev  );
-	if( 0 == scanning->dwLinesToProcess )
+	scan->dwLinesToProcess = usb_ReadData( dev  );
+	if( 0 == scan->dwLinesToProcess )
 		return _E_DATAREAD;
 
 	return 0;
@@ -1261,76 +1334,72 @@ static int usbDev_Prepare( Plustek_Device *dev, SANE_Byte *buf )
  */
 static int usbDev_ReadLine( Plustek_Device *dev )
 {
-	int       wrap;
-	u_long    cur;
-	pScanDef  scanning = &dev->scanning;
-	pHWDef    hw       = &dev->usbDev.HwSetting;
+	int      wrap;
+	u_long   cur;
+	ScanDef *scan = &dev->scanning;
+	HWDef   *hw   = &dev->usbDev.HwSetting;
 
-	cur = scanning->dwLinesUser;
+	cur = scan->dwLinesUser;
 
 	/* we stay within this sample loop until one line has been processed for
 	 * the user...
 	 */
-	while( cur == scanning->dwLinesUser ) {
+	while( cur == scan->dwLinesUser ) {
 
 		if( usb_IsEscPressed()) {
 			DBG( _DBG_INFO, "readLine() - Cancel detected...\n" );
 			return _E_ABORT;
 		}
 
-		if( !(scanning->dwFlag & SCANFLAG_SampleY))	{
+		if( !(scan->dwFlag & SCANFLAG_SampleY))	{
 
-			scanning->pfnProcess( dev );
+			scan->pfnProcess( dev );
 
 			/* Adjust user buffer pointer */
-			scanning->UserBuf.pb += scanning->lBufAdjust;
-			scanning->dwLinesUser--;
+			scan->UserBuf.pb += scan->lBufAdjust;
+			scan->dwLinesUser--;
 
 		} else {
 
-			scanning->wSumY += scanning->sParam.UserDpi.y;
+			scan->wSumY += scan->sParam.UserDpi.y;
 
-			if( scanning->wSumY >= scanning->sParam.PhyDpi.y ) {
-				scanning->wSumY -= scanning->sParam.PhyDpi.y;
+			if( scan->wSumY >= scan->sParam.PhyDpi.y ) {
+				scan->wSumY -= scan->sParam.PhyDpi.y;
 
-				scanning->pfnProcess( dev );
+				scan->pfnProcess( dev );
 
 				/* Adjust user buffer pointer */
-				scanning->UserBuf.pb += scanning->lBufAdjust;
-				scanning->dwLinesUser--;
+				scan->UserBuf.pb += scan->lBufAdjust;
+				scan->dwLinesUser--;
 			}
 		}
 
 		/* Adjust get buffer pointers */
 		wrap = 0;
 
-		if( scanning->sParam.bDataType == SCANDATATYPE_Color ) {
+		if( scan->sParam.bDataType == SCANDATATYPE_Color ) {
 
-			scanning->Red.pb += scanning->sParam.Size.dwPhyBytes;
-			if( scanning->Red.pb >= scanning->pbScanBufEnd ) {
-				scanning->Red.pb = scanning->pbScanBufBegin +
-								   scanning->dwRedShift;
+			scan->Red.pb += scan->sParam.Size.dwPhyBytes;
+			if( scan->Red.pb >= scan->pbScanBufEnd ) {
+				scan->Red.pb = scan->pbScanBufBegin + scan->dwRedShift;
 				wrap = 1;
 			}
 
-			scanning->Green.pb += scanning->sParam.Size.dwPhyBytes;
-			if( scanning->Green.pb >= scanning->pbScanBufEnd ) {
-				scanning->Green.pb = scanning->pbScanBufBegin +
-									 scanning->dwGreenShift;
+			scan->Green.pb += scan->sParam.Size.dwPhyBytes;
+			if( scan->Green.pb >= scan->pbScanBufEnd ) {
+				scan->Green.pb = scan->pbScanBufBegin + scan->dwGreenShift;
 				wrap = 1;
 			}
 
-			scanning->Blue.pb += scanning->sParam.Size.dwPhyBytes;
-			if( scanning->Blue.pb >= scanning->pbScanBufEnd ) {
-				scanning->Blue.pb = scanning->pbScanBufBegin +
-									scanning->dwBlueShift;
+			scan->Blue.pb += scan->sParam.Size.dwPhyBytes;
+			if( scan->Blue.pb >= scan->pbScanBufEnd ) {
+				scan->Blue.pb = scan->pbScanBufBegin + scan->dwBlueShift;
 				wrap = 1;
 			}
 		} else {
-			scanning->Green.pb += scanning->sParam.Size.dwPhyBytes;
-			if( scanning->Green.pb >= scanning->pbScanBufEnd )
-				scanning->Green.pb = scanning->pbScanBufBegin +
-									 scanning->dwGreenShift;
+			scan->Green.pb += scan->sParam.Size.dwPhyBytes;
+			if( scan->Green.pb >= scan->pbScanBufEnd )
+				scan->Green.pb = scan->pbScanBufBegin + scan->dwGreenShift;
 		}
 
 		/* on any wrap-around of the get pointers in one channel mode
@@ -1338,27 +1407,27 @@ static int usbDev_ReadLine( Plustek_Device *dev )
 		 */
 		if( wrap ) {
 
-			u_long len = scanning->sParam.Size.dwPhyBytes;
+			u_long len = scan->sParam.Size.dwPhyBytes;
 
 			if( hw->bReg_0x26 & _ONE_CH_COLOR ) {
 
-				if(scanning->sParam.bDataType == SCANDATATYPE_Color) {
+				if(scan->sParam.bDataType == SCANDATATYPE_Color) {
 					len /= 3;
 				}
-				scanning->Red.pb   = scanning->pbScanBufBegin;
-				scanning->Green.pb = scanning->pbScanBufBegin + len;
-				scanning->Blue.pb  = scanning->pbScanBufBegin + len * 2UL;
+				scan->Red.pb   = scan->pbScanBufBegin;
+				scan->Green.pb = scan->pbScanBufBegin + len;
+				scan->Blue.pb  = scan->pbScanBufBegin + len * 2UL;
 			}
 		}
 
 		/* line processed, check if we have to get more...
 		 */
-		scanning->dwLinesToProcess--;
+		scan->dwLinesToProcess--;
 
-		if( 0 == scanning->dwLinesToProcess ) {
+		if( 0 == scan->dwLinesToProcess ) {
 
-			scanning->dwLinesToProcess = usb_ReadData( dev );
-			if( 0 == scanning->dwLinesToProcess ) {
+			scan->dwLinesToProcess = usb_ReadData( dev );
+			if( 0 == scan->dwLinesToProcess ) {
 
 				if( usb_IsEscPressed())
 					return _E_ABORT;
