@@ -7,7 +7,7 @@
  *  @brief Main defines for the USB devices.
  *
  * Based on sources acquired from Plustek Inc.<br>
- * Copyright (C) 2001-2004 Gerhard Jaeger <gerhard@gjaeger.de>
+ * Copyright (C) 2001-2005 Gerhard Jaeger <gerhard@gjaeger.de>
  *
  * History:
  * - 0.40 - starting version of the USB support
@@ -33,6 +33,10 @@
  * - 0.48 - added DEVCAPSFLAG_LargeTPA
  *        - added _WAF_BIN_FROM_COLOR and _WAF_GRAY_FROM_COLOR
  *        - added dHighSpeed to struct HwDefault
+ * - 0.49 - added a_bRegs, fModFirstHome and fLastScanIsAdf
+ *          to struct DeviceDef
+ *        - added CRYSTAL_FREQ
+ *        - added IPCDef
  * .
  * <hr>
  * This file is part of the SANE package.
@@ -90,6 +94,9 @@
 #define kCIS1240    9
 #define kEPSON     10
 
+/** 48MHz Quartz */
+#define CRYSTAL_FREQ 48000000UL
+
 /*********************************** plustek_types.h!!! ************************/
 
 /* makes trouble with gcc3
@@ -477,6 +484,9 @@ typedef struct DeviceDef
 	u_long      dwLampOnPeriod;/**< How many seconds to keep lamp on         */
 	SANE_Bool   bLampOffOnEnd; /**< switch lamp off on end or keep cur. state*/
 	int         currentLamp;   /**< The lamp ID of the currently used lamp   */
+	SANE_Bool   fModFirstHome; /**<                                          */
+	SANE_Bool   fLastScanIsAdf;/**<                                          */
+	u_char      a_bRegs[0x80]; /**< our global register file                 */
 
 } DeviceDef, *pDeviceDef;
 
@@ -530,7 +540,6 @@ typedef struct
 	double  dMCLK;        /**< for positive & negative & Adf               */
 	short   brightness;
 	short   contrast;
-	short   siThreshold;  /**< only for B/W output                         */
 	u_char  bSource;      /**< Reflection/Positive/Negative/Adf(SOURCE_xxx)*/
 	u_char  bDataType;    /**< Bw, Gray or Color (see _SCANDATATYPE)       */
 	u_char  bBitDepth;    /**< 1/8/14                                      */
@@ -638,6 +647,15 @@ typedef struct {
 
 } ClkMotorDef, *pClkMotorDef;
 
+/** for transferring some info between child and parent after calibration
+ */
+#define _MAX_SHAD       0x4000
+#define _SHADING_BUF    (_MAX_SHAD*3)    /**< max size of the shading buffer */
+
+typedef struct {
+	u_long  transferRate;
+} IPCDef;
+
 #endif /* guard __PLUSTEK_USB_H__ */
 
 /* END PLUSTEK-USB.H ........................................................*/
