@@ -7,7 +7,7 @@
  *  @brief Scanning...
  *
  * Based on sources acquired from Plustek Inc.<br>
- * Copyright (C) 2001-2004 Gerhard Jaeger <gerhard@gjaeger.de>
+ * Copyright (C) 2001-2005 Gerhard Jaeger <gerhard@gjaeger.de>
  *
  * History:
  * - 0.40 - starting version of the USB support
@@ -24,6 +24,9 @@
  * - 0.46 - fixed problem in usb_SetScanParameters()
  * - 0.47 - no changes
  * - 0.48 - minor fixes
+ * - 0.49 - a_bRegs is now part of the device structure
+ *        - using now PhyDpi.y as selector for the motor MCLK range
+ *        - changed usb_MapDownload prototype
  * .
  * <hr>
  * This file is part of the SANE package.
@@ -68,7 +71,7 @@
 
 /** array used to get motor-settings and mclk-settings
  */
-static int dpi_ranges[] = {	75,100,150,200,300,400,600,800,1200,2400 };
+static int dpi_ranges[] = { 75,100,150,200,300,400,600,800,1200,2400 };
 
 static u_char     bMaxITA;
 
@@ -131,6 +134,7 @@ static u_short usb_SetAsicDpiX( Plustek_Device *dev, u_short xdpi )
 	u_short   res;
     pScanDef  scanning = &dev->scanning;
 	pDCapsDef scaps    = &dev->usbDev.Caps;
+	u_char   *regs     = dev->usbDev.a_bRegs;
 
 	/*
      * limit xdpi to lower value for certain devices...
@@ -148,47 +152,47 @@ static u_short usb_SetAsicDpiX( Plustek_Device *dev, u_short xdpi )
 	if (m_dHDPIDivider < 1.5)
 	{
 		m_dHDPIDivider = 1.0;
-		a_bRegs[0x09]  = 0;
+		regs[0x09]  = 0;
 	}
 	else if (m_dHDPIDivider < 2.0)
 	{
 		m_dHDPIDivider = 1.5;
-		a_bRegs[0x09]  = 1;
+		regs[0x09]  = 1;
 	}
 	else if (m_dHDPIDivider < 3.0)
 	{
 		m_dHDPIDivider = 2.0;
-		a_bRegs[0x09]  = 2;
+		regs[0x09]  = 2;
 	}
 	else if (m_dHDPIDivider < 4.0)
 	{
 		m_dHDPIDivider = 3.0;
-		a_bRegs[0x09]  = 3;
+		regs[0x09]  = 3;
 	}
 	else if (m_dHDPIDivider < 6.0)
 	{
 		m_dHDPIDivider = 4.0;
-		a_bRegs[0x09]  = 4;
+		regs[0x09]  = 4;
 	}
 	else if (m_dHDPIDivider < 8.0)
 	{
 		m_dHDPIDivider = 6.0;
-		a_bRegs[0x09]  = 5;
+		regs[0x09]  = 5;
 	}
 	else if (m_dHDPIDivider < 12.0)
 	{
 		m_dHDPIDivider = 8.0;
-		a_bRegs[0x09]  = 6;
+		regs[0x09]  = 6;
 	}
 	else
 	{
 		m_dHDPIDivider = 12.0;
-		a_bRegs[0x09]  = 7;
+		regs[0x09]  = 7;
 	}
 
 	/* adjust, if any turbo/preview mode is set, should be 0 here... */
-	if( a_bRegs[0x0a] )
-		a_bRegs[0x09] -= ((a_bRegs[0x0a] >> 2) + 2);
+	if( regs[0x0a] )
+		regs[0x09] -= ((regs[0x0a] >> 2) + 2);
 
 	DBG( _DBG_INFO2, "* HDPI: %.3f\n", m_dHDPIDivider );
 	res = (u_short)((double)scaps->OpticDpi.x / m_dHDPIDivider);
@@ -258,7 +262,8 @@ static u_short usb_SetAsicDpiY( Plustek_Device *dev, u_short ydpi )
  */
 static void usb_SetColorAndBits( Plustek_Device *dev, pScanParam pParam )
 {
-	pHWDef hw = &dev->usbDev.HwSetting;
+	pHWDef    hw   = &dev->usbDev.HwSetting;
+	u_char   *regs = dev->usbDev.a_bRegs;
 
 	/*
      * Set pixel packing, data mode and AFE operation
@@ -266,38 +271,38 @@ static void usb_SetColorAndBits( Plustek_Device *dev, pScanParam pParam )
 	switch( pParam->bDataType ) {
 		case SCANDATATYPE_Color:
 			m_bCM = 3;
-			a_bRegs[0x26] = hw->bReg_0x26 & 0x7;
+			regs[0x26] = hw->bReg_0x26 & 0x7;
 
 			/* if set to one channel color, we select the blue channel
              * as input source, this is the default, but I don't know
              * what happens, if we deselect this
              */
-			if( a_bRegs[0x26] & _ONE_CH_COLOR )
-				a_bRegs[0x26] |= (_BLUE_CH | 0x01);
+			if( regs[0x26] & _ONE_CH_COLOR )
+				regs[0x26] |= (_BLUE_CH | 0x01);
 
-			memcpy( &a_bRegs[0x0f], hw->bReg_0x0f_Color, 10 );
+			memcpy( &regs[0x0f], hw->bReg_0x0f_Color, 10 );
 			break;
 
 		case SCANDATATYPE_Gray:
 			m_bCM = 1;
-			a_bRegs[0x26] = (hw->bReg_0x26 & 0x18) | 0x04;
-			memcpy( &a_bRegs[0x0f], hw->bReg_0x0f_Mono, 10 );
+			regs[0x26] = (hw->bReg_0x26 & 0x18) | 0x04;
+			memcpy( &regs[0x0f], hw->bReg_0x0f_Mono, 10 );
 			break;
 
 		case SCANDATATYPE_BW:
 			m_bCM = 1;
-			a_bRegs[0x26] = (hw->bReg_0x26 & 0x18) | 0x04;
-			memcpy( &a_bRegs[0x0f], hw->bReg_0x0f_Mono, 10 );
+			regs[0x26] = (hw->bReg_0x26 & 0x18) | 0x04;
+			memcpy( &regs[0x0f], hw->bReg_0x0f_Mono, 10 );
 			break;
 	}
 			
-	a_bRegs[0x27] = hw->bReg_0x27;
+	regs[0x27] = hw->bReg_0x27;
 
 	if( pParam->bBitDepth > 8 ) {
-		a_bRegs[0x09] |= 0x20;         /* 14/16bit image data */
+		regs[0x09] |= 0x20;         /* 14/16bit image data */
 
 	} else if( pParam->bBitDepth == 8 ) {
-		a_bRegs[0x09] |= 0x18;        /* 8bits/per pixel */
+		regs[0x09] |= 0x18;        /* 8bits/per pixel */
 	}
 }
 
@@ -319,10 +324,11 @@ static void usb_GetScanRect( Plustek_Device *dev, pScanParam pParam )
 
 	pDCapsDef sCaps = &dev->usbDev.Caps;
 	pHWDef    hw    = &dev->usbDev.HwSetting;
+	u_char   *regs  = dev->usbDev.a_bRegs;
 
 	/* Convert pixels to physical dpi based */
 	pParam->Size.dwValidPixels = pParam->Size.dwPixels *
-								 pParam->PhyDpi.x / pParam->UserDpi.x;
+	                             pParam->PhyDpi.x / pParam->UserDpi.x;
 
 /* HEINER: check ADF stuff... */
 #if 0
@@ -402,12 +408,12 @@ static void usb_GetScanRect( Plustek_Device *dev, pScanParam pParam )
 		                                           wDataPixelStart, wLineEnd );
 	}
 
-	a_bRegs[0x22] = _HIBYTE( wDataPixelStart );
-	a_bRegs[0x23] = _LOBYTE( wDataPixelStart );
+	regs[0x22] = _HIBYTE( wDataPixelStart );
+	regs[0x23] = _LOBYTE( wDataPixelStart );
 
 	/* should match: wLineEnd-wDataPixelStart%(m_dHDPIDivider*2) = 0!! */
-	a_bRegs[0x24] = _HIBYTE( wLineEnd );
-	a_bRegs[0x25] = _LOBYTE( wLineEnd );
+	regs[0x24] = _HIBYTE( wLineEnd );
+	regs[0x25] = _LOBYTE( wLineEnd );
 
 	DBG( _DBG_INFO2, ">> End-Start=%u, HDPI=%.2f\n",
 	                                 wLineEnd-wDataPixelStart, m_dHDPIDivider);
@@ -475,8 +481,8 @@ static void usb_GetScanRect( Plustek_Device *dev, pScanParam pParam )
 
 	DBG(_DBG_INFO2,"* Full Steps to Skip at Start = 0x%04x\n",pParam->Origin.y);
 
-	a_bRegs[0x4a] = _HIBYTE( pParam->Origin.y );
-	a_bRegs[0x4b] = _LOBYTE( pParam->Origin.y );
+	regs[0x4a] = _HIBYTE( pParam->Origin.y );
+	regs[0x4b] = _LOBYTE( pParam->Origin.y );
 }
 
 /** preset scan stepsize and fastfeed stepsize
@@ -486,14 +492,15 @@ static void usb_PresetStepSize( Plustek_Device *dev, pScanParam pParam )
 	u_short ssize;
 	double  mclkdiv = pParam->dMCLK;
 	pHWDef  hw      = &dev->usbDev.HwSetting;
+	u_char *regs    = dev->usbDev.a_bRegs;
 
-	ssize = (u_short)((double)dwCrystalFrequency / ( mclkdiv * 8.0 *
+	ssize = (u_short)((double)CRYSTAL_FREQ / ( mclkdiv * 8.0 *
             (double)m_bCM * hw->dMaxMotorSpeed * 4.0 * (double)hw->wMotorDpi));
 
-	a_bRegs[0x46] = _HIBYTE( ssize );
-	a_bRegs[0x47] = _LOBYTE( ssize );
-	a_bRegs[0x48] = _HIBYTE( ssize );
-	a_bRegs[0x49] = _LOBYTE( ssize );
+	regs[0x46] = _HIBYTE( ssize );
+	regs[0x47] = _LOBYTE( ssize );
+	regs[0x48] = _HIBYTE( ssize );
+	regs[0x49] = _LOBYTE( ssize );
 
 	DBG( _DBG_INFO2, "* StepSize(Preset) = %u (0x%04x)\n", ssize, ssize );
 }
@@ -508,23 +515,24 @@ static void usb_GetDPD( Plustek_Device *dev  )
 	int    dpd;	    /* calculated dpd reg 52:53               */
 	int    st;		/* step size reg 46:47                    */
 
-	pHWDef hw = &dev->usbDev.HwSetting;
+	pHWDef  hw    = &dev->usbDev.HwSetting;
+	u_char *regs  = dev->usbDev.a_bRegs;
 
-	qtcnt = (a_bRegs [0x51] & 0x30) >> 4;	/* quarter speed count */
-	hfcnt = (a_bRegs [0x51] & 0xc0) >> 6;	/* half speed count    */
+	qtcnt = (regs[0x51] & 0x30) >> 4;	/* quarter speed count */
+	hfcnt = (regs[0x51] & 0xc0) >> 6;	/* half speed count    */
 
 	if( _LM9831 == hw->chip )
-		strev = a_bRegs [0x50] & 0x3f;		/* steps to reverse */
+		strev = regs[0x50] & 0x3f;		/* steps to reverse */
 	else /* LM9832/3 */
 	{
 		if (qtcnt == 3)
 			qtcnt = 8;
 		if (hfcnt == 3)
 			hfcnt = 8;
-		strev = a_bRegs[0x50]; /* steps to reverse */
+		strev = regs[0x50]; /* steps to reverse */
 	}
 
-	st = a_bRegs[0x46] * 256 + a_bRegs[0x47]; /* scan step size */
+	st = regs[0x46] * 256 + regs[0x47]; /* scan step size */
 
 	if (m_wLineLength == 0)
 		dpd = 0;
@@ -541,9 +549,82 @@ static void usb_GetDPD( Plustek_Device *dev  )
 	DBG( _DBG_INFO2, "* llen=%u, lineRateColor=%u, qtcnt=%u, hfcnt=%u\n",
 						  m_wLineLength, m_bLineRateColor, qtcnt, hfcnt );
 
-	a_bRegs[0x51] |= (u_char)((dpd >> 16) & 0x03);
-	a_bRegs[0x52] = (u_char)(dpd >> 8);
-	a_bRegs[0x53] = (u_char)(dpd & 0xFF);
+	regs[0x51] |= (u_char)((dpd >> 16) & 0x03);
+	regs[0x52] = (u_char)(dpd >> 8);
+	regs[0x53] = (u_char)(dpd & 0xFF);
+}
+
+#define MCLKDIV_SCALING 2
+#define _MIN(a,b) ((a) < (b) ? (a) : (b))
+#define _MAX(a,b) ((a) > (b) ? (a) : (b))
+
+/**
+ */
+static int usb_GetMCLKDiv( Plustek_Device *dev )
+{
+	int     j, pixelbits, pixelsperline, r;
+	int     minmclk, maxmclk, mclkdiv;
+	double  hdpi, min_int_time;
+	u_char *regs = dev->usbDev.a_bRegs;
+	HWDef  *hw   = &dev->usbDev.HwSetting;
+
+	DBG( _DBG_INFO, "usb_GetMCLKDiv()\n" );
+
+	r = 8; /* line rate */
+	if ((regs[0x26] & 7) == 0) 
+		r = 24; /* pixel rate */
+
+	/*use high or low res min integration time */
+	min_int_time = ((regs[0x9]&7) > 2 ? hw->dMinIntegrationTimeLowres:
+	                                    hw->dMinIntegrationTimeHighres);
+
+	minmclk = (int)ceil((double) MCLKDIV_SCALING * CRYSTAL_FREQ *
+	                       min_int_time /((double)1000. * r * m_wLineLength));
+	minmclk = _MAX(minmclk,MCLKDIV_SCALING);
+
+	maxmclk = (int)(32.5*MCLKDIV_SCALING + .5); 
+
+	DBG(_DBG_INFO2,"- lower mclkdiv limit=%f\n",(double)minmclk/MCLKDIV_SCALING);
+	DBG(_DBG_INFO2,"- upper mclkdiv limit=%f\n",(double)maxmclk/MCLKDIV_SCALING);
+
+	/* get the bits per pixel */
+	switch (regs[0x9] & 0x38) {
+		case 0:	    pixelbits=1; break;
+		case 0x8:   pixelbits=2; break;
+		case 0x10:  pixelbits=4; break;
+		case 0x18:  pixelbits=8; break;
+		default:	pixelbits=16;break;
+	}
+
+	/* compute the horizontal dpi (pixels per inch) */
+	j    = regs[0x9] & 0x7;
+	hdpi = ((j&1)*.5+1)*(j&2?2:1)*(j&4?4:1);	
+
+	pixelsperline = (int)((256*regs[0x24]+regs[0x25]-256*regs[0x22]-regs[0x23])
+	                       *pixelbits/(hdpi * 8));
+	mclkdiv = (int)ceil((double)MCLKDIV_SCALING * pixelsperline * CRYSTAL_FREQ
+	                /((double) 8. * m_wLineLength * dev->transferRate));
+
+	DBG( _DBG_INFO2, "- hdpi          = %.3f\n", hdpi );
+	DBG( _DBG_INFO2, "- pixelbits     = %u\n", pixelbits );
+	DBG( _DBG_INFO2, "- pixelsperline = %u\n", pixelsperline );
+	DBG( _DBG_INFO2, "- linelen       = %u\n", m_wLineLength );
+	DBG( _DBG_INFO2, "- transferrate  = %lu\n", dev->transferRate );
+	DBG( _DBG_INFO2, "- MCLK Divider  = %u\n", mclkdiv/MCLKDIV_SCALING );
+
+	mclkdiv = _MAX(mclkdiv,minmclk);
+	mclkdiv = _MIN(mclkdiv,maxmclk);
+	DBG( _DBG_INFO2, "- Current MCLK Divider = %u\n", mclkdiv/MCLKDIV_SCALING );
+
+	if( dev->transferRate == 2000000 ) {
+		while (mclkdiv * hdpi < 6.*MCLKDIV_SCALING) {
+			mclkdiv++;
+		}
+		DBG( _DBG_INFO2, "- HIGHSPEED MCLK Divider = %u\n", 
+		     mclkdiv/MCLKDIV_SCALING );
+	}
+
+	return mclkdiv;
 }
 
 /** Plusteks' poor-man MCLK calculation...
@@ -557,9 +638,15 @@ static double usb_GetMCLKDivider( Plustek_Device *dev, pScanParam pParam )
 
 	pDCapsDef sCaps = &dev->usbDev.Caps;
 	pHWDef    hw    = &dev->usbDev.HwSetting;
+	u_char   *regs  = dev->usbDev.a_bRegs;
 
 	DBG( _DBG_INFO, "usb_GetMCLKDivider()\n" );
 
+	if( dev->transferRate == 2000000 ) {
+		int mclkdiv = usb_GetMCLKDiv(dev);
+		pParam->dMCLK = (double)mclkdiv/MCLKDIV_SCALING;
+	}
+
 	m_dMCLKDivider = pParam->dMCLK;
 
 	if (m_dHDPIDivider*m_dMCLKDivider >= 5.3/*6*/)
@@ -568,7 +655,9 @@ static double usb_GetMCLKDivider( Plustek_Device *dev, pScanParam pParam )
 		m_bIntTimeAdjust = ceil( 5.3/*6.0*/ / (m_dHDPIDivider*m_dMCLKDivider));
 
 	if( pParam->bCalibration == PARAM_Scan ) {
-  
+
+		usb_GetMCLKDiv(dev);
+
 		/*  Compare Integration with USB speed to find the best ITA value */
 		if( pParam->bBitDepth > 8 )	{
 
@@ -584,9 +673,9 @@ static double usb_GetMCLKDivider( Plustek_Device *dev, pScanParam pParam )
 			}
 
 			if(((hw->motorModel == MODEL_HP) && (sCaps->bCCD == kNECSLIM))/* ||
-			 	( a_bRegs[0x26] & _ONE_CH_COLOR )*/) {
+			 	( regs[0x26] & _ONE_CH_COLOR )*/) {
 
-       			bMaxITA = (u_char)floor((m_dMCLKDivider + 1) / 2.0);
+				bMaxITA = (u_char)floor((m_dMCLKDivider + 1) / 2.0);
 
 				DBG( _DBG_INFO2, "* MaxITA (HP) = %u\n", bMaxITA );
 				if( m_bIntTimeAdjust > bMaxITA ) {
@@ -597,10 +686,10 @@ static double usb_GetMCLKDivider( Plustek_Device *dev, pScanParam pParam )
 		}
 	}
 	DBG( _DBG_INFO2, "* Integration Time Adjust = %u (HDPI=%.3f,MCLKD=%.3f)\n",
-							m_bIntTimeAdjust, m_dHDPIDivider, m_dMCLKDivider );
+	                    m_bIntTimeAdjust, m_dHDPIDivider, m_dMCLKDivider );
 
-	a_bRegs[0x08] = (u_char)((m_dMCLKDivider - 1) * 2);
-	a_bRegs[0x19] = m_bIntTimeAdjust;
+	regs[0x08] = (u_char)((m_dMCLKDivider - 1) * 2);
+	regs[0x19] = m_bIntTimeAdjust;
 
 	if( m_bIntTimeAdjust != 0 ) {
 
@@ -609,11 +698,11 @@ static double usb_GetMCLKDivider( Plustek_Device *dev, pScanParam pParam )
 		if( m_wStepSize < 2 )
 			m_wStepSize = 2;
 
-		a_bRegs[0x46] = _HIBYTE(m_wStepSize);
-		a_bRegs[0x47] = _LOBYTE(m_wStepSize);
+		regs[0x46] = _HIBYTE(m_wStepSize);
+		regs[0x47] = _LOBYTE(m_wStepSize);
 
 		DBG( _DBG_INFO2, "* Stepsize = %u, 0x46=0x%02x 0x47=0x%02x\n",
-		                   m_wStepSize, a_bRegs[0x46], a_bRegs[0x47] );
+		                   m_wStepSize, regs[0x46], regs[0x47] );
 	    usb_GetDPD( dev );
 	}
 	
@@ -621,21 +710,21 @@ static double usb_GetMCLKDivider( Plustek_Device *dev, pScanParam pParam )
      * high lamp PWM, use equation 4
      */
 	dMaxIntegrationTime = hw->dIntegrationTimeHighLamp;
-	dMaxMCLKDivider = (double)dwCrystalFrequency * dMaxIntegrationTime /
+	dMaxMCLKDivider = (double)CRYSTAL_FREQ * dMaxIntegrationTime /
 					   (1000 * 8 * m_bCM * m_wLineLength);
 
 	/* Determine lamp PWM setting */
 	if( m_dMCLKDivider > dMaxMCLKDivider ) {
 
 		DBG( _DBG_INFO2, "* Setting GreenPWMDutyCycleLow\n" );
-		a_bRegs[0x2a] = _HIBYTE( hw->wGreenPWMDutyCycleLow );
-		a_bRegs[0x2b] = _LOBYTE( hw->wGreenPWMDutyCycleLow );
+		regs[0x2a] = _HIBYTE( hw->wGreenPWMDutyCycleLow );
+		regs[0x2b] = _LOBYTE( hw->wGreenPWMDutyCycleLow );
 
 	} else {
 
 		DBG( _DBG_INFO2, "* Setting GreenPWMDutyCycleHigh\n" );
-		a_bRegs[0x2a] = _HIBYTE( hw->wGreenPWMDutyCycleHigh );
-		a_bRegs[0x2b] = _LOBYTE( hw->wGreenPWMDutyCycleHigh );
+		regs[0x2a] = _HIBYTE( hw->wGreenPWMDutyCycleHigh );
+		regs[0x2b] = _LOBYTE( hw->wGreenPWMDutyCycleHigh );
 	}
 
 	DBG( _DBG_INFO2, "* Current MCLK Divider = %f\n", m_dMCLKDivider );
@@ -646,7 +735,8 @@ static double usb_GetMCLKDivider( Plustek_Device *dev, pScanParam pParam )
  */
 static void usb_GetStepSize( Plustek_Device *dev, pScanParam pParam )
 {
-	pHWDef hw = &dev->usbDev.HwSetting;
+	pHWDef  hw   = &dev->usbDev.HwSetting;
+	u_char *regs = dev->usbDev.a_bRegs;
 
 	/* Compute step size using equation 1 */
 	if (m_bIntTimeAdjust != 0) {
@@ -657,17 +747,17 @@ static void usb_GetStepSize( Plustek_Device *dev, pScanParam pParam )
 		m_wStepSize = (u_short)(((u_long) pParam->PhyDpi.y * m_wLineLength *
 								   m_bLineRateColor) / (4 * hw->wMotorDpi));
 	}
-									   
+
 	if (m_wStepSize < 2)
 		m_wStepSize = 2;
 
 	m_wStepSize = m_wStepSize * 298 / 297;
 
-	a_bRegs[0x46] = _HIBYTE( m_wStepSize );
-	a_bRegs[0x47] = _LOBYTE( m_wStepSize );
+	regs[0x46] = _HIBYTE( m_wStepSize );
+	regs[0x47] = _LOBYTE( m_wStepSize );
 
 	DBG( _DBG_INFO2, "* Stepsize = %u, 0x46=0x%02x 0x47=0x%02x\n",
-	                  m_wStepSize, a_bRegs[0x46], a_bRegs[0x47] );
+	                  m_wStepSize, regs[0x46], regs[0x47] );
 }
 
 /**
@@ -694,22 +784,23 @@ static void usb_GetLineLength( Plustek_Device *dev )
 	int tradj;	/* ITA                                                   */
 	int en_tradj;
 
-	pHWDef hw = &dev->usbDev.HwSetting;
+	pHWDef  hw   = &dev->usbDev.HwSetting;
+	u_char *regs = dev->usbDev.a_bRegs;
 
-	tpspd = (a_bRegs[0x0a] & 0x0c) >> 2; /* turbo/preview mode speed  */
-	tpsel = a_bRegs[0x0a] & 3; 			 /* turbo/preview mode select */
+	tpspd = (regs[0x0a] & 0x0c) >> 2; /* turbo/preview mode speed  */
+	tpsel = regs[0x0a] & 3; 			 /* turbo/preview mode select */
 
-	gbnd = (a_bRegs[0x0e] & 0xf0) >> 4;	 /* TR fi1 guardband duration */
-	dur = (a_bRegs[0x0e] & 0xf);		 /* TR pulse duration         */
+	gbnd = (regs[0x0e] & 0xf0) >> 4;	 /* TR fi1 guardband duration */
+	dur = (regs[0x0e] & 0xf);		 /* TR pulse duration         */
 
-	ntr = a_bRegs[0x0d] / 128;			 /* number of tr pulses       */
+	ntr = regs[0x0d] / 128;			 /* number of tr pulses       */
 
-	afeop = a_bRegs[0x26] & 7;			 /* afe op - 3 channel or 1 channel */
+	afeop = regs[0x26] & 7;			 /* afe op - 3 channel or 1 channel */
 
-	tradj = a_bRegs[0x19] & 0x7f;		 /* integration time adjust */
+	tradj = regs[0x19] & 0x7f;		 /* integration time adjust */
 	en_tradj = (tradj) ? 1 : 0;
 
-	ctmode = (a_bRegs[0x0b] >> 3) & 3;	 /* cis tr timing mode */
+	ctmode = (regs[0x0b] >> 3) & 3;	 /* cis tr timing mode */
 
 	m_bLineRateColor = 1;	
 	if (afeop == 1 || afeop == 5) /* if 3 channel line or 1 channel mode b */
@@ -770,7 +861,7 @@ static void usb_GetLineLength( Plustek_Device *dev )
 	m_wLineLength = tr / m_bLineRateColor;
 
 	DBG( _DBG_INFO2, "* LineLength=%d, LineRateColor=%u\n",
-											m_wLineLength, m_bLineRateColor );
+	                    m_wLineLength, m_bLineRateColor );
 }
 
 /** usb_GetMotorParam
@@ -778,11 +869,12 @@ static void usb_GetLineLength( Plustek_Device *dev )
  */
 static void usb_GetMotorParam( Plustek_Device *dev, pScanParam pParam )
 {
-	int       	 idx, i;
+	int          idx, i;
 	pClkMotorDef clk;
-	pMDef     	 md;
-	pDCapsDef 	 sCaps = &dev->usbDev.Caps;
-	pHWDef    	 hw    = &dev->usbDev.HwSetting;
+	pMDef        md;
+	pDCapsDef    sCaps = &dev->usbDev.Caps;
+	pHWDef       hw    = &dev->usbDev.HwSetting;
+	u_char      *regs  = dev->usbDev.a_bRegs;
 
 	if( !_IS_PLUSTEKMOTOR(hw->motorModel)) {
 	
@@ -790,29 +882,29 @@ static void usb_GetMotorParam( Plustek_Device *dev, pScanParam pParam )
 		md  = clk->motor_sets;
  		idx = 0;
 		for( i = 0; i < _MAX_CLK; i++ ) {
-			if( pParam->PhyDpi.x <= dpi_ranges[i] )
+			if( pParam->PhyDpi.y <= dpi_ranges[i] )
   				break;
 			idx++;
 		}
 		if( idx >= _MAX_CLK )
 			idx = _MAX_CLK - 1;
 
-		a_bRegs[0x56] = md[idx].pwm;
-		a_bRegs[0x57] = md[idx].pwm_duty;
+		regs[0x56] = md[idx].pwm;
+		regs[0x57] = md[idx].pwm_duty;
 
-		a_bRegs[0x43] = 0;
-		a_bRegs[0x44] = 0;
+		regs[0x43] = 0;
+		regs[0x44] = 0;
 
 		if( md[idx].scan_lines_per_line > 1 ) {
 
 			if((pParam->bBitDepth > 8) &&
 				(pParam->bDataType == SCANDATATYPE_Color)) {
 
-				a_bRegs[0x43] = 0xff;
-				a_bRegs[0x44] = md[idx].scan_lines_per_line;
+				regs[0x43] = 0xff;
+				regs[0x44] = md[idx].scan_lines_per_line;
 
 				DBG( _DBG_INFO2, "* Line Skipping : 0x43=0x%02x, 0x44=0x%02x\n",
-	  											a_bRegs[0x43], a_bRegs[0x44] );
+	  											regs[0x43], regs[0x44] );
 			}
 		}
 
@@ -826,28 +918,28 @@ static void usb_GetMotorParam( Plustek_Device *dev, pScanParam pParam )
         	default:
         		if(pParam->PhyDpi.x <= 200)
         		{
-        			a_bRegs[0x56] = 1;
-        			a_bRegs[0x57] = 48;	/* 63; */
+        			regs[0x56] = 1;
+        			regs[0x57] = 48;	/* 63; */
         		}
         		else if(pParam->PhyDpi.x <= 300)
         		{
-        			a_bRegs[0x56] = 2;	/* 8;  */
-        			a_bRegs[0x57] = 48;	/* 56; */
+        			regs[0x56] = 2;	/* 8;  */
+        			regs[0x57] = 48;	/* 56; */
         		}
         		else if(pParam->PhyDpi.x <= 400)
         		{
-        			a_bRegs[0x56] = 8;	
-        			a_bRegs[0x57] = 48;	
+        			regs[0x56] = 8;	
+        			regs[0x57] = 48;	
         		}
         		else if(pParam->PhyDpi.x <= 600)
         		{
-        			a_bRegs[0x56] = 2;	/* 10; */
-        			a_bRegs[0x57] = 48;	/* 56; */
+        			regs[0x56] = 2;	/* 10; */
+        			regs[0x57] = 48;	/* 56; */
         		}
         		else /* pParam->PhyDpi.x == 1200) */
         		{
-        			a_bRegs[0x56] = 1;	/* 8;  */
-        			a_bRegs[0x57] = 48;	/* 56; */
+        			regs[0x56] = 1;	/* 8;  */
+        			regs[0x57] = 48;	/* 56; */
         		}
         		break;
         	}
@@ -855,40 +947,40 @@ static void usb_GetMotorParam( Plustek_Device *dev, pScanParam pParam )
         	switch ( hw->motorModel ) {
 
         	case MODEL_Tokyo600:
-        		a_bRegs[0x56] = 16;
-        		a_bRegs[0x57] = 4;	/* 2; */
+        		regs[0x56] = 16;
+        		regs[0x57] = 4;	/* 2; */
         		break;
         	case MODEL_HuaLien:
         		{
         			if(pParam->PhyDpi.x <= 200)
         			{
-        				a_bRegs[0x56] = 64;	/* 24; */
-        				a_bRegs[0x57] = 4;	/* 16; */
+        				regs[0x56] = 64;	/* 24; */
+        				regs[0x57] = 4;	/* 16; */
         			}
         			else if(pParam->PhyDpi.x <= 300)
         			{
-        				a_bRegs[0x56] = 64;	/* 16; */
-        				a_bRegs[0x57] = 4;	/* 16; */
+        				regs[0x56] = 64;	/* 16; */
+        				regs[0x57] = 4;	/* 16; */
         			}
         			else if(pParam->PhyDpi.x <= 400)
         			{
-        				a_bRegs[0x56] = 64;	/* 16; */
-        				a_bRegs[0x57] = 4;	/* 16; */
+        				regs[0x56] = 64;	/* 16; */
+        				regs[0x57] = 4;	/* 16; */
         			}
         			else /* if(pParam->PhyDpi.x <= 600) */
         			{
-     /* HEINER: check ADF stuff... */
-     #if 0
+/* HEINER: check ADF stuff... */
+#if 0
         				if(ScanInf.m_fADF)
         				{
-        					a_bRegs[0x56] = 8;
-        					a_bRegs[0x57] = 48;
+        					regs[0x56] = 8;
+        					regs[0x57] = 48;
         				}
         				else
-     #endif
+#endif
         				{
-        					a_bRegs[0x56] = 64;	/* 2;  */
-        					a_bRegs[0x57] = 4;	/* 48; */
+        					regs[0x56] = 64;	/* 2;  */
+        					regs[0x57] = 4;	/* 48; */
         				}
         			}
         		}
@@ -897,23 +989,23 @@ static void usb_GetMotorParam( Plustek_Device *dev, pScanParam pParam )
         	default:
         		if(pParam->PhyDpi.x <= 200)
         		{
-        			a_bRegs[0x56] = 24;
-        			a_bRegs[0x57] = 16;
+        			regs[0x56] = 24;
+        			regs[0x57] = 16;
         		}
         		else if(pParam->PhyDpi.x <= 300)
         		{
-        			a_bRegs[0x56] = 16;
-        			a_bRegs[0x57] = 16;
+        			regs[0x56] = 16;
+        			regs[0x57] = 16;
         		}
         		else if(pParam->PhyDpi.x <= 400)
         		{
-        			a_bRegs[0x56] = 16;
-        			a_bRegs[0x57] = 16;
+        			regs[0x56] = 16;
+        			regs[0x57] = 16;
         		}
         		else /* if(pParam->PhyDpi.x <= 600) */
         		{
-        			a_bRegs[0x56] = 2;
-        			a_bRegs[0x57] = 48;
+        			regs[0x56] = 2;
+        			regs[0x57] = 48;
         		}
         		break;
         	}
@@ -921,15 +1013,16 @@ static void usb_GetMotorParam( Plustek_Device *dev, pScanParam pParam )
 	}
 	
 	DBG( _DBG_INFO2, "* MOTOR-Settings: PWM=0x%02x, PWM_DUTY=0x%02x\n",
-	                 a_bRegs[0x56], a_bRegs[0x57] );
+	                 regs[0x56], regs[0x57] );
 }
 
 /**
  */
 static void usb_GetPauseLimit( Plustek_Device *dev, pScanParam pParam )
 {
-	int    coeffsize, scaler;
-	pHWDef hw = &dev->usbDev.HwSetting;
+	int     coeffsize, scaler;
+	pHWDef  hw   = &dev->usbDev.HwSetting;
+	u_char *regs = dev->usbDev.a_bRegs;
 
 	scaler = 1;
 	if( hw->bReg_0x26 & _ONE_CH_COLOR ) {
@@ -942,7 +1035,7 @@ static void usb_GetPauseLimit( Plustek_Device *dev, pScanParam pParam )
 	coeffsize = 4 + 16 + 16;	/* gamma and shading and offset */
 
 	/* if 16 bit, then not all is used */
-	if( a_bRegs[0x09] & 0x20 ) {
+	if( regs[0x09] & 0x20 ) {
 		coeffsize = 16 + 16;	/* no gamma */
 	}
 	coeffsize *= (2*3); /* 3 colors and 2 bytes/word */
@@ -956,8 +1049,8 @@ static void usb_GetPauseLimit( Plustek_Device *dev, pScanParam pParam )
 	m_dwPauseLimit -= ((pParam->Size.dwPhyBytes*scaler) / 1024 + 1);
 
 	/* If not reversing, take into account the steps to reverse */
-	if( a_bRegs[0x50] == 0 )
-		m_dwPauseLimit -= ((a_bRegs[0x54] & 7) *
+	if( regs[0x50] == 0 )
+		m_dwPauseLimit -= ((regs[0x54] & 7) *
 							(pParam->Size.dwPhyBytes * scaler) + 1023) / 1024;
 
 	DBG( _DBG_INFO2, "* PL=%lu, coeffsize=%u, scaler=%u\n",
@@ -966,22 +1059,21 @@ static void usb_GetPauseLimit( Plustek_Device *dev, pScanParam pParam )
 	m_dwPauseLimit = usb_max( usb_min(m_dwPauseLimit,
 						(u_long)ceil(pParam->Size.dwTotalBytes / 1024.0)), 2);
 
-	a_bRegs[0x4e] = (u_char)floor((m_dwPauseLimit*512.0)/(2.0*hw->wDRAMSize));
+	regs[0x4e] = (u_char)floor((m_dwPauseLimit*512.0)/(2.0*hw->wDRAMSize));
 
-	if( a_bRegs[0x4e] > 1 )	{
-		a_bRegs[0x4e]--;
-		if(a_bRegs[0x4e] > 1)
-			a_bRegs[0x4e]--;
+	if( regs[0x4e] > 1 ) {
+		regs[0x4e]--;
+		if(regs[0x4e] > 1)
+			regs[0x4e]--;
 	} else
-		a_bRegs[0x4e] = 1;
+		regs[0x4e] = 1;
 
-	/*
-	 * resume, when buffer is 2/8 kbytes full (512k/2M memory)
+	/* resume, when buffer is 2/8 kbytes full (512k/2M memory)
 	 */
-	a_bRegs[0x4f] = 1;
+	regs[0x4f] = 1;
 
 	DBG( _DBG_INFO2, "* PauseLimit = %lu, [0x4e] = 0x%02x, [0x4f] = 0x%02x\n",
-	                  m_dwPauseLimit, a_bRegs[0x4e], a_bRegs[0x4f] );
+	                  m_dwPauseLimit, regs[0x4e], regs[0x4f] );
 }
 
 /** usb_GetScanLinesAndSize
@@ -1018,6 +1110,7 @@ static SANE_Bool usb_SetScanParameters( Plustek_Device *dev, pScanParam pParam )
 
 	pScanParam pdParam = &dev->scanning.sParam;
 	pHWDef     hw      = &dev->usbDev.HwSetting;
+	u_char    *regs    = dev->usbDev.a_bRegs;
 
 	m_pParam = pParam;
 
@@ -1032,7 +1125,7 @@ static SANE_Bool usb_SetScanParameters( Plustek_Device *dev, pScanParam pParam )
 */
 		hw->dMaxMoveSpeed = 1.0;
 		usb_MotorSelect( dev, SANE_TRUE );
-		usb_MotorOn( dev->fd, SANE_TRUE );
+		usb_MotorOn( dev, SANE_TRUE );
 	}
 
 	/*
@@ -1051,32 +1144,32 @@ static SANE_Bool usb_SetScanParameters( Plustek_Device *dev, pScanParam pParam )
 		if( pParam->bCalibration == PARAM_Scan ) {
 
 			if( pdParam->bSource == SOURCE_ADF ) {
-				a_bRegs[0x50] = 0;
-				a_bRegs[0x51] = 0x40;
+				regs[0x50] = 0;
+				regs[0x51] = 0x40;
 				if( pParam->PhyDpi.x <= 300)
-					a_bRegs[0x54] = (a_bRegs[0x54] & ~7) | 4;	/* 3; */
+					regs[0x54] = (regs[0x54] & ~7) | 4;	/* 3; */
 				else
-					a_bRegs[0x54] = (a_bRegs[0x54] & ~7) | 5;	/* 4; */
+					regs[0x54] = (regs[0x54] & ~7) | 5;	/* 4; */
 			} else {
-				a_bRegs[0x50] = hw->bStepsToReverse;
-				a_bRegs[0x51] = hw->bReg_0x51;
-				a_bRegs[0x54] &= ~7;
+				regs[0x50] = hw->bStepsToReverse;
+				regs[0x51] = hw->bReg_0x51;
+				regs[0x54] &= ~7;
 			}
 		} else
-			a_bRegs[0x50] = 0;
+			regs[0x50] = 0;
 	} else {
 		if( pParam->bCalibration == PARAM_Scan )
-			a_bRegs[0x50] = hw->bStepsToReverse;
+			regs[0x50] = hw->bStepsToReverse;
 		else
-			a_bRegs[0x50] = 0;
+			regs[0x50] = 0;
 	}
 
 	/* Assume we will not use ITA */
-	a_bRegs[0x19] = m_bIntTimeAdjust = 0;
+	regs[0x19] = m_bIntTimeAdjust = 0;
 
 	/* Get variables from calculation algorithms */
 	if(!(pParam->bCalibration == PARAM_Scan &&
-         pParam->bSource == SOURCE_ADF && fLastScanIsAdf )) {
+          pParam->bSource == SOURCE_ADF && dev->usbDev.fLastScanIsAdf )) {
 
 		DBG( _DBG_INFO2, "* Scan calculations...\n" );
 		usb_GetLineLength ( dev );
@@ -1090,7 +1183,7 @@ static SANE_Bool usb_SetScanParameters( Plustek_Device *dev, pScanParam pParam )
 	if( m_dMCLKDivider < 1.0)
 		m_dMCLKDivider = 1.0;
 
-	m_wFastFeedStepSize = (u_short)(dwCrystalFrequency /
+	m_wFastFeedStepSize = (u_short)(CRYSTAL_FREQ /
 	                          (m_dMCLKDivider * 8 * m_bCM * hw->dMaxMoveSpeed *
 	                           4 * hw->wMotorDpi));
 	/* CIS special ;-) */
@@ -1102,13 +1195,13 @@ static SANE_Bool usb_SetScanParameters( Plustek_Device *dev, pScanParam pParam )
 	if( m_bIntTimeAdjust != 0 )
 		m_wFastFeedStepSize /= m_bIntTimeAdjust;
 
-	if(a_bRegs[0x0a])
-		m_wFastFeedStepSize *= ((a_bRegs[0x0a] >> 2) + 2);
-	a_bRegs[0x48] = _HIBYTE( m_wFastFeedStepSize );
-	a_bRegs[0x49] = _LOBYTE( m_wFastFeedStepSize );
+	if(regs[0x0a])
+		m_wFastFeedStepSize *= ((regs[0x0a] >> 2) + 2);
+	regs[0x48] = _HIBYTE( m_wFastFeedStepSize );
+	regs[0x49] = _LOBYTE( m_wFastFeedStepSize );
 
 	DBG( _DBG_INFO2, "* FFStepSize = %u, [0x48] = 0x%02x, [0x49] = 0x%02x\n",
-	                       m_wFastFeedStepSize, a_bRegs[0x48], a_bRegs[0x49] );
+	                       m_wFastFeedStepSize, regs[0x48], regs[0x49] );
 
 	/* Compute the number of lines to scan using actual Y resolution */
 	usb_GetScanLinesAndSize( dev, pParam );
@@ -1121,17 +1214,17 @@ static SANE_Bool usb_SetScanParameters( Plustek_Device *dev, pScanParam pParam )
 	/* For ADF .... */
 	if(pParam->bCalibration == PARAM_Scan && pParam->bSource == SOURCE_ADF)	{
 
-		if( fLastScanIsAdf ) {
+		if( dev->usbDev.fLastScanIsAdf ) {
 
-			a_bRegs[0x08] = reg8;
-			memcpy( &a_bRegs[0x38], reg38, sizeof(reg38));
-			memcpy( &a_bRegs[0x48], reg48, sizeof(reg48));
+			regs[0x08] = reg8;
+			memcpy( &regs[0x38], reg38, sizeof(reg38));
+			memcpy( &regs[0x48], reg48, sizeof(reg48));
 
 		} else {
 
-			reg8 = a_bRegs[0x08];
-			memcpy( reg38, &a_bRegs[0x38], sizeof(reg38));
-			memcpy( reg48, &a_bRegs[0x48], sizeof(reg48));
+			reg8 = regs[0x08];
+			memcpy( reg38, &regs[0x38], sizeof(reg38));
+			memcpy( reg48, &regs[0x48], sizeof(reg48));
 		}
 		usb_MotorSelect( dev, SANE_TRUE );
 	}
@@ -1148,18 +1241,30 @@ static SANE_Bool usb_SetScanParameters( Plustek_Device *dev, pScanParam pParam )
 	if( !usbio_WriteReg( dev->fd, 0x19, 6 ))
 		return SANE_FALSE;
 
-	a_bRegs[0x07] = 0;
-	a_bRegs[0x28] = 0;
+	regs[0x07] = 0;
+	regs[0x28] = 0;
 
 	/* set unused registers to 0 */
-	memset( &a_bRegs[0x03], 0, 3 );
-	memset( &a_bRegs[0x5f], 0, 0x7f-0x5f+1 );
+	memset( &regs[0x03], 0, 3 );
+	memset( &regs[0x5f], 0, 0x7f-0x5f+1 );
+
+	if(pParam->bCalibration == PARAM_Scan && pParam->bSource != SOURCE_ADF) {
+
+		u_short scansteps = (u_short)ceil((double)(pParam->Size.dwPhyLines + 10)*
+		                                     hw->wMotorDpi / pParam->PhyDpi.y);
+		DBG( _DBG_INFO, "* Scansteps=%u (%lu*%u/%u)\n", scansteps, 
+		                 pParam->Size.dwPhyLines, hw->wMotorDpi, pParam->PhyDpi.y );
+#if 1
+		regs[0x4c] = _HIBYTE(scansteps);
+		regs[0x4d] = _LOBYTE(scansteps);
+#endif
+	}
 
 	/* set the merlin registers */
-	_UIO(sanei_lm983x_write( dev->fd, 0x03, &a_bRegs[0x03], 3, SANE_TRUE));
-	_UIO(sanei_lm983x_write( dev->fd, 0x08, &a_bRegs[0x08], 0x7f - 0x08+1, SANE_TRUE));
+	_UIO(sanei_lm983x_write( dev->fd, 0x03, &regs[0x03], 3, SANE_TRUE));
+	_UIO(sanei_lm983x_write( dev->fd, 0x08, &regs[0x08], 0x7f - 0x08+1, SANE_TRUE));
 
-    usleep( 100 );
+	usleep( 100 );
 	
 	if( !usbio_WriteReg( dev->fd, 0x07, 0 ))
 		return SANE_FALSE;
@@ -1174,8 +1279,9 @@ static SANE_Bool usb_ScanBegin( Plustek_Device *dev, SANE_Bool auto_park )
 {
 	u_char  value;
 	u_short inches;
-	pHWDef       hw = &dev->usbDev.HwSetting;
-	pDCapsDef    sc = &dev->usbDev.Caps;
+	pHWDef       hw   = &dev->usbDev.HwSetting;
+	pDCapsDef    sc   = &dev->usbDev.Caps;
+	u_char      *regs = dev->usbDev.a_bRegs;
 
 	DBG( _DBG_INFO, "usb_ScanBegin()\n" );
 
@@ -1184,7 +1290,7 @@ static SANE_Bool usb_ScanBegin( Plustek_Device *dev, SANE_Bool auto_park )
 
 	/* Disable home sensor during scan, or the chassis cannot move */
 	value = ((m_pParam->bCalibration == PARAM_Scan &&
-			  m_pParam->bSource == SOURCE_ADF)? (a_bRegs[0x58] & ~7): 0);
+	        m_pParam->bSource == SOURCE_ADF)? (regs[0x58] & ~7): 0);
 
 	if(!usbio_WriteReg( dev->fd, 0x58, value ))
 		return SANE_FALSE;
@@ -1223,7 +1329,7 @@ static SANE_Bool usb_ScanBegin( Plustek_Device *dev, SANE_Bool auto_park )
 
 	/* Download map & Shading data */
 	if(( m_pParam->bCalibration == PARAM_Scan &&
-		!usb_MapDownload( dev, m_pParam->bDataType)) ||
+		!usb_MapDownload( dev )) ||
 		!usb_DownloadShadingData( dev, m_pParam->bCalibration ))
 		return SANE_FALSE;
 
@@ -1240,9 +1346,11 @@ static SANE_Bool usb_ScanBegin( Plustek_Device *dev, SANE_Bool auto_park )
 	dumpregs( dev->fd, NULL );
 
 	inches = (u_short)((m_pParam->Origin.y *300UL)/hw->wMotorDpi);
-	DBG( _DBG_INFO2, ">>> INC=%u, DOY=%u\n", inches, sc->Normal.DataOrigin.y );
+	DBG( _DBG_INFO2, ">>> INCH=%u, DOY=%u\n", inches, sc->Normal.DataOrigin.y );
 	if( inches > sc->Normal.DataOrigin.y )
 		usb_WaitPos( dev, 150, SANE_FALSE );
+
+	DBG( _DBG_INFO, "usb_ScanBegin() done.\n" );
 	return SANE_TRUE;
 }
 
@@ -1255,7 +1363,6 @@ static SANE_Bool usb_ScanEnd( Plustek_Device *dev )
 
 	DBG( _DBG_INFO, "usbDev_ScanEnd(), start=%u, park=%u\n",
 	                                                   m_fStart, m_fAutoPark );
-
 	usbio_ReadReg( dev->fd, 0x07, &value );
 	if( value == 3 || value != 2 )
 		usbio_WriteReg( dev->fd, 0x07, 0 );
@@ -1270,7 +1377,6 @@ static SANE_Bool usb_ScanEnd( Plustek_Device *dev )
 	
 		usb_ModuleToHome( dev, SANE_FALSE );
 	}
-
 	return SANE_TRUE;
 }
 
@@ -1286,6 +1392,7 @@ static SANE_Bool usb_IsDataAvailableInDRAM( Plustek_Device *dev )
 	u_char         a_bBand[3];
 	long           dwTicks;
     struct timeval t;
+	u_char         *regs = dev->usbDev.a_bRegs;
 
 	DBG( _DBG_INFO, "usb_IsDataAvailableInDRAM()\n" );
 
@@ -1313,9 +1420,9 @@ static SANE_Bool usb_IsDataAvailableInDRAM( Plustek_Device *dev )
 
 			if( m_pParam->bSource != SOURCE_Reflection )
 
-				usleep(1000*(30 * a_bRegs[0x08] * dev->usbDev.Caps.OpticDpi.x / 600));
+				usleep(1000*(30 * regs[0x08] * dev->usbDev.Caps.OpticDpi.x / 600));
 			else
-				usleep(1000*(20 * a_bRegs[0x08] * dev->usbDev.Caps.OpticDpi.x / 600));
+				usleep(1000*(20 * regs[0x08] * dev->usbDev.Caps.OpticDpi.x / 600));
 
 			DBG( _DBG_INFO, "Data is available\n" );
 			return SANE_TRUE;
@@ -1332,7 +1439,8 @@ static SANE_Bool usb_ScanReadImage( Plustek_Device *dev,
                                     void *pBuf, u_long dwSize )
 {
 	static u_long dwBytes = 0;
-	SANE_Status res;
+	u_char       *regs = dev->usbDev.a_bRegs;
+	SANE_Status   res;
 
 	DBG( _DBG_READ, "usb_ScanReadImage(%lu)\n", dwSize );
 
@@ -1345,7 +1453,10 @@ static SANE_Bool usb_ScanReadImage( Plustek_Device *dev,
 		if (!usb_IsDataAvailableInDRAM( dev )) {
 			DBG( _DBG_ERROR, "Nothing to read...\n" );
 			return SANE_FALSE;
-		}			
+		}
+
+		/* restore the fast forward stepsize...*/
+		sanei_lm983x_write(dev->fd, 0x48, &regs[0x48], 2, SANE_TRUE);
 	}
 /* HEINER: ADF */
 #if 0
@@ -1393,7 +1504,6 @@ static SANE_Bool usb_ScanReadImage( Plustek_Device *dev,
 		return TRUE;
 	}
 #endif
-
 	res = sanei_lm983x_read(dev->fd, 0x00, (u_char *)pBuf, dwSize, SANE_FALSE);
 
 	/* check for pressed ESC button, as sanei_lm983x_read() may take some time
@@ -1404,17 +1514,16 @@ static SANE_Bool usb_ScanReadImage( Plustek_Device *dev,
 	}
 
 	DBG( _DBG_READ, "usb_ScanReadImage() done, result: %d\n", res );
-
 	if( SANE_STATUS_GOOD == res ) {
 		return SANE_TRUE;
 	}
 
 	DBG( _DBG_ERROR, "usb_ScanReadImage() failed\n" );
-	
 	return SANE_FALSE;
 }
 
-/**
+/** calculate the number of pixels per line and lines out of a given
+ * crop-area. The size of the area is given on a 300dpi base!
  */
 static void usb_GetImageInfo( Plustek_Device *dev, pImgDef pInfo, pWinInfo pSize )
 {
@@ -1423,6 +1532,8 @@ static void usb_GetImageInfo( Plustek_Device *dev, pImgDef pInfo, pWinInfo pSize
 	pSize->dwPixels = (u_long)pInfo->crArea.cx * pInfo->xyDpi.x / 300UL;
 	pSize->dwLines  = (u_long)pInfo->crArea.cy * pInfo->xyDpi.y / 300UL;
 
+	DBG( _DBG_INFO2,"Area: cx=%u, cy=%u\n",pInfo->crArea.cx,pInfo->crArea.cy);
+
 	switch( pInfo->wDataType ) {
 
 		case COLOR_TRUE48:
