@@ -46,7 +46,7 @@
    This file is a component of the implementation of a backend for many
    of the the AGFA SnapScan and Acer Vuego/Prisa flatbed scanners. */
 
-/* $Id: snapscan-options.c,v 1.17 2004/09/02 20:59:11 oliver-guest Exp $
+/* $Id: snapscan-options.c,v 1.21 2005/07/20 21:37:29 oliver-guest Exp $
    SANE SnapScan backend */
 
 /* default option values */
@@ -161,7 +161,7 @@ static const SANE_Range x_range_tpo_2480 =
 };        /* mm */
 static const SANE_Range y_range_tpo_2480 =
 {
-    SANE_FIX (0.0), SANE_FIX (125.0), 0
+    SANE_FIX (0.0), SANE_FIX (80.0), 0
 };        /* mm */
 
 static SANE_Range x_range_tpo;
@@ -244,6 +244,7 @@ static void init_options (SnapScan_Scanner * ps)
         x_range_tpo = x_range_tpo_e50;
         y_range_tpo = y_range_tpo_e50;
         break;
+    case PERFECTION1270:
     case PERFECTION1670:
         x_range_tpo = x_range_tpo_1670;
         y_range_tpo = y_range_tpo_1670;
@@ -335,7 +336,13 @@ static void init_options (SnapScan_Scanner * ps)
         SANE_CAP_SOFT_SELECT | SANE_CAP_SOFT_DETECT | SANE_CAP_AUTOMATIC;
     po[OPT_HIGHQUALITY].constraint_type = SANE_CONSTRAINT_NONE;
     ps->highquality = DEFAULT_HIGHQUALITY;
-
+    if (ps->pdev->model == PERFECTION1270)
+    {
+        po[OPT_HIGHQUALITY].cap |= SANE_CAP_INACTIVE;
+        ps->val[OPT_QUALITY_CAL].b = SANE_TRUE;
+        ps->highquality=SANE_TRUE;
+    }
+    
     po[OPT_BRIGHTNESS].name = SANE_NAME_BRIGHTNESS;
     po[OPT_BRIGHTNESS].title = SANE_TITLE_BRIGHTNESS;
     po[OPT_BRIGHTNESS].desc = SANE_DESC_BRIGHTNESS;
@@ -525,7 +532,6 @@ static void init_options (SnapScan_Scanner * ps)
     if ((!(ps->hconfig & HCFG_CAL_ALLOWED))
         || (ps->pdev->model == SNAPSCANE52)
         || (ps->pdev->model == PERFECTION1670)
-        || (ps->pdev->model == PERFECTION2480)
         || (ps->pdev->model == PRISA5300)) {
         po[OPT_QUALITY_CAL].cap |= SANE_CAP_INACTIVE;
         ps->val[OPT_QUALITY_CAL].b = SANE_FALSE;
@@ -1508,6 +1514,18 @@ SANE_Status sane_control_option (SANE_Handle h,
 
 /*
  * $Log: snapscan-options.c,v $
+ * Revision 1.21  2005/07/20 21:37:29  oliver-guest
+ * Changed TPO scanning area for 2480/2580, reenabled 2400 DPI for 2480/2580
+ *
+ * Revision 1.20  2005/05/22 11:50:24  oliver-guest
+ * Disabled 2400 DPI for Epson 2480
+ *
+ * Revision 1.19  2004/12/09 23:21:47  oliver-guest
+ * Added quality calibration for Epson 2480 (by Simon Munton)
+ *
+ * Revision 1.18  2004/12/01 22:12:02  oliver-guest
+ * Added support for Epson 1270
+ *
  * Revision 1.17  2004/09/02 20:59:11  oliver-guest
  * Added support for Epson 2480
  *
