@@ -47,7 +47,7 @@
    of the the AGFA SnapScan and Acer Vuego/Prisa flatbed scanners. */
 
 
-/* $Id: snapscan-sources.c,v 1.6 2001/12/17 22:51:49 oliverschwartz Exp $
+/* $Id: snapscan-sources.c,v 1.12 2004/11/14 19:26:38 oliver-guest Exp $
    SnapScan backend data sources (implementation) */
 
 /**************************************************************************************
@@ -175,23 +175,14 @@ static SANE_Status SCSISource_get (Source *pself,
     DBG (DL_CALL_TRACE, "%s\n", me);
     while (remaining > 0
            && pself->remaining(pself) > 0
-           && status == SANE_STATUS_GOOD)
+           && status == SANE_STATUS_GOOD
+           && !cancelRead)
     {
         SANE_Int ndata = ps->scsi_buf_max - ps->scsi_buf_pos;
         DBG (DL_DATA_TRACE, "%s: ndata %d; remaining %d\n", me, ndata, remaining);
         if (ndata == 0)
         {
-            ps->pss->expected_read_bytes = ps->absolute_max;
-/*
-                SANE_Int lines;
-
-                if (is_colour_mode(actual_mode(ps->pss)) == SANE_TRUE)
-                    lines = ps->pss->rgb_lpr;
-                else
-                    lines = ps->pss->gs_lpr;
-                ps->pss->expected_read_bytes = lines * ps->pss->bytes_per_line;
-*/
-            ps->pss->expected_read_bytes = MIN(ps->pss->expected_read_bytes,
+            ps->pss->expected_read_bytes = MIN((size_t)ps->absolute_max,
                                            ps->pss->bytes_remaining);
             ps->scsi_buf_pos = 0;
             ps->scsi_buf_max = 0;
@@ -201,9 +192,9 @@ static SANE_Status SCSISource_get (Source *pself,
             ps->scsi_buf_max = ps->pss->read_bytes;
             ndata = ps->pss->read_bytes;
             ps->pss->bytes_remaining -= ps->pss->read_bytes;
-            DBG (DL_DATA_TRACE, "%s: pos: %d; max: %d; expected: %d; read: %d\n",
-                me, ps->scsi_buf_pos, ps->scsi_buf_max, ps->pss->expected_read_bytes,
-                ps->pss->read_bytes);
+            DBG (DL_DATA_TRACE, "%s: pos: %d; max: %d; expected: %lu; read: %lu\n",
+                me, ps->scsi_buf_pos, ps->scsi_buf_max, (u_long) ps->pss->expected_read_bytes,
+                (u_long) ps->pss->read_bytes);
         }
         ndata = MIN(ndata, remaining);
         memcpy (pbuf, ps->pss->buf + ps->scsi_buf_pos, (size_t)ndata);
@@ -246,11 +237,13 @@ typedef struct
 {
     SOURCE_GUTS;
     int fd;
+    SANE_Int bytes_remaining;
 } FDSource;
 
 static SANE_Int FDSource_remaining (Source *pself)
 {
-    return pself->pss->bytes_remaining;
+    FDSource *ps = (FDSource *) pself;    
+    return ps->bytes_remaining;
 }
 
 static SANE_Status FDSource_get (Source *pself, SANE_Byte *pbuf, SANE_Int *plen)
@@ -282,7 +275,7 @@ static SANE_Status FDSource_get (Source *pself, SANE_Byte *pbuf, SANE_Int *plen)
             DBG(DL_DATA_TRACE, "%s: EOF\n",__FUNCTION__);
             break;
         }
-        ps->pss->bytes_remaining -= bytes_read;
+        ps->bytes_remaining -= bytes_read;
         remaining -= bytes_read;
         pbuf += bytes_read;
     }
@@ -308,7 +301,10 @@ static SANE_Status FDSource_init (FDSource *pself,
                                       FDSource_get,
                                       FDSource_done);
     if (status == SANE_STATUS_GOOD)
+    {
         pself->fd = fd;
+        pself->bytes_remaining = pss->bytes_per_line * (pss->lines + pss->chroma);
+    }
     return status;
 }
 
@@ -547,7 +543,8 @@ static SANE_Status Expander_get (Source *pself, SANE_Byte *pbuf, SANE_Int *plen)
 
     while (remaining > 0
            &&
-           pself->remaining(pself) > 0)
+           pself->remaining(pself) > 0 &&
+           !cancelRead)
     {
         if (ps->ch_pos == ps->ch_ndata)
         {
@@ -667,6 +664,192 @@ static SANE_Status create_Expander (SnapScan_Scanner *pss,
     return status;
 }
 
+/* 
+   This filter implements a fix for scanners that have some columns
+   of pixels offset. Currently it only shifts every other column
+   starting with the first one down 4 pixels.
+   
+   The Deinterlacer detects if data is in SANE RGB frame format (3 bytes/pixel)
+   or in Grayscale (1 byte/pixel).
+   
+   The first 4 lines of data in the output are fudged so that even indexed
+   add odd indexed pixels will have the same value. This is necessary because
+   the real pixel values of the columns that are shifted down are not 
+   in the data for the first 4 lines. A better way to handle this would be to
+   scan in 4 extra lines of data, but I haven't figured out how to do this 
+   yet.
+
+*/
+
+typedef struct
+{
+    TX_SOURCE_GUTS;
+    SANE_Byte *ch_buf;            /* channel buffer */
+    SANE_Int   ch_size;           /* channel buffer size */
+    SANE_Int   ch_ndata;          /* actual #bytes in channel buffer */
+    SANE_Int   ch_pos;            /* position in buffer */
+    SANE_Int   ch_bytes_per_pixel;
+    SANE_Bool  ch_past_init;      /* flag indicating if we have enough data to shift pixels down */
+} Deinterlacer;
+
+static SANE_Int Deinterlacer_remaining (Source *pself)
+{
+    Deinterlacer *ps = (Deinterlacer *) pself;
+    SANE_Int result = TxSource_remaining(pself);
+    result += ps->ch_ndata - ps->ch_pos;
+    return result;
+}
+
+static SANE_Status Deinterlacer_get (Source *pself, SANE_Byte *pbuf, SANE_Int *plen)
+{
+    Deinterlacer *ps = (Deinterlacer *) pself;
+    SANE_Status status = SANE_STATUS_GOOD;
+    SANE_Int remaining = *plen;
+    SANE_Int org_len = *plen;
+    char *me = "Deinterlacer_get";
+    
+    DBG(DL_DATA_TRACE, "%s: remaining=%d, pself->remaining=%d, ch_ndata=%d, ch_pos=%d\n",
+            me, remaining, pself->remaining(pself), ps->ch_ndata, ps->ch_pos);
+
+    while (remaining > 0
+           &&
+           pself->remaining(pself) > 0 &&
+           !cancelRead)
+    {
+        if (ps->ch_pos % (ps->ch_size/5) == ps->ch_ndata % (ps->ch_size/5) )
+        {
+            /* we need more data; try to get the remainder of the current
+               line, or else the next line */
+            SANE_Int ndata = (ps->ch_size/5) - ps->ch_ndata % (ps->ch_size/5);
+            if (ps->ch_pos >= ps->ch_size)
+            {
+	        /* wrap to the beginning of the buffer if we need to */
+                ps->ch_ndata = 0;
+                ps->ch_pos = 0;
+                ndata = ps->ch_size /5;
+            }
+            status = TxSource_get(pself, ps->ch_buf + ps->ch_pos, &ndata);
+            if (status != SANE_STATUS_GOOD)
+                break;
+            if (ndata == 0)
+                break;
+            ps->ch_ndata += ndata;
+        }
+	
+        if ((ps->ch_pos/ps->ch_bytes_per_pixel) % 2 == 0)
+        {
+	    /* the even indexed pixels need to be shifted down */
+	    if (ps->ch_past_init){
+	    	/* We need to use data 4 lines back */
+		/* So we just go one forward and it will wrap around to 4 back. */
+	    	*pbuf = ps->ch_buf[(ps->ch_pos + (ps->ch_size/5)) % ps->ch_size];
+	    }else{
+		/* Use data from the next pixel for even indexed pixels
+		   if we are on the first few lines. 
+	           TODO: also we will overread the buffer if the buffer read ended 
+		   on the first pixel. */
+		if (ps->ch_pos % (ps->ch_size/5) == 0 )
+	    	    *pbuf = ps->ch_buf[ps->ch_pos+ps->ch_bytes_per_pixel];
+		else
+	    	    *pbuf = ps->ch_buf[ps->ch_pos-ps->ch_bytes_per_pixel];
+	    }
+        }else{
+	    /* odd indexed pixels are okay */
+            *pbuf = ps->ch_buf[ps->ch_pos];
+        }
+	/* set the flag so we know we have enough data to start shifting columns */
+	if (ps->ch_pos >= ps->ch_size /5 *4)
+	   ps->ch_past_init = SANE_TRUE;
+
+        pbuf++;
+        remaining--;
+        ps->ch_pos++;
+    }
+
+    *plen -= remaining;
+    
+    DBG(DL_DATA_TRACE,
+        "%s: Request=%d, remaining=%d, read=%d, TXSource_rem=%d, bytes_rem=%lu\n",
+        me,
+        org_len,
+        pself->remaining(pself),
+        *plen,
+        TxSource_remaining(pself),
+        (u_long) ps->pss->bytes_remaining);    
+    return status;
+}
+
+static SANE_Status Deinterlacer_done (Source *pself)
+{
+    Deinterlacer *ps = (Deinterlacer *) pself;
+    SANE_Status status = TxSource_done(pself);
+    free(ps->ch_buf);
+    ps->ch_buf = NULL;
+    ps->ch_size = 0;
+    ps->ch_pos = 0;
+    return status;
+}
+
+static SANE_Status Deinterlacer_init (Deinterlacer *pself,
+                  SnapScan_Scanner *pss,
+                                  Source *psub)
+{
+    SANE_Status status = TxSource_init((TxSource *) pself,
+                                       pss,
+                                       Deinterlacer_remaining,
+                                       TxSource_bytesPerLine,
+                                       TxSource_pixelsPerLine,
+                                       Deinterlacer_get,
+                                       Deinterlacer_done,
+                                       psub);
+    if (status == SANE_STATUS_GOOD)
+    {
+        /* We need at least 5 lines of buffer in order to shift up 4 pixels. */
+        pself->ch_size = TxSource_bytesPerLine((Source *) pself) *5;
+        pself->ch_buf = (SANE_Byte *) malloc(pself->ch_size);
+        if (pself->ch_buf == NULL)
+        {
+            DBG (DL_MAJOR_ERROR,
+                 "%s: couldn't allocate channel buffer.\n",
+                 __FUNCTION__);
+            status = SANE_STATUS_NO_MEM;
+        }
+        else
+        {
+            pself->ch_ndata = 0;
+            pself->ch_pos = 0;
+            pself->ch_past_init = SANE_FALSE;
+	    if (actual_mode(pss) == MD_GREYSCALE)
+	    	pself->ch_bytes_per_pixel = 1;
+	    else
+	    	pself->ch_bytes_per_pixel = 3;
+        }
+    }
+    return status;
+}
+
+static SANE_Status create_Deinterlacer (SnapScan_Scanner *pss,
+                                    Source *psub,
+                                    Source **pps)
+{
+    SANE_Status status = SANE_STATUS_GOOD;
+    *pps = (Source *) malloc(sizeof(Deinterlacer));
+    if (*pps == NULL)
+    {
+        DBG (DL_MAJOR_ERROR,
+             "%s: failed to allocate Deinterlacer.\n",
+             __FUNCTION__);
+        status = SANE_STATUS_NO_MEM;
+    }
+    else
+    {
+        status = Deinterlacer_init ((Deinterlacer *) *pps, pss, psub);
+    }
+    return status;
+}
+
+/* ----------------------------------------------------- */
+
 /* the RGB router assumes 8-bit RGB data arranged in contiguous
    channels, possibly with R-G and R-B offsets, and rearranges the
    data into SANE RGB frame format */
@@ -711,7 +894,7 @@ static SANE_Status RGBRouter_get (Source *pself,
     SANE_Int org_len = *plen;
     char *me = "RGBRouter_get";
 
-    while (remaining > 0  &&  pself->remaining(pself) > 0)
+    while (remaining > 0  &&  pself->remaining(pself) > 0 && !cancelRead)
     {
         DBG(DL_DATA_TRACE, "%s: remaining=%d, pself->remaining=%d, round_req=%d, cb_size=%d\n",
             me, remaining, pself->remaining(pself), ps->round_req, ps->cb_size);
@@ -736,7 +919,7 @@ static SANE_Status RGBRouter_get (Source *pself,
                 }
                 ps->round_read += run_req;
             }
-            while (ps->round_req > ps->round_read);
+            while ((ps->round_req > ps->round_read) && !cancelRead);
 
             /* route RGB */
             ps->cb_start = (ps->cb_start + ps->round_read)%ps->cb_size;
@@ -769,13 +952,13 @@ static SANE_Status RGBRouter_get (Source *pself,
     }
     *plen -= remaining;
     DBG(DL_DATA_TRACE,
-        "%s: Request=%d, remaining=%d, read=%d, TXSource_rem=%d, bytes_rem=%d\n",
+        "%s: Request=%d, remaining=%d, read=%d, TXSource_rem=%d, bytes_rem=%lu\n",
         me,
         org_len,
         pself->remaining(pself),
         *plen,
         TxSource_remaining(pself),
-        ps->pss->bytes_remaining);
+        (u_long) ps->pss->bytes_remaining);
     return status;
 }
 
@@ -938,13 +1121,27 @@ static SANE_Status create_source_chain (SnapScan_Scanner *pss,
         {
         case MD_COLOUR:
             status = create_RGBRouter (pss, *pps, pps);
+            /* We only have the interlace probelms on 
+               some scanners like the Epson Perfection 2480/2580 
+               at 2400 dpi. */
+            if (status == SANE_STATUS_GOOD && 
+                pss->pdev->model == PERFECTION2480 &&
+                pss->res == 2400)
+                status = create_Deinterlacer (pss, *pps, pps);
             break;
         case MD_BILEVELCOLOUR:
             status = create_Expander (pss, *pps, pps);
             if (status == SANE_STATUS_GOOD)
                 status = create_RGBRouter (pss, *pps, pps);
+            if (status == SANE_STATUS_GOOD && 
+                pss->pdev->model == PERFECTION2480 &&
+                pss->res == 2400)
+                status = create_Deinterlacer (pss, *pps, pps);
             break;
         case MD_GREYSCALE:
+            if (pss->pdev->model == PERFECTION2480 &&
+                pss->res == 2400)
+                status = create_Deinterlacer (pss, *pps, pps);
             break;
         case MD_LINEART:
             /* The SnapScan creates a negative image by
@@ -965,6 +1162,24 @@ static SANE_Status create_source_chain (SnapScan_Scanner *pss,
 
 /*
  * $Log: snapscan-sources.c,v $
+ * Revision 1.12  2004/11/14 19:26:38  oliver-guest
+ * Applied patch from Julien Blache to change ch_past_init from SANE_Int to SANE_Bool
+ *
+ * Revision 1.11  2004/11/09 23:17:38  oliver-guest
+ * First implementation of deinterlacer for Epson scanners at high resolutions (thanks to Brad Johnson)
+ *
+ * Revision 1.10  2004/10/03 17:34:36  hmg-guest
+ * 64 bit platform fixes (bug #300799).
+ *
+ * Revision 1.9  2004/04/09 16:18:37  oliver-guest
+ * Fix initialization of FDSource.bytes_remaining
+ *
+ * Revision 1.8  2004/04/09 11:59:02  oliver-guest
+ * Fixes for pthread implementation
+ *
+ * Revision 1.7  2004/04/08 21:53:10  oliver-guest
+ * Use sanei_thread in snapscan backend
+ *
  * Revision 1.6  2001/12/17 22:51:49  oliverschwartz
  * Update to snapscan-20011212 (snapscan 1.4.3)
  *
