@@ -47,7 +47,7 @@
    of the the AGFA SnapScan and Acer Vuego/Prisa flatbed scanners. */
 
 
-/* $Id: snapscan.c,v 1.45 2004/10/03 17:34:36 hmg-guest Exp $
+/* $Id: snapscan.c,v 1.48 2004/12/09 23:21:48 oliver-guest Exp $
    SANE SnapScan backend */
 
 #include "../include/sane/config.h"
@@ -79,7 +79,7 @@
 
 #define EXPECTED_MAJOR       1
 #define MINOR_VERSION        4
-#define BUILD               40
+#define BUILD               42
 
 #define BACKEND_NAME snapscan
 
@@ -176,6 +176,7 @@ static inline int calibration_line_length(SnapScan_Scanner *pss)
     case PRISA5000:
         pos_factor = 600;
         break;
+    case PERFECTION1270:
     case PERFECTION1670:
         pos_factor = 800;
         break;
@@ -213,10 +214,9 @@ static const SANE_Device **get_devices_list = NULL;
 #include "snapscan-options.c"
 
 /* Initialize gamma tables */
-static SANE_Status init_gamma(SnapScan_Scanner * ps)
+static SANE_Status alloc_gamma_tables(SnapScan_Scanner * ps)
 {
-    static const char me[] = "init_gamma";
-    u_char *gamma;
+    static const char me[] = "alloc_gamma_tables";
 
     ps->gamma_length = 1 << ps->bpp;
     DBG (DL_MINOR_INFO, "%s: using 4*%d bytes for gamma table\n",
@@ -226,16 +226,8 @@ static SANE_Status init_gamma(SnapScan_Scanner * ps)
     ps->gamma_tables =
         (SANE_Int *) malloc(4 * ps->gamma_length * sizeof(SANE_Int));
 
-    gamma = (u_char*) malloc(ps->gamma_length * sizeof(u_char));
-
-    if (!ps->gamma_tables || !gamma)
+    if (!ps->gamma_tables)
     {
-        if (ps->gamma_tables)
-            free (ps->gamma_tables);
-
-        if (gamma)
-            free (gamma);
-
         return SANE_STATUS_NO_MEM;
     }
 
@@ -244,6 +236,20 @@ static SANE_Status init_gamma(SnapScan_Scanner * ps)
     ps->gamma_table_g = &ps->gamma_tables[2 * ps->gamma_length];
     ps->gamma_table_b = &ps->gamma_tables[3 * ps->gamma_length];
 
+    return SANE_STATUS_GOOD;
+}
+
+static SANE_Status init_gamma(SnapScan_Scanner * ps)
+{
+    u_char *gamma;
+
+    gamma = (u_char*) malloc(ps->gamma_length * sizeof(u_char));
+
+    if (!gamma)
+    {
+        return SANE_STATUS_NO_MEM;
+    }
+
     /* Default tables */
     gamma_n (SANE_UNFIX(ps->gamma_gs), ps->bright, ps->contrast, gamma, ps->bpp);
     gamma_to_sane (ps->gamma_length, gamma, ps->gamma_table_gs);
@@ -970,6 +976,17 @@ SANE_Status sane_open (SANE_String_Const name, SANE_Handle * h)
         }
         close_scanner(pss);
 
+        status = alloc_gamma_tables (pss);
+        if (status != SANE_STATUS_GOOD)
+        {
+            DBG (DL_MAJOR_ERROR,
+                 "%s: error in alloc_gamma_tables: %s\n",
+                 me,
+                 sane_strstatus (status));
+            free (pss);
+            return status;
+        }
+
         init_options (pss);
         status = init_gamma (pss);
         if (status != SANE_STATUS_GOOD)
@@ -1261,11 +1278,17 @@ static SANE_Status send_gamma_table (SnapScan_Scanner *pss, u_char dtc, u_char d
     SANE_Status status = SANE_STATUS_GOOD;
     status = send (pss, dtc, dtcq);
     CHECK_STATUS (status, me, "send");
-    if ((pss->pdev->model == PERFECTION1670) || (pss->pdev->model == PERFECTION2480))
+    switch (pss->pdev->model)
     {
-        /* Epson Perfection 1670 needs an extra invitation */
-        status = send (pss, dtc, dtcq);
-        CHECK_STATUS (status, me, "send");
+        case PERFECTION1270:
+        case PERFECTION1670:
+        case PERFECTION2480:
+            /* Some epson scanners need the gamma table twice */
+            status = send (pss, dtc, dtcq);
+            CHECK_STATUS (status, me, "2nd send");
+            break;
+        default:
+            break;
     }
     return status;
 }
@@ -1567,16 +1590,21 @@ SANE_Status sane_start (SANE_Handle h)
 
     status = download_gamma_tables(pss);
     CHECK_STATUS (status, me, "download_gamma_tables");
-    if ((pss->pdev->model == PERFECTION1670) || (pss->pdev->model == PERFECTION2480))
-    {
-        /* Epson Perfections needs the gamma table twice */
-        status = download_gamma_tables(pss);
-        CHECK_STATUS (status, me, "download_gamma_tables (2nd try)");
-    }
-
+    
     status = download_halftone_matrices(pss);
     CHECK_STATUS (status, me, "download_halftone_matrices");
 
+    if (pss->pdev->model == PERFECTION2480 && pss->val[OPT_QUALITY_CAL].b)
+    {
+        status = calibrate(pss);
+        if (status != SANE_STATUS_GOOD)
+        {
+            DBG (DL_MAJOR_ERROR, "%s: calibration failed.\n", me);
+            release_unit (pss);
+            return status;
+        }
+    }
+
     /* we must measure the data transfer rate between the host and the
        scanner, and the method varies depending on whether there is a
        ring buffer or not. */
@@ -1597,7 +1625,7 @@ SANE_Status sane_start (SANE_Handle h)
          pss->bytes_per_line/pss->ms_per_line);
 
 
-    if(pss->val[OPT_QUALITY_CAL].b)
+    if (pss->pdev->model != PERFECTION2480 && pss->val[OPT_QUALITY_CAL].b)
     {
         status = calibrate(pss);
         if (status != SANE_STATUS_GOOD)
@@ -1837,6 +1865,15 @@ SANE_Status sane_get_select_fd (SANE_Handle h, SANE_Int * fd)
 
 /*
  * $Log: snapscan.c,v $
+ * Revision 1.48  2004/12/09 23:21:48  oliver-guest
+ * Added quality calibration for Epson 2480 (by Simon Munton)
+ *
+ * Revision 1.47  2004/12/01 22:49:14  oliver-guest
+ * Fix for allocation of gamma tables by Simon Munton
+ *
+ * Revision 1.46  2004/12/01 22:12:03  oliver-guest
+ * Added support for Epson 1270
+ *
  * Revision 1.45  2004/10/03 17:34:36  hmg-guest
  * 64 bit platform fixes (bug #300799).
  *
