@@ -1,6 +1,6 @@
 /* sane - Scanner Access Now Easy.
 
-   Copyright (C) 2002 Frank Zago (fzago at austin dot rr dot com)
+   Copyright (C) 2002 Frank Zago (sane at zago dot net)
    
    This file is part of the SANE package.
    
@@ -42,13 +42,13 @@
 */
 
 /*
-   $Id: teco1.c,v 1.12 2002/11/19 05:01:22 fzago Exp $
+   $Id: teco1.c,v 1.15 2005/07/07 11:55:42 fzago-guest Exp $
    Some Relisys scanners AVEC and RELI series
 */
 
 /*--------------------------------------------------------------------------*/
 
-#define BUILD 9			/* 2002/10/09 */
+#define BUILD 10			/* 2004/02/08 */
 #define BACKEND_NAME teco1
 #define TECO_CONFIG_FILE "teco1.conf"
 
@@ -190,6 +190,16 @@ static const struct scanners_supported scanners[] = {
    99
   },
 
+  {6, "TECO VM4540", 
+   TECO_VM4540, 
+   "Relisys", "RELI 4816", 
+   {1, 1600, 1},       /* resolution */ 
+   400, 1600,          /* max x and Y resolution */ 
+   1,                  /* color 1 pass */ 
+   256,                /* number of bytes per gamma color */ 
+   99                  /* number of bytes in a window */ 
+  },
+  
   {6, "TECO VM4542",
    TECO_VM4542,
    "Relisys", "RELI 4830",
@@ -566,16 +576,13 @@ teco_get_inquiry_82 (Teco_Scanner * dev)
 /* SCSI sense handler. Callback for SANE.
  * These scanners never set asc or ascq. */
 static SANE_Status
-teco_sense_handler (int scsi_fd, unsigned char *result, void *arg)
+teco_sense_handler (int __unused__ scsi_fd, unsigned char *result, void __unused__ *arg)
 {
   int sensekey;
   int len;
 
   DBG (DBG_proc, "teco_sense_handler: enter\n");
 
-  arg = arg;			/* silence gcc */
-  scsi_fd = scsi_fd;		/* silence gcc */
-
   sensekey = get_RS_sense_key (result);
   len = 7 + get_RS_additional_length (result);
 
@@ -1461,7 +1468,7 @@ do_cancel (Teco_Scanner * dev)
 /* Sane entry points */
 
 SANE_Status
-sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
+sane_init (SANE_Int * version_code, SANE_Auth_Callback __unused__ authorize)
 {
   FILE *fp;
   char dev_name[PATH_MAX];
@@ -1471,8 +1478,6 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
 
   DBG (DBG_sane_init, "sane_init\n");
 
-  authorize = authorize;	/* silence gcc */
-
   DBG (DBG_error, "This is sane-teco1 version %d.%d-%d\n", V_MAJOR,
        V_MINOR, BUILD);
   DBG (DBG_error, "(C) 2002 by Frank Zago\n");
@@ -1510,15 +1515,13 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
 }
 
 SANE_Status
-sane_get_devices (const SANE_Device *** device_list, SANE_Bool local_only)
+sane_get_devices (const SANE_Device *** device_list, SANE_Bool __unused__ local_only)
 {
   Teco_Scanner *dev;
   int i;
 
   DBG (DBG_proc, "sane_get_devices: enter\n");
 
-  local_only = local_only;	/* silence gcc */
-
   if (devlist)
     free (devlist);
 
@@ -2157,16 +2160,13 @@ sane_read (SANE_Handle handle, SANE_Byte * buf, SANE_Int max_len,
 }
 
 SANE_Status
-sane_set_io_mode (SANE_Handle handle, SANE_Bool non_blocking)
+sane_set_io_mode (SANE_Handle __unused__ handle, SANE_Bool __unused__ non_blocking)
 {
   SANE_Status status;
   Teco_Scanner *dev = handle;
 
   DBG (DBG_proc, "sane_set_io_mode: enter\n");
 
-  handle = handle;		/* silence gcc */
-  non_blocking = non_blocking;	/* silence gcc */
-
   if (dev->scanning == SANE_FALSE)
     {
       return SANE_STATUS_INVAL;
@@ -2187,13 +2187,10 @@ sane_set_io_mode (SANE_Handle handle, SANE_Bool non_blocking)
 }
 
 SANE_Status
-sane_get_select_fd (SANE_Handle handle, SANE_Int * fd)
+sane_get_select_fd (SANE_Handle __unused__ handle, SANE_Int __unused__ *fd)
 {
   DBG (DBG_proc, "sane_get_select_fd: enter\n");
 
-  handle = handle;		/* silence gcc */
-  fd = fd;			/* silence gcc */
-
   DBG (DBG_proc, "sane_get_select_fd: exit\n");
 
   return SANE_STATUS_UNSUPPORTED;
