@@ -1,6 +1,6 @@
 /* sane - Scanner Access Now Easy.
 
-   Copyright (C) 2002 Frank Zago (fzago at austin dot rr dot com)
+   Copyright (C) 2002 Frank Zago (sane at zago dot net)
    
    This file is part of the SANE package.
    
@@ -42,7 +42,7 @@
 */
 
 /*
-   $Id: teco3.c,v 1.5 2004/10/03 17:34:36 hmg-guest Exp $
+   $Id: teco3.c,v 1.7 2005/07/07 11:55:42 fzago-guest Exp $
    VM3552 (and maybe VM4552 and VM6552)
 */
 
@@ -422,16 +422,13 @@ teco_identify_scanner (Teco_Scanner * dev)
 /* SCSI sense handler. Callback for SANE.
  * These scanners never set asc or ascq. */
 static SANE_Status
-teco_sense_handler (int scsi_fd, unsigned char *result, void *arg)
+teco_sense_handler (int __unused__ scsi_fd, unsigned char *result, void __unused__ *arg)
 {
   int sensekey;
   int len;
 
   DBG (DBG_proc, "teco_sense_handler: enter\n");
 
-  arg = arg;			/* silence gcc */
-  scsi_fd = scsi_fd;		/* silence gcc */
-
   sensekey = get_RS_sense_key (result);
   len = 7 + get_RS_additional_length (result);
 
@@ -1493,7 +1490,7 @@ do_cancel (Teco_Scanner * dev)
 /* Sane entry points */
 
 SANE_Status
-sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
+sane_init (SANE_Int * version_code, SANE_Auth_Callback __unused__ authorize)
 {
   FILE *fp;
   char dev_name[PATH_MAX];
@@ -1503,8 +1500,6 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
 
   DBG (DBG_sane_init, "sane_init\n");
 
-  authorize = authorize;	/* silence gcc */
-
   DBG (DBG_error, "This is sane-teco3 version %d.%d-%d\n", V_MAJOR,
        V_MINOR, BUILD);
   DBG (DBG_error, "(C) 2002 by Frank Zago\n");
@@ -1542,15 +1537,13 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
 }
 
 SANE_Status
-sane_get_devices (const SANE_Device *** device_list, SANE_Bool local_only)
+sane_get_devices (const SANE_Device *** device_list, SANE_Bool __unused__ local_only)
 {
   Teco_Scanner *dev;
   int i;
 
   DBG (DBG_proc, "sane_get_devices: enter\n");
 
-  local_only = local_only;	/* silence gcc */
-
   if (devlist)
     free (devlist);
 
@@ -2159,16 +2152,13 @@ sane_read (SANE_Handle handle, SANE_Byte * buf, SANE_Int max_len,
 }
 
 SANE_Status
-sane_set_io_mode (SANE_Handle handle, SANE_Bool non_blocking)
+sane_set_io_mode (SANE_Handle __unused__ handle, SANE_Bool __unused__ non_blocking)
 {
   SANE_Status status;
   Teco_Scanner *dev = handle;
 
   DBG (DBG_proc, "sane_set_io_mode: enter\n");
 
-  handle = handle;		/* silence gcc */
-  non_blocking = non_blocking;	/* silence gcc */
-
   if (dev->scanning == SANE_FALSE)
     {
       return SANE_STATUS_INVAL;
@@ -2189,13 +2179,10 @@ sane_set_io_mode (SANE_Handle handle, SANE_Bool non_blocking)
 }
 
 SANE_Status
-sane_get_select_fd (SANE_Handle handle, SANE_Int * fd)
+sane_get_select_fd (SANE_Handle __unused__ handle, SANE_Int __unused__ * fd)
 {
   DBG (DBG_proc, "sane_get_select_fd: enter\n");
 
-  handle = handle;		/* silence gcc */
-  fd = fd;			/* silence gcc */
-
   DBG (DBG_proc, "sane_get_select_fd: exit\n");
 
   return SANE_STATUS_UNSUPPORTED;
