@@ -1,6 +1,6 @@
 /* sane - Scanner Access Now Easy.
    Copyright (C) 1999 Juergen G. Schimmer
-   Updates and bugfixes (C) 2002, 2003 Henning Meier-Geinitz
+   Updates and bugfixes (C) 2002 - 2004 Henning Meier-Geinitz
 
    This file is part of the SANE package.
 
@@ -43,7 +43,7 @@
    This file implements a SANE backend for v4l-Devices.
 */
 
-#define BUILD 3
+#define BUILD 4
 
 #include "../include/sane/config.h"
 
@@ -93,7 +93,7 @@ static V4L_Scanner *first_handle;
 static char *buffer;
 
 static const SANE_String_Const mode_list[] = {
-  "Grey", "Color",
+  SANE_VALUE_SCAN_MODE_GRAY, SANE_VALUE_SCAN_MODE_COLOR,
   0
 };
 
@@ -178,7 +178,9 @@ attach (const char *devname, V4L_Device ** devp)
   if (!dev->sane.name)
     return SANE_STATUS_NO_MEM;
   dev->sane.vendor = "Noname";
-  dev->sane.model = capability.name;
+  dev->sane.model = strdup (capability.name);
+  if (!dev->sane.model)
+    return SANE_STATUS_NO_MEM;
   dev->sane.type = "virtual device";
 
   ++num_devices;
@@ -466,6 +468,7 @@ sane_exit (void)
     {
       next = dev->next;
       free ((void *) dev->sane.name);
+      free ((void *) dev->sane.model);
       free (dev);
     }
 
@@ -807,7 +810,7 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 	  s->val[option].s = strdup (val);
 	  if (!s->val[option].s)
 	    return SANE_STATUS_NO_MEM;
-	  if (strcmp (s->val[option].s, "Grey") == 0)
+	  if (strcmp (s->val[option].s, SANE_VALUE_SCAN_MODE_GRAY) == 0)
 	    s->pict.palette = VIDEO_PALETTE_GREY;
 	  else
 	    s->pict.palette = VIDEO_PALETTE_RGB24;
@@ -986,7 +989,7 @@ sane_start (SANE_Handle handle)
 	   s->mbuf.size, s->mbuf.frames, s->mbuf.offsets[0]);
       buffer =
 	mmap (0, s->mbuf.size, PROT_READ | PROT_WRITE, MAP_SHARED, s->fd, 0);
-      if ((int) buffer == -1)
+      if (buffer == (void *)-1)
 	{
 	  DBG (1, "sane_start: mmap failed: %s\n", strerror (errno));
 	  return SANE_STATUS_IO_ERROR;
