@@ -1,6 +1,6 @@
 /* sane - Scanner Access Now Easy.
    Copyright (C) 1997 Andreas Beck
-   Copyright (C) 2001 - 2003 Henning Meier-Geinitz
+   Copyright (C) 2001 - 2004 Henning Meier-Geinitz
    Copyright (C) 2003 Julien BLACHE <jb@jblache.org>
        AF-independent + IPv6 code
 
@@ -30,10 +30,18 @@
 
 #include "../include/sane/config.h"
 #include "../include/lalloca.h"
-#include "../include/sys/types.h"
+#include <sys/types.h>
 
 #if defined(HAVE_GETADDRINFO) && defined (HAVE_GETNAMEINFO)
 # define SANED_USES_AF_INDEP
+# ifdef HAS_SS_FAMILY
+#  define SS_FAMILY(ss) ss.ss_family
+# elif defined(HAS___SS_FAMILY)
+#  define SS_FAMILY(ss) ss.__ss_family
+# else /* fallback to the old, IPv4-only code */
+#  undef SANED_USES_AF_INDEP
+#  undef ENABLE_IPV6
+# endif
 #else
 # undef ENABLE_IPV6
 #endif /* HAVE_GETADDRINFO && HAVE_GETNAMEINFO && HAVE_POLL */
@@ -146,15 +154,15 @@ poll (struct pollfd *ufds, unsigned int nfds, int timeout)
 
 #ifdef ENABLE_IPV6
 # define SANE_IN6_IS_ADDR_LOOPBACK(a) \
-        (((const uint32_t *) (a))[0] == 0                                   \
-         && ((const uint32_t *) (a))[1] == 0                                \
-         && ((const uint32_t *) (a))[2] == 0                                \
-         && ((const uint32_t *) (a))[3] == htonl (1)) 
+        (((const u_int32_t *) (a))[0] == 0                                   \
+         && ((const u_int32_t *) (a))[1] == 0                                \
+         && ((const u_int32_t *) (a))[2] == 0                                \
+         && ((const u_int32_t *) (a))[3] == htonl (1)) 
 
 #define SANE_IN6_IS_ADDR_V4MAPPED(a) \
-((((const uint32_t *) (a))[0] == 0)                                 \
- && (((const uint32_t *) (a))[1] == 0)                              \
- && (((const uint32_t *) (a))[2] == htonl (0xffff))) 
+((((const u_int32_t *) (a))[0] == 0)                                 \
+ && (((const u_int32_t *) (a))[1] == 0)                              \
+ && (((const u_int32_t *) (a))[2] == htonl (0xffff))) 
 #endif /* ENABLE_IPV6 */
 
 #ifndef MAXHOSTNAMELEN
@@ -254,15 +262,15 @@ reset_watchdog (void)
 
 static void
 auth_callback (SANE_String_Const res,
-	       SANE_Char username[SANE_MAX_USERNAME_LEN],
-	       SANE_Char password[SANE_MAX_PASSWORD_LEN])
+	       SANE_Char *username,
+	       SANE_Char *password)
 {
   SANE_Net_Procedure_Number procnum;
   SANE_Authorization_Req req;
   SANE_Word word, ack = 0;
 
-  memset (username, 0, sizeof (username));
-  memset (password, 0, sizeof (password));
+  memset (username, 0, SANE_MAX_USERNAME_LEN);
+  memset (password, 0, SANE_MAX_PASSWORD_LEN);
 
   if (!can_authorize)
     {
@@ -438,7 +446,7 @@ get_free_handle (void)
 static void
 close_handle (int h)
 {
-  if (handle[h].inuse)
+  if (h >= 0 && handle[h].inuse)
     sane_close (handle[h].handle);
   handle[h].inuse = 0;
 }
@@ -459,6 +467,194 @@ decode_handle (Wire * w, const char *op)
   return h;
 }
 
+
+
+/* Convert a number of bits to an 8-bit bitmask */
+static unsigned int cidrtomask[9] = { 0x00, 0x80, 0xC0, 0xE0, 0xF0,
+				      0xF8, 0xFC, 0xFE, 0xFF };
+
+#ifdef SANED_USES_AF_INDEP
+static SANE_Bool
+check_v4_in_range (struct sockaddr_in *sin, char *base_ip, char *netmask)
+{
+  int cidr;
+  int i, err;
+  char *end;
+  u_int32_t mask; 
+  struct sockaddr_in *base;
+  struct addrinfo hints;
+  struct addrinfo *res;
+  SANE_Bool ret = SANE_FALSE;
+
+  cidr = -1;
+  cidr = strtol (netmask, &end, 10);
+  
+  /* Sanity check on the cidr value */
+  if ((cidr < 0) || (cidr > 32) || (end == netmask))
+    {
+      DBG (DBG_ERR, "check_v4_in_range: invalid CIDR value (%s) !\n", netmask);
+      return SANE_FALSE;
+    }
+
+  mask = 0;
+  cidr -= 8;
+
+  /* Build a bitmask out of the CIDR value */  
+  for (i = 3; cidr >= 0; i--)
+    {
+      mask |= (0xff << (8 * i));
+      cidr -= 8;
+    }
+  
+  if (cidr < 0)
+    mask |= (cidrtomask[cidr + 8] << (8 * i));
+
+  mask = htonl (mask);
+
+  /* get a sockaddr_in representing the base IP address */
+  memset (&hints, 0, sizeof (struct addrinfo));
+  hints.ai_flags = AI_NUMERICHOST;
+  hints.ai_family = PF_INET;
+  
+  err = getaddrinfo (base_ip, NULL, &hints, &res);
+  if (err)
+    {
+      DBG (DBG_DBG, "check_v4_in_range: getaddrinfo() failed: %s\n", gai_strerror (err));
+      return SANE_FALSE;
+    }
+
+  base = (struct sockaddr_in *) res->ai_addr;
+
+  /*
+   * Check that the address belongs to the specified subnet, using the bitmask.
+   * The address is represented by a 32bit integer.
+   */
+  if ((base->sin_addr.s_addr & mask) == (sin->sin_addr.s_addr & mask))
+    ret = SANE_TRUE;
+  
+  freeaddrinfo (res);
+  
+  return ret;
+}
+
+
+# ifdef ENABLE_IPV6
+
+static SANE_Bool
+check_v6_in_range (struct sockaddr_in6 *sin6, char *base_ip, char *netmask)
+{
+  int cidr;
+  int i, err;
+  unsigned int mask[16];
+  char *end;
+  struct sockaddr_in6 *base;
+  struct addrinfo hints;
+  struct addrinfo *res;
+  SANE_Bool ret = SANE_TRUE;
+
+  cidr = -1;
+  cidr = strtol (netmask, &end, 10);
+
+  /* Sanity check on the cidr value */
+  if ((cidr < 0) || (cidr > 128) || (end == netmask))
+    {
+      DBG (DBG_ERR, "check_v6_in_range: invalid CIDR value (%s) !\n", netmask);
+      return SANE_FALSE;
+    }
+
+  memset (mask, 0, (16 * sizeof (unsigned int)));
+  cidr -= 8;
+  
+  /* Build a bitmask out of the CIDR value */
+  for (i = 0; cidr >= 0; i++)
+    {
+      mask[i] = 0xff;
+      cidr -= 8;
+    }
+  
+  if (cidr < 0)
+    mask[i] = cidrtomask[cidr + 8];
+  
+  /* get a sockaddr_in6 representing the base IP address */
+  memset (&hints, 0, sizeof (struct addrinfo));
+  hints.ai_flags = AI_NUMERICHOST;
+  hints.ai_family = PF_INET6;
+
+  err = getaddrinfo (base_ip, NULL, &hints, &res);
+  if (err)
+    {
+      DBG (DBG_DBG, "check_v6_in_range: getaddrinfo() failed: %s\n", gai_strerror (err));
+      return SANE_FALSE;
+    }
+
+  base = (struct sockaddr_in6 *) res->ai_addr;
+
+  /*
+   * Check that the address belongs to the specified subnet.
+   * The address is reprensented by an array of 16 8bit integers.
+   */
+  for (i = 0; i < 16; i++)
+    {
+      if ((base->sin6_addr.s6_addr[i] & mask[i]) != (sin6->sin6_addr.s6_addr[i] & mask[i]))
+	{
+	  ret = SANE_FALSE;
+	  break;
+	}
+    }
+  
+  freeaddrinfo (res);
+  
+  return ret;
+}
+# endif /* ENABLE_IPV6 */
+#else /* !SANED_USES_AF_INDEP */
+static SANE_Bool
+check_v4_in_range (struct in_addr *inaddr, struct in_addr *base, char *netmask)
+{
+  int cidr;
+  int i;
+  char *end;
+  u_int32_t mask; 
+  SANE_Bool ret = SANE_FALSE;
+
+  cidr = -1;
+  cidr = strtol (netmask, &end, 10);
+  
+  /* sanity check on the cidr value */
+  if ((cidr < 0) || (cidr > 32) || (end == netmask))
+    {
+      DBG (DBG_ERR, "check_v4_in_range: invalid CIDR value (%s) !\n", netmask);
+      return SANE_FALSE;
+    }
+
+  mask = 0;
+  cidr -= 8;
+  
+  /* Build a bitmask out of the CIDR value */
+  for (i = 3; cidr >= 0; i--)
+    {
+      mask |= (0xff << (8 * i));
+      cidr -= 8;
+    }
+  
+  if (cidr < 0)
+    mask |= (cidrtomask[cidr + 8] << (8 * i));
+
+  mask = htonl (mask);
+
+  /*
+   * Check that the address belongs to the specified subnet, using the bitmask.
+   * The address is represented by a 32bit integer.
+   */
+  if ((base->s_addr & mask) == (inaddr->s_addr & mask))
+    ret = SANE_TRUE;
+  
+  return ret;
+}
+#endif /* SANED_USES_AF_INDEP */
+
+
+
 /* Access control */
 #ifdef SANED_USES_AF_INDEP
 static SANE_Status
@@ -475,16 +671,16 @@ check_host (int fd)
   int err;
   char text_addr[64];
 #ifdef ENABLE_IPV6
+  SANE_Bool IPv4map = SANE_FALSE;
   char *remote_ipv4 = NULL; /* in case we have an IPv4-mapped address (eg ::ffff:127.0.0.1) */
+  char *tmp;
   struct addrinfo *remote_ipv4_addr = NULL;
 #endif /* ENABLE_IPV6 */
-  char config_line[1024];
+  char config_line_buf[1024];
+  char *config_line;
+  char *netmask;
   char hostname[MAXHOSTNAMELEN];
 
-#ifdef ENABLE_IPV6  
-  SANE_Bool IPv4map = SANE_FALSE;
-#endif /* ENABLE_IPV6 */
-
   int len;
   FILE *fp;
 
@@ -556,7 +752,7 @@ check_host (int fd)
 
   sin = (struct sockaddr_in *) &remote_address;
 
-  switch (remote_address.ss_family)
+  switch (SS_FAMILY(remote_address))
     {
       case AF_INET:
 	if (IN_LOOPBACK (ntohl (sin->sin_addr.s_addr)))
@@ -624,7 +820,7 @@ check_host (int fd)
 		strncpy (text_addr, "[error]", 8);
 
 #ifdef ENABLE_IPV6	  
-	  if ((strcmp (text_addr, remote_ip) == 0) ||
+	  if ((strcasecmp (text_addr, remote_ip) == 0) ||
 	      ((IPv4map == SANE_TRUE) && (strcmp (text_addr, remote_ipv4) == 0)))
 #else
 	  if (strcmp (text_addr, remote_ip) == 0)
@@ -633,12 +829,14 @@ check_host (int fd)
 	      DBG (DBG_MSG, "check_host: remote host has same addr as local: access granted\n");
 	      
 	      freeaddrinfo (res);
-	      
+	      res = NULL;
+
 	      return SANE_STATUS_GOOD;
 	    }
 	}
       
       freeaddrinfo (res);
+      res = NULL;
       
       DBG (DBG_DBG, 
 	   "check_host: remote host doesn't have same addr as local\n");
@@ -662,9 +860,10 @@ check_host (int fd)
 	  continue;
 	}
       
-      while (!access_ok && sanei_config_read (config_line, 
-					      sizeof (config_line), fp))
+      while (!access_ok && sanei_config_read (config_line_buf, 
+					      sizeof (config_line_buf), fp))
 	{
+	  config_line = config_line_buf; /* from now on, use a pointer */
 	  DBG (DBG_DBG, "check_host: config file line: `%s'\n", config_line);
 	  if (config_line[0] == '#')	/* ignore line comments */
 	    continue;
@@ -672,7 +871,34 @@ check_host (int fd)
 	  
 	  if (!len)
 	    continue;		/* ignore empty lines */
-	  
+
+	  /* look for a subnet specification */
+	  netmask = strchr (config_line, '/');
+	  if (netmask != NULL)
+	    {
+	      *netmask = '\0';
+	      netmask++;
+	      DBG (DBG_DBG, "check_host: subnet with base IP = %s, CIDR netmask = %s\n",
+		   config_line, netmask);
+	    }
+
+#ifdef ENABLE_IPV6
+	  /* IPv6 addresses are enclosed in [] */
+	  if (*config_line == '[')
+	    {
+	      config_line++;
+	      tmp = strchr (config_line, ']');
+	      if (tmp == NULL)
+		{
+		  DBG (DBG_ERR,
+		       "check_host: malformed IPv6 address in config file, skipping: [%s\n",
+		       config_line);
+		  continue;
+		}
+	      *tmp = '\0';
+	    }
+#endif /* ENABLE_IPV6 */
+
 	  if (strcmp (config_line, "+") == 0)
 	    {
 	      access_ok = 1;
@@ -680,7 +906,7 @@ check_host (int fd)
 		   "check_host: access granted from any host (`+')\n");
 	    }
 	  /* compare remote_ip (remote IP address) to the config_line */
-	  else if (strcmp (config_line, remote_ip) == 0)
+	  else if (strcasecmp (config_line, remote_ip) == 0)
 	    {
 	      access_ok = 1;
 	      DBG (DBG_DBG,
@@ -693,6 +919,72 @@ check_host (int fd)
 	      DBG (DBG_DBG,
 		   "check_host: access granted from IP address %s (IPv4-mapped)\n", remote_ip);
 	    }
+	  /* handle IP ranges, take care of the IPv4map stuff */
+	  else if (netmask != NULL)
+	    {
+	      if (strchr (config_line, ':') != NULL) /* is a v6 address */
+		{
+		  if (SS_FAMILY(remote_address) == AF_INET6)
+		    {
+		      if (check_v6_in_range (sin6, config_line, netmask))
+			{
+			  access_ok = 1;
+			  DBG (DBG_DBG, "check_host: access granted from IP address %s (in subnet [%s]/%s)\n",
+			       remote_ip, config_line, netmask);
+			}
+		    }
+		}
+	      else /* is a v4 address */
+		{
+		  if (IPv4map == SANE_TRUE)
+		    {
+		      /* get a sockaddr_in representing the v4-mapped IP address */
+		      memset (&hints, 0, sizeof (struct addrinfo));
+		      hints.ai_flags = AI_NUMERICHOST;
+		      hints.ai_family = PF_INET;
+		      
+		      err = getaddrinfo (remote_ipv4, NULL, &hints, &res);
+		      if (err)
+			DBG (DBG_DBG, "check_host: getaddrinfo() failed: %s\n", gai_strerror (err));
+		      else
+			sin = (struct sockaddr_in *)res->ai_addr;
+		    }
+
+		  if ((SS_FAMILY(remote_address) == AF_INET) ||
+		      (IPv4map == SANE_TRUE))
+		    {
+		      
+		      if (check_v4_in_range (sin, config_line, netmask))
+			{
+			  DBG (DBG_DBG, "check_host: access granted from IP address %s (in subnet %s/%s)\n",
+			       ((IPv4map == SANE_TRUE) ? remote_ipv4 : remote_ip), config_line, netmask);
+			  access_ok = 1;
+			}
+		      else
+			{
+			  /* restore the old sin pointer */
+			  sin = (struct sockaddr_in *)&remote_address;
+			}
+		      
+		      if (res != NULL)
+			{
+			  freeaddrinfo (res);
+			  res = NULL;
+			}
+		    }
+		}
+	    }
+#else /* !ENABLE_IPV6 */
+	  /* handle IP ranges */
+	  else if (netmask != NULL)
+	    {
+	      if (check_v4_in_range (sin, config_line, netmask))
+		{
+		  access_ok = 1;
+		  DBG (DBG_DBG, "check_host: access granted from IP address %s (in subnet %s/%s)\n",
+		       remote_ip, config_line, netmask);
+		}
+	    }
 #endif /* ENABLE_IPV6 */
 	  else
 	    {
@@ -728,7 +1020,7 @@ check_host (int fd)
 			   text_addr); 
 		      
 #ifdef ENABLE_IPV6			  
-		      if ((strcmp (text_addr, remote_ip) == 0) ||
+		      if ((strcasecmp (text_addr, remote_ip) == 0) ||
 			  ((IPv4map == SANE_TRUE) && (strcmp (text_addr, remote_ipv4) == 0)))
 #else
 		      if (strcmp (text_addr, remote_ip) == 0)
@@ -739,6 +1031,7 @@ check_host (int fd)
 			break;
 		    }
 		  freeaddrinfo (res);
+		  res = NULL;
 		}
 	    }
 	}
@@ -760,7 +1053,9 @@ check_host (int fd)
   int j, access_ok = 0;
   struct hostent *he;
   char text_addr[64];
-  char config_line[1024];
+  char config_line_buf[1024];
+  char *config_line;
+  char *netmask;
   char hostname[MAXHOSTNAMELEN];
   char *r_hostname;
   static struct in_addr config_line_address;
@@ -854,9 +1149,10 @@ check_host (int fd)
 	  continue;
 	}
       
-      while (!access_ok && sanei_config_read (config_line, 
-					      sizeof (config_line), fp))
+      while (!access_ok && sanei_config_read (config_line_buf, 
+					      sizeof (config_line_buf), fp))
 	{
+	  config_line = config_line_buf; /* from now on, use a pointer */
 	  DBG (DBG_DBG, "check_host: config file line: `%s'\n", config_line);
 	  if (config_line[0] == '#')	/* ignore line comments */
 	    continue;
@@ -865,6 +1161,16 @@ check_host (int fd)
 	  if (!len)
 	    continue;		/* ignore empty lines */
 	  
+	  /* look for a subnet specification */
+	  netmask = strchr (config_line, '/');
+	  if (netmask != NULL)
+	    {
+	      *netmask = '\0';
+	      netmask++;
+	      DBG (DBG_DBG, "check_host: subnet with base IP = %s, CIDR netmask = %s\n",
+		   config_line, netmask);
+	    }
+
 	  if (strcmp (config_line, "+") == 0)
 	    {
 	      access_ok = 1;
@@ -878,6 +1184,15 @@ check_host (int fd)
 		  if (memcmp (&remote_address.s_addr, 
 			      &config_line_address.s_addr, 4) == 0)
 		    access_ok = 1;
+		  else if (netmask != NULL)
+		    {
+		      if (check_v4_in_range (&remote_address, &config_line_address, netmask))
+			{
+			  access_ok = 1;
+			  DBG (DBG_DBG, "check_host: access granted from IP address %s (in subnet %s/%s)\n",
+			       remote_ip, config_line, netmask);
+			}
+		    }
 		}
 	      else
 		{
@@ -1022,7 +1337,7 @@ start_scan (Wire * w, int h, SANE_Start_Reply * reply)
       return -1;
     }
 
-  fd = socket (ss.ss_family, SOCK_STREAM, 0);
+  fd = socket (SS_FAMILY(ss), SOCK_STREAM, 0);
   if (fd < 0)
     {
       DBG (DBG_ERR, "start_scan: failed to obtain data socket (%s)\n",
@@ -1031,7 +1346,7 @@ start_scan (Wire * w, int h, SANE_Start_Reply * reply)
       return -1;
     }
 
-  switch (ss.ss_family)
+  switch (SS_FAMILY(ss))
     {
       case AF_INET:
 	sin = (struct sockaddr_in *) &ss;
@@ -1071,7 +1386,7 @@ start_scan (Wire * w, int h, SANE_Start_Reply * reply)
       return -1;
     }
 
-  switch (ss.ss_family)
+  switch (SS_FAMILY(ss))
     {
       case AF_INET:
 	sin = (struct sockaddr_in *) &ss;
@@ -1400,6 +1715,14 @@ process_request (Wire * w)
 	    return;
 	  }
 
+	if (!name)
+	  {
+	    DBG (DBG_ERR, "process_request: (open) device_name == NULL\n");
+	    reply.status = SANE_STATUS_INVAL;
+	    sanei_w_reply (w, (WireCodecFunc) sanei_w_open_reply, &reply);
+	    return;
+	  }
+
 	can_authorize = 1;
 
 	resource = strdup (name);
@@ -1453,6 +1776,8 @@ process_request (Wire * w)
 	    free (resource);
 	    memset (&reply, 0, sizeof (reply));	/* avoid leaking bits */
 	    reply.status = sane_open (name, &be_handle);
+	    DBG (DBG_MSG, "process_request: sane_open returned: %s\n", 
+		 sane_strstatus (reply.status));
 	  }
 
 	if (reply.status == SANE_STATUS_GOOD)
@@ -1685,8 +2010,11 @@ process_request (Wire * w)
 	SANE_Word ack = 0;
 
 	h = decode_handle (w, "cancel");
-	sane_cancel (handle[h].handle);
-	handle[h].docancel = 1;
+	if (h >= 0)
+	  {
+	    sane_cancel (handle[h].handle);
+	    handle[h].docancel = 1;
+	  }
 	sanei_w_reply (w, (WireCodecFunc) sanei_w_word, &ack);
       }
       break;
@@ -1754,7 +2082,7 @@ main (int argc, char *argv[])
       DBG (DBG_WARN, "main: starting debug mode (level %d)\n", debug);
 
       DBG (DBG_DBG, 
-	   "main: trying to get port for service `sane' (getaddrinfo)\n");
+	   "main: trying to get port for service `sane-port' (getaddrinfo)\n");
 
       memset (&hints, 0, sizeof (struct addrinfo));
 
@@ -1762,11 +2090,11 @@ main (int argc, char *argv[])
       hints.ai_flags = AI_PASSIVE;
       hints.ai_socktype = SOCK_STREAM;
 
-      err = getaddrinfo (NULL, "sane", &hints, &res);
+      err = getaddrinfo (NULL, "sane-port", &hints, &res);
       if (err)
 	{
-	  DBG (DBG_WARN, "main: \"sane\" service unknown on your host; you should add\n");
-	  DBG (DBG_WARN, "main:      sane 6566/tcp saned # SANE network scanner daemon\n");
+	  DBG (DBG_WARN, "main: \"sane-port\" service unknown on your host; you should add\n");
+	  DBG (DBG_WARN, "main:      sane-port 6566/tcp saned # SANE network scanner daemon\n");
 	  DBG (DBG_WARN, "main: to your /etc/services file (or equivalent). Proceeding anyway.\n");
 	  err = getaddrinfo (NULL, "6566", &hints, &res);
 	  if (err)
@@ -2006,7 +2334,7 @@ main (int argc, char *argv[])
 
       DBG (DBG_DBG, 
 	   "main: trying to get port for service `sane' (getservbyname)\n");
-      serv = getservbyname ("sane", "tcp");
+      serv = getservbyname ("sane-port", "tcp");
       if (serv)
 	{
 	  port = serv->s_port;
