@@ -1,5 +1,5 @@
 /* sane - Scanner Access Now Easy.
-   Copyright (C) 2003 Rene Rebe (sanei_read_int)
+   Copyright (C) 2003, 2005 Rene Rebe (sanei_read_int,sanei_set_timeout)
    Copyright (C) 2001, 2002 Henning Meier-Geinitz
    This file is part of the SANE package.
 
@@ -16,6 +16,27 @@
    You should have received a copy of the GNU General Public License
    along with sane; see the file COPYING.  If not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+
+   As a special exception, the authors of SANE give permission for
+   additional uses of the libraries contained in this release of SANE.
+
+   The exception is that, if you link a SANE library with other files
+   to produce an executable, this does not by itself cause the
+   resulting executable to be covered by the GNU General Public
+   License.  Your use of that executable is in no way restricted on
+   account of linking the SANE library code into it.
+
+   This exception does not, however, invalidate any other reasons why
+   the executable file might be covered by the GNU General Public
+   License.
+
+   If you submit changes to SANE to the maintainers to be included in
+   a subsequent release, you agree by submitting the changes that
+   those changes may be distributed with this exception intact.
+
+   If you write modifications of your own for SANE, it is your choice
+   whether to permit this exception to apply to your modifications.
+   If you do not wish that, delete this exception notice.
 */
 
 /** @file sanei_usb.h
@@ -33,7 +54,7 @@
  * are not file descriptors.
  *
  * @sa sanei_lm983x.h, sanei_pa4s2.h, sanei_pio.h, sanei_scsi.h, and <a 
- * href="http://www.mostang.com/sane/man/sane-usb.5.html">man sane-usb(5)</a>
+ * href="http://www.sane-project.org/man/sane-usb.5.html">man sane-usb(5)</a>
  * for user-oriented documentation
  */
 
@@ -43,6 +64,8 @@
 #include "../include/sane/config.h"
 #include "../include/sane/sane.h"
 
+#include <stdlib.h> /* for size_t */
+
 /* USB spec defines */
 #ifndef USB_CLASS_PER_INTERFACE
 /* Also defined in libusb */
@@ -135,7 +158,7 @@
 
 /** @name USB directions */
 /* @{ */
-#define USB_DIR_OUT                     0
+#define USB_DIR_OUT                     0x00
 #define USB_DIR_IN                      0x80
 /* @} */
 
@@ -206,6 +229,13 @@ extern SANE_Status sanei_usb_open (SANE_String_Const devname, SANE_Int * dn);
  */
 extern void sanei_usb_close (SANE_Int dn);
 
+/** Set the libusb timeout for bulk and interrupt reads.
+ * 
+ * @param timeout the new timeout in ms
+ */
+#define HAVE_SANEI_USB_SET_TIMEOUT
+extern void sanei_usb_set_timeout (SANE_Int timeout);
+
 /** Initiate a bulk transfer read.
  *
  * Read up to size bytes from the device to buffer. After the read, size
@@ -287,6 +317,7 @@ sanei_usb_control_msg (SANE_Int dn, SANE_Int rtype, SANE_Int req,
  * - SANE_STATUS_INVAL - on every other error
  *
  */
+
 extern SANE_Status
 sanei_usb_read_int (SANE_Int dn, SANE_Byte * buffer, size_t * size);
 
@@ -310,4 +341,77 @@ extern void
 sanei_usb_attach_matching_devices (const char *name,
 				   SANE_Status (*attach) (const char *dev));
 
+/** Initiate set configuration.
+ *
+ * Change set configuration
+ *
+ * @param dn device number
+ * @param configuration, configuration nummber
+ *
+ * @return 
+ * - SANE_STATUS_GOOD - on succes
+ * - SANE_STATUS_EOF - if zero bytes have been read
+ * - SANE_STATUS_IO_ERROR - if an error occured during the read
+ * - SANE_STATUS_INVAL - on every other error
+ *
+ */
+
+extern SANE_Status
+sanei_usb_set_configuration (SANE_Int dn, SANE_Int configuration);
+
+/** Initiate claim interface.
+ *
+ * Change claim interface
+ *
+ * @param dn device number
+ * @param interface, interface nummber
+ *
+ * @return 
+ * - SANE_STATUS_GOOD - on succes
+ * - SANE_STATUS_EOF - if zero bytes have been read
+ * - SANE_STATUS_IO_ERROR - if an error occured during the read
+ * - SANE_STATUS_INVAL - on every other error
+ *
+ */
+
+extern SANE_Status
+sanei_usb_claim_interface (SANE_Int dn, SANE_Int interface);
+
+/** Initiate release interface.
+ *
+ * Change release interface
+ *
+ * @param dn device number
+ * @param interface, interface nummber
+ *
+ * @return 
+ * - SANE_STATUS_GOOD - on succes
+ * - SANE_STATUS_EOF - if zero bytes have been read
+ * - SANE_STATUS_IO_ERROR - if an error occured during the read
+ * - SANE_STATUS_INVAL - on every other error
+ *
+ */
+
+extern SANE_Status
+sanei_usb_release_interface (SANE_Int dn, SANE_Int interface);
+
+/** Initiate a set altinterface.
+ *
+ * Change set alternate
+ *
+ * @param dn device number
+ * @param alternate, alternate nummber
+ *
+ * @return 
+ * - SANE_STATUS_GOOD - on succes
+ * - SANE_STATUS_EOF - if zero bytes have been read
+ * - SANE_STATUS_IO_ERROR - if an error occured during the read
+ * - SANE_STATUS_INVAL - on every other error
+ *
+ */
+
+extern SANE_Status
+sanei_usb_set_altinterface (SANE_Int dn, SANE_Int alternate);
+
+/*------------------------------------------------------*/
 #endif /* sanei_usb_h */
